/*******************************************************************************
********************************************************************************

   COPYRIGHT  2004, Advanced Illumination, Inc. 

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Advanced Illumination.

   MPDH8USB.cpp   Source code for the Low Level Command Interface for the 
                  S1000 Strobe Controller.

   About:         This module contains the implementation of the USB Host
                  Interface Toolkit functions in the AICONS01 project.

   Target:        Win32 Compatible

   Written for:   Advanced Illumination, Inc.
                  24 Peavine Drive
                  Rochester, VT 05767
                  http://advancedillumination.com

   Contacts:      John Thrailkill
                  Phone:  (802) 767-3830
                  FAX:    (802) 767-3831
                  jthrailkill@advill.com

   Written by:    Rob Macklin
                  Microprocessor Designs, Inc.
                  65 Longmeadow Drive
                  PO Box 160
                  Shelburne, VT 05482
                  http://www.updesigns.com

   $Revision: 4 $

--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/Pulsar320/pc_src/MPDH8USB_TCP/MPDH8USB_TCP.cpp $
 * 4	 10/14/11 JST
   Fixed comments for connect() prototype
 * 3     8/22/08 3:06p Jpiet
 * Made additions for repeat mode
 * 
 * 2     8/15/08 4:25p Jpiet
 * 
 * 1     8/15/08 12:24p Jpiet
 * 
 * 14    5/21/08 9:32a Rmack
 * Changed interface to 32-bit pulse width values.
 * 
 * 10    1/25/06 11:54a Rmack
 * Defined new structure to pass lighthead data to DLL so H8 byte order is
 * handled internally.
 * 
 * 9     1/25/06 9:56a Rmack
 * Set reserved data of lighthead structure to zero.
 * 
 * 8     1/25/06 9:49a Rmack
 * Fixed problem with double pointer when computing lighthead information
 * CRC.
 * 
 * 7     1/24/06 4:05p Rmack
 * Added functions to get/set lighthead data structure.
 * 
 * 6     5/02/05 8:25a Rmack
 * Changed timeouts for commands that change supply voltages.
 * 
 * 5     2/23/05 4:02p Rmack
 * Add interfaces to new target commands.
 * 
 * 4     10/15/04 8:36a Rmack
 * Channel specific supply voltage functions.
 * 
 * 3     9/28/04 10:39a Rmack
 * Added commands for direct control of timer outputs.
 * 
 * 2     5/13/04 2:17p Rmack
 * First port to AI hardware.
 * 
 * 1     3/30/04 3:25p Rmack
 * 
********************************************************************************
*******************************************************************************/
#include "stdafx.h"
#include <time.h>
#include <stdlib.h>
#include <stdio.h>
#include <commdlg.h>
#include "compiler.h"
#include "pc_error.h"
#include "usbdefs.h"
#include "tarmap.h"
extern "C" {
#include "hidsdi.h"
#include "setupapi.h"
}
#include "MPDH8USB_TCP.h"

#include <direct.h>
#include <wchar.h>
#include "tk_crc.h"
#include "tk_reg.h"
#include "tk_time.h"
#include "tk_usb.h"
#include "tk_tcp.h"
#include "hostcmds.h"

/* The following include files are shared with the firmware.  The pragma is used
to ensure the byte alignment of structures matches the H8.  */
#pragma pack(2)
#include "ap_header.h"
#pragma pack()



/* Registry entry definitions */
#define TKCMDSET_REG_SUBKEY              "Software\\Advanced Illumination\\Pulsar 320\\Command Set Toolkit"
#define TKCMDSET_REG_ECODES_FILENAME     "ErrorCodesFile"
#define TKCMDSET_REG_DEFAULT_ECODES_FILENAME    ".\\ecodes.ini"
#define TKCMDSET_INI_ECODE_NAME          "Name"
#define TKCMDSET_INI_ERRCODE_TEXT        "Text"

/* Command / Response processing definitions */
#define TKCMDSET_MUTEX_TIMEOUT           2000
#define TKCMDSET_PROGRESS_MESSAGE_TIME   100

#define TKCMDSET_GET_CMD_LEN_M(msg_id, lenptr) \
        tkcmdset_get_msg_data_len(msg_id, TRUE, lenptr)
#define TKCMDSET_GET_RSP_LEN_M(msg_id, lenptr) \
        tkcmdset_get_msg_data_len(msg_id, FALSE, lenptr)

/* 
   This pragma is required to force the variables into the same data segment
   so that each instance of the DLL sees the same variable.  Other global
   variables declared outside this pragma block get new copies created
   for every instance of the DLL.
*/
#pragma data_seg(".DLLDAT")
UINT32_T dll_instance_count = 0;
char DLLPath[MAX_PATH] = {""};
#pragma data_seg()
#pragma comment(linker,"/SECTION:.DLLDAT,RWS")
UINT32_T	command_fail_cnt=0;

typedef struct
{
   UINT8_T short_cmd_id;
   UINT16_T short_cmd_len;
   UINT8_T long_cmd_id;
   UINT16_T long_cmd_len;
   UINT8_T short_rsp_id;
   UINT16_T short_rsp_len;
   UINT8_T long_rsp_id;
   UINT16_T long_rsp_len;
   UINT8_T feature_id;
   UINT16_T feature_len;
}  TKCMDSET_USB_DATA_STRUCT;

typedef enum 
{
   IFACE_USB,
   IFACE_TCP,
}TKCMDSET_IFACE_E;

typedef struct
{
   UINT8_T msg_id;
   UINT8_T cmd_len;
   UINT8_T rsp_len;
} TKCMDSET_MSG_INFO_STRUCT_T;

static TKCMDSET_MSG_INFO_STRUCT_T tkcmdset_msg_info[] =
{
   {HOSTCMD_ECHO_CMD                      ,HOSTCMD_ECHO_CMD_LEN                     ,HOSTCMD_ECHO_RSP_LEN},                 
   {HOSTCMD_VERSION_CMD                   ,HOSTCMD_VERSION_CMD_LEN                  ,HOSTCMD_VERSION_RSP_LEN},              
   {HOSTCMD_STATUS_CMD                    ,HOSTCMD_STATUS_CMD_LEN                   ,HOSTCMD_STATUS_RSP_LEN},               
   {HOSTCMD_DIAGS_CMD                     ,HOSTCMD_DIAGS_CMD_LEN                    ,HOSTCMD_DIAGS_RSP_LEN},                
   {HOSTCMD_CALIBRATE_CMD                 ,HOSTCMD_CALIBRATE_CMD_LEN                ,HOSTCMD_CALIBRATE_RSP_LEN},            
   {HOSTCMD_SPECIAL_CMD                   ,HOSTCMD_SPECIAL_CMD_LEN                  ,HOSTCMD_SPECIAL_RSP_LEN},              
   {HOSTCMD_SETANOUT_CMD                  ,HOSTCMD_SETANOUT_CMD_LEN                 ,HOSTCMD_SETANOUT_RSP_LEN},             
   {HOSTCMD_GETANOUT_CMD                  ,HOSTCMD_GETANOUT_CMD_LEN                 ,HOSTCMD_GETANOUT_RSP_LEN},             
   {HOSTCMD_GETANIN_CMD                   ,HOSTCMD_GETANIN_CMD_LEN                  ,HOSTCMD_GETANIN_RSP_LEN},              
   {HOSTCMD_GETVOLTS_CMD                  ,HOSTCMD_GETVOLTS_CMD_LEN                 ,HOSTCMD_GETVOLTS_RSP_LEN},             
   {HOSTCMD_SETDIGOUT_CMD                 ,HOSTCMD_SETDIGOUT_CMD_LEN                ,HOSTCMD_SETDIGOUT_RSP_LEN},            
   {HOSTCMD_GETDIGOUT_CMD                 ,HOSTCMD_GETDIGOUT_CMD_LEN                ,HOSTCMD_GETDIGOUT_RSP_LEN},            
   {HOSTCMD_GETDIGIN_CMD                  ,HOSTCMD_GETDIGIN_CMD_LEN                 ,HOSTCMD_GETDIGIN_RSP_LEN},             
   {HOSTCMD_WRITEMEM_CMD                  ,HOSTCMD_WRITEMEM_CMD_LEN                 ,HOSTCMD_WRITEMEM_RSP_LEN},             
   {HOSTCMD_READMEM_CMD                   ,HOSTCMD_READMEM_CMD_LEN                  ,HOSTCMD_READMEM_RSP_LEN},              
   {HOSTCMD_WRITEMEMBLK_CMD               ,HOSTCMD_WRITEMEMBLK_CMD_LEN              ,HOSTCMD_WRITEMEMBLK_RSP_LEN},          
   {HOSTCMD_READMEMBLK_CMD                ,HOSTCMD_READMEMBLK_CMD_LEN               ,HOSTCMD_READMEMBLK_RSP_LEN},           
   {HOSTCMD_WRITEEEMEMBLK_CMD             ,HOSTCMD_WRITEEEMEMBLK_CMD_LEN            ,HOSTCMD_WRITEEEMEMBLK_RSP_LEN},        
   {HOSTCMD_READEEMEMBLK_CMD              ,HOSTCMD_READEEMEMBLK_CMD_LEN             ,HOSTCMD_READEEMEMBLK_RSP_LEN},         
   {HOSTCMD_EXECUTE_CMD                   ,HOSTCMD_EXECUTE_CMD_LEN                  ,HOSTCMD_EXECUTE_RSP_LEN},              
   {HOSTCMD_SET_PULSE_CMD                 ,HOSTCMD_SET_PULSE_CMD_LEN                ,HOSTCMD_SET_PULSE_RSP_LEN},            
   {HOSTCMD_GET_PULSE_CMD                 ,HOSTCMD_GET_PULSE_CMD_LEN                ,HOSTCMD_GET_PULSE_RSP_LEN},            
   {HOSTCMD_SET_TRIG_MAP_CMD              ,HOSTCMD_SET_TRIG_MAP_CMD_LEN             ,HOSTCMD_SET_TRIG_MAP_RSP_LEN},         
   {HOSTCMD_GET_TRIG_MAP_CMD              ,HOSTCMD_GET_TRIG_MAP_CMD_LEN             ,HOSTCMD_GET_TRIG_MAP_RSP_LEN},         
   {HOSTCMD_SET_HV_SUPPLY_CMD             ,HOSTCMD_SET_HV_SUPPLY_CMD_LEN            ,HOSTCMD_SET_HV_SUPPLY_RSP_LEN},        
   {HOSTCMD_GET_HV_SUPPLY_CMD             ,HOSTCMD_GET_HV_SUPPLY_CMD_LEN            ,HOSTCMD_GET_HV_SUPPLY_RSP_LEN},        
   {HOSTCMD_SET_LV_SUPPLY_CMD             ,HOSTCMD_SET_LV_SUPPLY_CMD_LEN            ,HOSTCMD_SET_LV_SUPPLY_RSP_LEN},        
   {HOSTCMD_GET_LV_SUPPLY_CMD             ,HOSTCMD_GET_LV_SUPPLY_CMD_LEN            ,HOSTCMD_GET_LV_SUPPLY_RSP_LEN},        
   {HOSTCMD_TRIGGER_CMD                   ,HOSTCMD_TRIGGER_CMD_LEN                  ,HOSTCMD_TRIGGER_RSP_LEN},              
   {HOSTCMD_SET_PULSE_OUT_CMD             ,HOSTCMD_SET_PULSE_OUT_CMD_LEN            ,HOSTCMD_SET_PULSE_OUT_RSP_LEN},        
   {HOSTCMD_GET_PULSE_OUT_CMD             ,HOSTCMD_GET_PULSE_OUT_CMD_LEN            ,HOSTCMD_GET_PULSE_OUT_RSP_LEN},        
   {HOSTCMD_SET_GATE_MAP_CMD              ,HOSTCMD_SET_GATE_MAP_CMD_LEN             ,HOSTCMD_SET_GATE_MAP_RSP_LEN},         
   {HOSTCMD_CALC_LED_TEMP_CMD             ,HOSTCMD_CALC_LED_TEMP_CMD_LEN            ,HOSTCMD_CALC_LED_TEMP_RSP_LEN},        
   {HOSTCMD_CALC_MAX_WIDTH_CMD            ,HOSTCMD_CALC_MAX_WIDTH_CMD_LEN           ,HOSTCMD_CALC_MAX_WIDTH_RSP_LEN},       
   {HOSTCMD_CALC_MAX_CURRENT_CMD          ,HOSTCMD_CALC_MAX_CURRENT_CMD_LEN         ,HOSTCMD_CALC_MAX_CURRENT_RSP_LEN},     
   {HOSTCMD_CALC_MAX_FREQ_CMD             ,HOSTCMD_CALC_MAX_FREQ_CMD_LEN            ,HOSTCMD_CALC_MAX_FREQ_RSP_LEN},        
   {HOSTCMD_CALC_SUPPLY_VOLTAGE_CMD       ,HOSTCMD_CALC_SUPPLY_VOLTAGE_CMD_LEN      ,HOSTCMD_CALC_SUPPLY_VOLTAGE_RSP_LEN},  
   {HOSTCMD_SET_PULSE_EX_CMD              ,HOSTCMD_SET_PULSE_EX_CMD_LEN             ,HOSTCMD_SET_PULSE_EX_RSP_LEN            },
   {HOSTCMD_GET_PULSE_EX_CMD              ,HOSTCMD_GET_PULSE_EX_CMD_LEN             ,HOSTCMD_GET_PULSE_EX_RSP_LEN            },
   {HOSTCMD_CALC_LED_TEMP_EX_CMD          ,HOSTCMD_CALC_LED_TEMP_EX_CMD_LEN         ,HOSTCMD_CALC_LED_TEMP_EX_RSP_LEN        },
   {HOSTCMD_CALC_MAX_WIDTH_EX_CMD         ,HOSTCMD_CALC_MAX_WIDTH_EX_CMD_LEN        ,HOSTCMD_CALC_MAX_WIDTH_EX_RSP_LEN       },
   {HOSTCMD_CALC_MAX_CURRENT_EX_CMD       ,HOSTCMD_CALC_MAX_CURRENT_EX_CMD_LEN      ,HOSTCMD_CALC_MAX_CURRENT_EX_RSP_LEN     },
   {HOSTCMD_CALC_MAX_FREQ_EX_CMD          ,HOSTCMD_CALC_MAX_FREQ_EX_CMD_LEN         ,HOSTCMD_CALC_MAX_FREQ_EX_RSP_LEN        },
   {HOSTCMD_CALC_SUPPLY_VOLTAGE_EX_CMD    ,HOSTCMD_CALC_SUPPLY_VOLTAGE_EX_CMD_LEN   ,HOSTCMD_CALC_SUPPLY_VOLTAGE_EX_RSP_LEN  },
#if 1
   {HOSTCMD_GET_IP_CMD                    ,HOSTCMD_GET_IP_CMD_LEN                   ,HOSTCMD_GET_IP_RSP_LEN                  },
   {HOSTCMD_GET_MAC_CMD                   ,HOSTCMD_GET_MAC_CMD_LEN                  ,HOSTCMD_GET_MAC_RSP_LEN                 },
   {HOSTCMD_SET_IP_CMD                    ,HOSTCMD_SET_IP_CMD_LEN                   ,HOSTCMD_SET_IP_RSP_LEN                  },
   {HOSTCMD_SET_MAC_CMD                   ,HOSTCMD_SET_MAC_CMD_LEN                  ,HOSTCMD_SET_MAC_RSP_LEN                 },
   {HOSTCMD_SET_REPEAT_CMD                ,HOSTCMD_SET_REPEAT_CMD_LEN               ,HOSTCMD_SET_REPEAT_RSP_LEN              },
   {HOSTCMD_GET_REPEAT_CMD                ,HOSTCMD_GET_REPEAT_CMD_LEN               ,HOSTCMD_GET_REPEAT_RSP_LEN              },
#endif
};

TKCMDSET_USB_DATA_STRUCT tkcmdset_usb_data[TKUSB_MAX_USB_PORTS];

#define TKCMDSET_SHORT_CMD_ID(port)     tkcmdset_usb_data[port].short_cmd_id;
#define TKCMDSET_LONG_CMD_ID(port)      tkcmdset_usb_data[port].long_cmd_id;
#define TKCMDSET_SHORT_RSP_ID(port)     tkcmdset_usb_data[port].short_rsp_id;
#define TKCMDSET_LONG_RSP_ID(port)      tkcmdset_usb_data[port].long_rsp_id;
#define TKCMDSET_FEATURE_ID(port)       tkcmdset_usb_data[port].feature_id;
#define TKCMDSET_SHORT_CMD_LEN(port)    tkcmdset_usb_data[port].short_cmd_len;
#define TKCMDSET_LONG_CMD_LEN(port)     \
   (iface_map[port]==IFACE_USB) ? tkcmdset_usb_data[port].long_cmd_len:HOSTCMD_READEEMEMBLK_RSP_LEN;
#define TKCMDSET_SHORT_RSP_LEN(port)    tkcmdset_usb_data[port].short_rsp_len;
#define TKCMDSET_LONG_RSP_LEN(port)     \
   (iface_map[port]==IFACE_USB) ? tkcmdset_usb_data[port].long_rsp_len:HOSTCMD_READEEMEMBLK_RSP_LEN;
#define TKCMDSET_FEATURE_LEN(port)      tkcmdset_usb_data[port].feature_len;

#define TKCMDSET_DO_COMMAND_M(port, tx_ptr, rx_ptr, timeout) \
	((iface_map[port]==IFACE_USB) ? tkcmdset_do_command_usb(port, tx_ptr, rx_ptr, timeout) : \
									tkcmdset_do_command_tcp(port, tx_ptr, rx_ptr, timeout))

/* Global variables unique to each process */
HKEY tkcmdset_reg_key;
HANDLE tkcmdset_usb_muti[TKUSB_MAX_USB_PORTS];
HANDLE tkcmdset_tcp_muti[TKUSB_MAX_USB_PORTS];
UINT16_T tkcmdset_default_timeout = 1500;
UINT16_T tkcmdset_tcp_timeout = 5000;
UINT16_T tkcmdset_flash_write_timeout = 6000;
STATIC TKCMDSET_IFACE_E iface_map[TKUSB_MAX_USB_PORTS];

/* Prototypes for local functions */
STATIC ERRCODE_T tkcmdset_do_command_usb(UINT8_T port, UINT8_T *tx_ptr, UINT8_T *rx_ptr, UINT16_T timeout);
STATIC ERRCODE_T tkcmdset_do_command_tcp(UINT8_T port, UINT8_T *tx_ptr, UINT8_T *rx_ptr, UINT16_T timeout);
STATIC ERRCODE_T tkcmdset_get_msg_data_len(UINT8_T msg_id, UINT8_T is_cmd, UINT16_T *len);
STATIC UINT8_T * tkcmdset_add_num8(UINT8_T num, UINT8_T *bufptr);
STATIC UINT8_T * tkcmdset_add_num16(UINT16_T num, UINT8_T *bufptr);
STATIC UINT8_T * tkcmdset_add_num32(UINT32_T num, UINT8_T *bufptr);
STATIC UINT8_T * tkcmdset_get_num8(UINT8_T *numptr, UINT8_T *bufptr);
STATIC UINT8_T * tkcmdset_get_num16(UINT16_T *numptr, UINT8_T *bufptr);
STATIC UINT8_T * tkcmdset_get_num32(UINT32_T *numptr, UINT8_T *bufptr);

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
                     )
{
   ERRCODE_T ecode;
   UINT16_T i;
   char temp[MAX_PATH];

   switch(ul_reason_for_call)
   {
   case DLL_PROCESS_ATTACH:

      if(dll_instance_count++ == 0)
      {
         _getcwd(DLLPath,sizeof(DLLPath));

         strcat(DLLPath,"\\");

      }

      /* Open the Registry */
      ecode = TKREG_INIT_M(TKCMDSET_REG_SUBKEY);

      /* Create a default link for ECODES.INI if not already present */
      ecode = TKREG_OPEN_STRING_M(TKCMDSET_REG_ECODES_FILENAME, TKCMDSET_REG_DEFAULT_ECODES_FILENAME, temp, sizeof(temp));

      /* Create a mutex for each USB port to force serialization of commands */
      for(i = 0; i < TKUSB_MAX_USB_PORTS; i++)
      {
         sprintf(temp,"USB_mutex_%d",i);
         tkcmdset_usb_muti[i] = CreateMutex(NULL,FALSE,temp);
      }

      /* Create a mutex for each TCP port to force serialization of commands */
      for(i = 0; i < TKUSB_MAX_USB_PORTS; i++)
      {
         sprintf(temp,"TCP_mutex_%d",i);
         tkcmdset_tcp_muti[i] = CreateMutex(NULL,FALSE,temp);
      }

      /* Initialize toolkits */
      TKTIME_INIT_M();

      break;

   case DLL_THREAD_ATTACH:
      break;
   case DLL_THREAD_DETACH:
      break;
   case DLL_PROCESS_DETACH:

      /* Close the Registry */
      ecode = TKREG_SHUTDOWN_M();

      /* Free mutex handles for each USB port */
      for(i = 0; i < TKUSB_MAX_USB_PORTS; i++)
      {
         CloseHandle(tkcmdset_usb_muti[i]);
      }
      /* Free mutex handles for each TCP port */
      for(i = 0; i < TKUSB_MAX_USB_PORTS; i++)
      {
         CloseHandle(tkcmdset_tcp_muti[i]);
      }
      break;
   }

   return TRUE;
}


/****************************************************************************
   tkcmdset_get_dll_version()

   Returns the current version number of this software.

   Inputs:     None

   Outputs:    None
   
   Returns:    UINT16_T    ver_info    High byte is major version number
                                       Low byte is minor version number

****************************************************************************/
MPDH8USB_TCP_API UINT16_T tkcmdset_get_dll_version(void)
{

   char *appname = "mpdh8usb.dll";
   UINT16_T retval = 0;
   DWORD dummy;
   VS_FIXEDFILEINFO * FixedInfo;
   unsigned int FixedInfoLen;

   /* Get the size of the version information */
   int VersionInfoSize = GetFileVersionInfoSize(appname,&dummy);
   if(!VersionInfoSize)
      return 0;

   /* Allocate memory to hold version information */
   char *VersionBuf = new char[VersionInfoSize];
   if(!VersionBuf)
      return 0;

   /* Load version information into buffer */
   if(GetFileVersionInfo(appname,dummy,VersionInfoSize,VersionBuf))
   {

      /* Extract file version information from buffer */
      VerQueryValue(VersionBuf,"\\",(void **) &FixedInfo,&FixedInfoLen);

      /* Encode version info into UINT16_T */
      retval = (UINT16_T) FixedInfo->dwFileVersionMS;
      retval <<= 8;
      retval |= (FixedInfo->dwFileVersionLS & 0xFF);
   }

   /* Free allocated memory */
   delete [] VersionBuf;

   /* Return version number */
   return retval;
}


/****************************************************************************
   tkcmdset_get_error_name

   About:      Returns the name of the error associated with the code

   Inputs:     ERRCODE_T   ecode       Code to be looked up
               UINT16_T    namelen     Maximum length of name string

   Outputs:    char *      nameptr     Location to place name string

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_get_error_name(ERRCODE_T ecode,char * nameptr,UINT16_T namelen)
{
   ERRCODE_T my_ecode;
   char temp[10];
   char ecodes_filename[MAX_PATH];

   /* Get the error filename */
   my_ecode = TKREG_GET_STRING_M(TKCMDSET_REG_ECODES_FILENAME,ecodes_filename,sizeof(ecodes_filename));
   if(ecodes_filename[0] == '.')
   {
      char drive[_MAX_DRIVE];
      char dir[_MAX_DIR];
      char fname[_MAX_FNAME];
      char ext[_MAX_EXT];

      _splitpath(ecodes_filename,NULL,NULL,fname,ext);
      _splitpath(DLLPath,drive,dir,NULL,NULL);

      _makepath(ecodes_filename,drive,dir,fname,ext);
   }

   /* Lookup error name based on code */
   sprintf(temp,"%d", ecode);
   GetPrivateProfileString(temp,TKCMDSET_INI_ECODE_NAME,"Unknown Error",
                           nameptr,namelen,ecodes_filename);
   return 0;
}

/****************************************************************************
   tkcmdset_get_error_text

   About:      Returns the text description of the error associated with the code

   Inputs:     ERRCODE_T   ecode       Code to be looked up
               UINT16_T    namelen     Maximum length of text description string

   Outputs:    char *      nameptr     Location to place text description string

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_get_error_text(ERRCODE_T ecode,char * textptr,UINT16_T textlen)
{
   ERRCODE_T my_ecode;
   char temp[10];
   char ecodes_filename[MAX_PATH];

   /* Get the error filename */
   my_ecode = TKREG_GET_STRING_M(TKCMDSET_REG_ECODES_FILENAME,ecodes_filename,sizeof(ecodes_filename));
   if(ecodes_filename[0] == '.')
   {
      char drive[_MAX_DRIVE];
      char dir[_MAX_DIR];
      char fname[_MAX_FNAME];
      char ext[_MAX_EXT];

      _splitpath(ecodes_filename,NULL,NULL,fname,ext);
      _splitpath(DLLPath,drive,dir,NULL,NULL);

      _makepath(ecodes_filename,drive,dir,fname,ext);
   }

   /* Lookup error text based on code */
   sprintf(temp,"%d", ecode);
   GetPrivateProfileString(temp,TKCMDSET_INI_ERRCODE_TEXT,"No further information",
                           textptr,textlen,ecodes_filename);

   return 0;
}

/****************************************************************************
   tkcmdset_init_ex

   About:      Initializes the Command Set Toolkit functions
               Extended to indicate this DLL supports 32-bit pulse widths.

   Inputs:     UINT8_T     port        Communication port to use
               INT8_T *    product_name Name of USB device to open

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_init_ex(UINT8_T port, INT8_T * product_name, INT8_T * serial_num)
{
   ERRCODE_T ecode = NO_ERR;
   FILE * ecodes_file;
   DWORD size;
   char ecodes_filename[MAX_PATH];
   OPENFILENAME ofn;
   UINT8_T i;

   char default_serialnum[]="7";
   /* Verify Error Codes file.                                          */
   /* If not found, get the path with a Common Dialog Box               */
   /* and then try to open the file again.                              */

   /* Disable the ecodes filename verification for now */
   while(FALSE) // (TRUE)
   {

      /* Get the filename for the Error Code Definition File */
      size = sizeof(ecodes_filename);
      RegQueryValueEx(tkcmdset_reg_key,TKCMDSET_REG_ECODES_FILENAME,0,NULL,
                      (LPBYTE) &ecodes_filename[0],&size);


      /* Check for the file */
      ecodes_file = fopen(ecodes_filename,"rt");

      /* Display an File Open Dialog box if the filename is invalid */
      if(ecodes_file == NULL)
      {
         MessageBox(GetActiveWindow(),"Error Code Description File not found, "   \
                    "please locate the file.","File Open Error",MB_ICONEXCLAMATION | MB_OK);


         /* Build the OPENFILENAME structure */
         ecodes_filename[0] = '\0';
         memset(&ofn,0,sizeof(OPENFILENAME));
         ofn.lStructSize = sizeof(OPENFILENAME);
         ofn.hwndOwner = GetActiveWindow();
         ofn.lpstrFile = ecodes_filename;
         ofn.nMaxFile = sizeof(ecodes_filename);
         ofn.lpstrFilter = "INI\0*.ini\0Text\0*.txt\0All\0*.*\0";
         ofn.nFilterIndex = 1;
         ofn.lpstrFileTitle = NULL;
         ofn.nMaxFileTitle = 0;
         ofn.lpstrInitialDir = NULL;
         ofn.lpstrTitle = "Open Error Code Description File";
         ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

         /* Display the File Open Dialog Box */
         if(GetOpenFileName(&ofn) != FALSE)
         {
            RegSetValueEx(tkcmdset_reg_key,TKCMDSET_REG_ECODES_FILENAME,0,REG_SZ,
                          (LPBYTE) &ecodes_filename[0],strlen(ecodes_filename));
         }
         /* Cancel selected, exit the application */
         else
         {
            MessageBox(GetActiveWindow(),"No filename specified, error descriptions "  \
                       "will not be available.","File Open Error",MB_ICONERROR | MB_OK);
            ecode = TKCMDSET_NO_ECODE_FILE_ERR;
            break;
         }
      }
      else
      {
         fclose(ecodes_file);
         break;
      }
   }


   /* Establish a connection with the device */
   if(ecode == NO_ERR)
   {
      /* Attempt to connect via the USB HID Interface */
      if(ecode = TKUSB_INIT_M(port,product_name,serial_num))
      {
      }
      else
      {
         TKUSB_REPORT_INFO_STRUCT ri = {0};
         UINT8_T usage;
         for(i = 0; i < 5; i++)
         {
            switch(i)
            {
            case 0: usage = HOSTCMD_SHORT_CMD_USAGE;  break;
            case 1: usage = HOSTCMD_SHORT_RSP_USAGE;  break;
            case 2: usage = HOSTCMD_LONG_CMD_USAGE;   break;
            case 3: usage = HOSTCMD_LONG_RSP_USAGE;   break;
            case 4: usage = HOSTCMD_FEATURE_USAGE;    break;
            }

            if(ecode = TKUSB_GET_REPORT_INFO_M(port,usage,&ri))
            {
               break;
            }
            else switch(usage)
               {
               case HOSTCMD_SHORT_CMD_USAGE:
                  tkcmdset_usb_data[port].short_cmd_id = ri.report_id;
                  tkcmdset_usb_data[port].short_cmd_len = ri.report_size;
                  break;
               case HOSTCMD_SHORT_RSP_USAGE:
                  tkcmdset_usb_data[port].short_rsp_id = ri.report_id;
                  tkcmdset_usb_data[port].short_rsp_len = ri.report_size;
                  break;
               case HOSTCMD_LONG_CMD_USAGE:
                  tkcmdset_usb_data[port].long_cmd_id = ri.report_id;
                  tkcmdset_usb_data[port].long_cmd_len = ri.report_size;
                  break;
               case HOSTCMD_LONG_RSP_USAGE:
                  tkcmdset_usb_data[port].long_rsp_id = ri.report_id;
                  tkcmdset_usb_data[port].long_rsp_len = ri.report_size;
                  break;
               case HOSTCMD_FEATURE_USAGE:
                  tkcmdset_usb_data[port].feature_id = ri.report_id;
                  tkcmdset_usb_data[port].feature_len = ri.report_size;
                  break;

               }
         }
         if( ecode == NO_ERR )
         {
            iface_map[port] = IFACE_USB;
         }

      }
   }
   return ecode;
}

/****************************************************************************
   tkcmdset_connect

   About:      Initializes the Command Set Toolkit functions
               Extended to indicate this DLL supports 32-bit pulse widths.

   Inputs:     UINT8_T     port         Communication port to use
               INT8_T *    product_name Name of USB device to open
               INT8_T *   ip_addr		IP address of device to open	

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_connect(UINT8_T port, INT8_T * product_name, INT8_T * ip_addr)
{
   ERRCODE_T ecode = NO_ERR;
   FILE * ecodes_file;
   DWORD size;
   char ecodes_filename[MAX_PATH];
   OPENFILENAME ofn;

   char default_serialnum[]="7";
   /* Verify Error Codes file.                                          */
   /* If not found, get the path with a Common Dialog Box               */
   /* and then try to open the file again.                              */

   /* Disable the ecodes filename verification for now */
   while(FALSE) // (TRUE)
   {

      /* Get the filename for the Error Code Definition File */
      size = sizeof(ecodes_filename);
      RegQueryValueEx(tkcmdset_reg_key,TKCMDSET_REG_ECODES_FILENAME,0,NULL,
                      (LPBYTE) &ecodes_filename[0],&size);


      /* Check for the file */
      ecodes_file = fopen(ecodes_filename,"rt");

      /* Display an File Open Dialog box if the filename is invalid */
      if(ecodes_file == NULL)
      {
         MessageBox(GetActiveWindow(),"Error Code Description File not found, "   \
                    "please locate the file.","File Open Error",MB_ICONEXCLAMATION | MB_OK);


         /* Build the OPENFILENAME structure */
         ecodes_filename[0] = '\0';
         memset(&ofn,0,sizeof(OPENFILENAME));
         ofn.lStructSize = sizeof(OPENFILENAME);
         ofn.hwndOwner = GetActiveWindow();
         ofn.lpstrFile = ecodes_filename;
         ofn.nMaxFile = sizeof(ecodes_filename);
         ofn.lpstrFilter = "INI\0*.ini\0Text\0*.txt\0All\0*.*\0";
         ofn.nFilterIndex = 1;
         ofn.lpstrFileTitle = NULL;
         ofn.nMaxFileTitle = 0;
         ofn.lpstrInitialDir = NULL;
         ofn.lpstrTitle = "Open Error Code Description File";
         ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

         /* Display the File Open Dialog Box */
         if(GetOpenFileName(&ofn) != FALSE)
         {
            RegSetValueEx(tkcmdset_reg_key,TKCMDSET_REG_ECODES_FILENAME,0,REG_SZ,
                          (LPBYTE) &ecodes_filename[0],strlen(ecodes_filename));
         }
         /* Cancel selected, exit the application */
         else
         {
            MessageBox(GetActiveWindow(),"No filename specified, error descriptions "  \
                       "will not be available.","File Open Error",MB_ICONERROR | MB_OK);
            ecode = TKCMDSET_NO_ECODE_FILE_ERR;
            break;
         }
      }
      else
      {
         fclose(ecodes_file);
         break;
      }
   }

   /* Establish a connection with the device */
   if(ecode == NO_ERR)
   {
         /* Attempt to connect via TCP interface */
         if(ecode = TKTCP_INIT_M(port,product_name,ip_addr))
         {
         }
         else
         {
            iface_map[port] = IFACE_TCP;
         }

   }
   return ecode;
}

/*******************************************************************************
   tkcmdset_add_num8

   About:      Appends a 8-bit value to the buffer.
   
   Inputs:     UINT8_T  num      Value to be added to the buffer
               UINT8_T  *bufptr  Pointer to location in buffer to use
   
   Outputs:    
   
   Returns:    Pointer to next unused location in buffer
   
*******************************************************************************/
STATIC UINT8_T* tkcmdset_add_num8(UINT8_T num, UINT8_T *bufptr) 
{              
   *bufptr++ = num;

   return(bufptr);
}

/*******************************************************************************
   tkcmdset_get_num8

   About:      Gets a 8-bit value from the buffer.
   
   Inputs:     UINT8_T  *num     Pointer to location to store value
               UINT8_T  *bufptr  Pointer to location in buffer to use
   
   Outputs:    
   
   Returns:    Pointer to next unused location in buffer
   
*******************************************************************************/
STATIC UINT8_T* tkcmdset_get_num8(UINT8_T *num, UINT8_T *bufptr)
{
   *num = *bufptr++;

   return(bufptr);
}

/*******************************************************************************
   tkcmdset_add_num16

   About:      Appends a 16-bit value to the buffer.
   
   Inputs:     UINT16_T  num      Value to be added to the buffer
               UINT8_T   *bufptr  Pointer to location in buffer to use
   
   Outputs:    
   
   Returns:    Pointer to next unused location in buffer
   
*******************************************************************************/
STATIC UINT8_T* tkcmdset_add_num16(UINT16_T num, UINT8_T *bufptr) 
{              
   *bufptr++ = HI_BYTE(num);
   *bufptr++ = LO_BYTE(num);

   return(bufptr);
}

/*******************************************************************************
   tkcmdset_get_num16

   About:      Gets a 16-bit value from the buffer.
   
   Inputs:     UINT16_T *num     Pointer to location to store value
               UINT8_T  *bufptr  Pointer to location in buffer to use
   
   Outputs:    
   
   Returns:    Pointer to next unused location in buffer
   
*******************************************************************************/
STATIC UINT8_T* tkcmdset_get_num16(UINT16_T* num, UINT8_T *bufptr)
{
   *num = *bufptr++ << 8;
   *num |= *bufptr++;

   return(bufptr);
}

/*******************************************************************************
   tkcmdset_add_num32

   About:      Appends a 32-bit value to the buffer.
   
   Inputs:     UINT32_T  num      Value to be added to the buffer
               UINT8_T   *bufptr  Pointer to location in buffer to use
   
   Outputs:    
   
   Returns:    Pointer to next unused location in buffer
   
*******************************************************************************/
STATIC UINT8_T* tkcmdset_add_num32(UINT32_T num, UINT8_T *bufptr) 
{              
   *bufptr++ = HI_BYTE(HI_WORD(num));
   *bufptr++ = LO_BYTE(HI_WORD(num));
   *bufptr++ = HI_BYTE(LO_WORD(num));
   *bufptr++ = LO_BYTE(LO_WORD(num));

   return(bufptr);
}

/*******************************************************************************
   tkcmdset_get_num32

   About:      Gets a 32-bit value from the buffer.
   
   Inputs:     UINT32_T *num     Pointer to location to store value
               UINT8_T  *bufptr  Pointer to location in buffer to use
   
   Outputs:    
   
   Returns:    Pointer to next unused location in buffer
   
*******************************************************************************/
STATIC UINT8_T* tkcmdset_get_num32(UINT32_T* num, UINT8_T *bufptr)
{  
   UINT8_T i;

   *num = 0;
   for(i = 0; i < sizeof(UINT32_T); i++)
   {
      *num = *num << 8;
      *num |= *bufptr++ & LO_BYTE_MASK;
   }

   return(bufptr);
}

/*******************************************************************************
   tkcmdset_add_float32

   About:      Appends a 32-bit floating point value to the buffer.
   
   Inputs:     FLOAT32_T flt      Value to be added to the buffer
               UINT8_T   *bufptr  Pointer to location in buffer to use
   
   Outputs:    
   
   Returns:    Pointer to next unused location in buffer
   
*******************************************************************************/
STATIC UINT8_T* tkcmdset_add_float32(FLOAT32_T flt, UINT8_T *bufptr) 
{              
   UINT32_T * numptr = (UINT32_T *) &flt;

   *bufptr++ = HI_BYTE(HI_WORD(*numptr));
   *bufptr++ = LO_BYTE(HI_WORD(*numptr));
   *bufptr++ = HI_BYTE(LO_WORD(*numptr));
   *bufptr++ = LO_BYTE(LO_WORD(*numptr));

   return(bufptr);
}

/*******************************************************************************
   tkcmdset_get_float32

   About:      Gets a 32-bit floating point value from the buffer.
   
   Inputs:     FLOAT32_T *fltptr  Pointer to location to store value
               UINT8_T   *bufptr  Pointer to location in buffer to use
   
   Outputs:    
   
   Returns:    Pointer to next unused location in buffer
   
*******************************************************************************/
STATIC UINT8_T* tkcmdset_get_float32(FLOAT32_T* fltptr, UINT8_T *bufptr)
{  
   UINT32_T * numptr = (UINT32_T *) fltptr;
   UINT8_T i;

   *numptr = 0;
   for(i = 0; i < sizeof(UINT32_T); i++)
   {
      *numptr = *numptr << 8;
      *numptr |= *bufptr++ & LO_BYTE_MASK;
   }

   return(bufptr);
}

/****************************************************************************
   tkcmdset_set_feature

   About:      Sends a USB SetFeature Standard Request to the device
               This is independent of the command / response protocol

   Inputs:     UINT8_T     port        Communication port to use
               UINT16_T    feature     Identifier for desired feature to activate

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_set_feature(UINT8_T port, UINT16_T feature)
{

   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];

   /* Build the Feature Report */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_FEATURE_ID(port);

   if(feature == TKCMDSET_START_LOADER_FEATURE)
   {
      feature = HOSTCMD_STARTLOADER_FEATURE;
   }
   else if(feature == TKCMDSET_START_APPLICATION_FEATURE)
   {
      feature = HOSTCMD_STARTAPPLICATION_FEATURE;
   }
   else if(feature == TKCMDSET_RESET_FEATURE)
   {
      feature = HOSTCMD_RESET_FEATURE;
   }

   tkcmdset_add_num16(feature,&cmd_buf[HOSTCMD_FEATURE_DATA_OFFSET]);

   return(TKUSB_SET_FEATURE_M(port,cmd_buf));
}

/****************************************************************************
   tkcmdset_do_command_usb

   About:      Handle the command / response transaction with the USB device

   Inputs:     UINT8_T     port        Identifier for device
               UINT8_T *   tx_ptr      Pointer to data block to be transmitted
               UINT16_T    timeout     Time to wait for response in milliseconds

   Outputs:    UINT8_T *   rx_ptr      Pointer to data block to save received data

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
STATIC ERRCODE_T tkcmdset_do_command_usb(UINT8_T port, UINT8_T *tx_ptr, UINT8_T *rx_ptr, UINT16_T timeout)
{
   ERRCODE_T ecode;
   DWORD retval;
   UINT8_T sent_command = tx_ptr[HOSTCMD_CMD_ID_OFFSET];
   UINT8_T rsp_id = rx_ptr[HOSTCMD_REPORT_ID_OFFSET];

   /* Wait for Mutex before accessing hardware */
   retval = WaitForSingleObject(tkcmdset_usb_muti[port],TKCMDSET_MUTEX_TIMEOUT);

   /* Make sure the mutex didn't timeout */
   if(retval == WAIT_TIMEOUT)
   {
      ecode = TKCMDSET_MUTEX_TIMEOUT_ERR;
   }
   /* Make sure there weren't other failures */
   else if(retval == WAIT_FAILED)
   {
      ecode = TKCMDSET_MUTEX_FAILED_ERR;
   }
   /* Make sure port is open and valid.  */
   else if(ecode = TKUSB_VALIDATE_PORT_M(port))
   {
   }
   /* Mutex available and port is good, now send the command */
   else
   {
      /* Adjust priority of this task during transaction */
//      retval = SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_TIME_CRITICAL);

      /* Clear the receive buffer before every command */
      if(ecode = TKUSB_CLEAR_RX_M(port))
      {
      }
      /* Send the command to the device */
      else
         if(ecode = TKUSB_WRITE_M(port,tx_ptr,timeout))
      {
      }
      /* Delay while command is processed.         */
      /* This also allows the Windows message loop */
      /* to run so that screen updates take place. */
      else if(ecode = TKTIME_DELAY_MS_M(0))
      {
      }
      /* Receive the response from the device */
      else if(ecode = TKUSB_READ_M(port,rx_ptr,timeout))
      {
      }
      /* Check if the response indicates an error */
      else if(rx_ptr[HOSTCMD_RSP_ID_OFFSET] == HOSTCMD_ERRORFLAG)
      {
         /* Extract the error number from the response */
         ecode = rx_ptr[HOSTCMD_RSP_DATA_OFFSET] << 8;
         ecode |= rx_ptr[HOSTCMD_RSP_DATA_OFFSET + 1];
      }
      /* Check if the expected report was returned */
      else if(rsp_id != rx_ptr[HOSTCMD_REPORT_ID_OFFSET])
      {
         ecode = TKCMDSET_UNEXPECTED_RESPONSE_ERR;
      }
      /* Verify that the command and response identifiers match */
      else if(sent_command != rx_ptr[HOSTCMD_RSP_ID_OFFSET])
      {
         ecode = TKCMDSET_INVALID_RESPONSE_ERR;
      }

      /* Return to normal priority */
//      retval = SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_NORMAL);

   }

  /* Release the mutex */
  ReleaseMutex(tkcmdset_usb_muti[port]);
   return ecode;
}

/****************************************************************************
   tkcmdset_do_command_tcp

   About:      Handle the command / response transaction with the USB device

   Inputs:     UINT8_T     port        Identifier for device
               UINT8_T *   tx_ptr      Pointer to data block to be transmitted
               UINT16_T    timeout     Time to wait for response in milliseconds

   Outputs:    UINT8_T *   rx_ptr      Pointer to data block to save received data

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
STATIC ERRCODE_T tkcmdset_do_command_tcp(UINT8_T port, UINT8_T *tx_ptr, UINT8_T *rx_ptr, UINT16_T timeout)
{
   ERRCODE_T ecode;
   DWORD retval;
   UINT8_T sent_command = tx_ptr[HOSTCMD_CMD_ID_OFFSET];
   UINT8_T rsp_id = rx_ptr[HOSTCMD_REPORT_ID_OFFSET];
   UINT16_T cmd_len;
   UINT16_T rsp_len=0xFFFF;
   UINT8_T rsp_crc;
   UINT32_T command_fail_cnt;

   /* Wait for Mutex before accessing hardware */
   retval = WaitForSingleObject(tkcmdset_tcp_muti[port],TKCMDSET_MUTEX_TIMEOUT);

   /* Make sure the mutex didn't timeout */
   if(retval == WAIT_TIMEOUT)
   {
      ecode = TKCMDSET_MUTEX_TIMEOUT_ERR;
   }
   /* Make sure there weren't other failures */
   else if(retval == WAIT_FAILED)
   {
      ecode = TKCMDSET_MUTEX_FAILED_ERR;
   }
   /* Make sure port is open and valid.  */
   else if(ecode = TKTCP_VALIDATE_PORT_M(port))
   {
   }
   /* Mutex available and port is good, now send the command */
   else
   {
      /* Adjust priority of this task during transaction */
//      retval = SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_TIME_CRITICAL);

      /* Clear the receive buffer before every command */
      if(ecode = TKTCP_CLEAR_RX_M(port))
      {
      }
      /* Calculate command buffer length */
      else if( ecode = TKCMDSET_GET_CMD_LEN_M( tx_ptr[HOSTCMD_CMD_ID_OFFSET], &cmd_len) )
      {
      }
      /* Calculate the CRC */
      else if( (tx_ptr[cmd_len+HOSTCMD_HEADERSIZE] = TKCRC_CALC_CRC8_M(tx_ptr, cmd_len + HOSTCMD_HEADERSIZE)) ? NO_ERR : NO_ERR )
      {
      }
      /* Send the command to the device */
      else if(ecode = TKTCP_WRITE_M(port,tx_ptr, cmd_len+HOSTCMD_HEADERSIZE+HOSTCMD_CSUM_SIZE, timeout))
      {
      }
      /* Delay while command is processed.         */
      /* This also allows the Windows message loop */
      /* to run so that screen updates take place. */
      else if(ecode = TKTIME_DELAY_MS_M(0))
      {
      }
      /* Receive enough of response to calculate length */
      else if(ecode = TKTCP_READ_M(port,rx_ptr, HOSTCMD_HEADERSIZE, timeout))
      {
      }
      /* Check if the response indicates an error */
      else if(rx_ptr[HOSTCMD_RSP_ID_OFFSET] == HOSTCMD_ERRORFLAG)
      {
         /* Receive the remainder of the error */
         if(ecode = TKTCP_READ_M(port,&rx_ptr[HOSTCMD_HEADERSIZE], (HOSTCMD_ERROR_RSP_LEN + HOSTCMD_CSUM_SIZE), timeout))
         {
         }
         /* Validate the CRC */
         else if( (rsp_crc = TKCRC_CALC_CRC8_M(rx_ptr, HOSTCMD_ERROR_RSP_LEN + HOSTCMD_HEADERSIZE)) ? NO_ERR : NO_ERR )
         {
         }
         else if( rsp_crc != rx_ptr[HOSTCMD_ERROR_RSP_LEN+HOSTCMD_HEADERSIZE] )
         {
            ecode = TKCMDSET_CRC_ERR;
         }
         else
         {
            /* Extract the error number from the response */
            ecode = rx_ptr[1+HOSTCMD_RSP_DATA_OFFSET+strlen(HOSTCMD_ERRORSTRING)] << 8;
            ecode |= rx_ptr[HOSTCMD_RSP_DATA_OFFSET+strlen(HOSTCMD_ERRORSTRING)];
         }
      }
      /* Get length for remainder of response */
      else if(ecode = TKCMDSET_GET_RSP_LEN_M(rx_ptr[HOSTCMD_RSP_ID_OFFSET], &rsp_len) )
      {
      }
      /* Receive remainder of response */
      else if(ecode = TKTCP_READ_M(port,&rx_ptr[HOSTCMD_HEADERSIZE], (rsp_len + HOSTCMD_CSUM_SIZE), timeout))
      {
      }
      /* Validate the CRC */
      else if( (rsp_crc = TKCRC_CALC_CRC8_M(rx_ptr, rsp_len + HOSTCMD_HEADERSIZE)) ? NO_ERR : NO_ERR )
      {
      }
      else if( rsp_crc != rx_ptr[rsp_len + HOSTCMD_HEADERSIZE] )
      {
         ecode = TKCMDSET_CRC_ERR;
      }
      /* Verify that the command and response identifiers match */
      else if(sent_command != rx_ptr[HOSTCMD_RSP_ID_OFFSET])
      {
         ecode = TKCMDSET_INVALID_RESPONSE_ERR;
      }

      /* Return to normal priority */
//      retval = SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_NORMAL);

      /* Release the mutex */
      ReleaseMutex(tkcmdset_tcp_muti[port]);
   }

   /*DEBUG*/
   if(ecode != 0)
   {
		TKREG_OPEN_UINT32_M("CommandFailCount", 0, &command_fail_cnt);
		command_fail_cnt+=1;
		TKREG_SET_UINT32_M("CommandFailCount", command_fail_cnt);
   }

   
   return ecode;
}

/****************************************************************************
   tkcmdset_echo

   About:      Send and receive a data block, no action taken by device

   Inputs:     UINT8_T     port        Identifier for device
               UINT8_T *   cmd_ptr     Location of data to send to device

   Outputs:    UINT8_T *   rsp_ptr     Location to store data returned by device

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_echo(UINT8_T port, UINT8_T * cmd_ptr, UINT8_T * rsp_ptr)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE];

   /* Initialize response string to NULL */
   *rsp_ptr = NULL;

   memset(rsp_buf,0,sizeof(rsp_buf));

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_LONG_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_LONG_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_ECHO_CMD);

   __try
   {
      memcpy(&cmd_buf[HOSTCMD_CMD_DATA_OFFSET],cmd_ptr,TKUSB_MAX_PACKET_SIZE);
   }
   __except(EXCEPTION_EXECUTE_HANDLER)
   {
   }



   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save results if no error */
   if(!ecode)
   {
      __try
      {
         memcpy(rsp_ptr,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET],TKUSB_MAX_PACKET_SIZE);
      }
      __except(EXCEPTION_EXECUTE_HANDLER)
      {
      }
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_fw_version

   About:      Reads firmware version information from the device

   Inputs:     UINT8_T     port        Identifier for device

   Outputs:    UINT8_T *   ver_ptr     FW version string stored at this location

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_fw_version(UINT8_T port, UINT8_T *ver_ptr)
{
   ERRCODE_T ecode = 0;
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];

   /* Initialize version string to NULL */
   *ver_ptr = NULL;

   if( iface_map[port] == IFACE_TCP )
   {
      /* Send command to get FW version */

      /* Build the command */
      cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
      rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
      cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_VERSION_CMD);

      /* Execute the command */
      ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

      /* Save results if no error */
      if(!ecode)
      {
         strcpy((char *) ver_ptr, (char *) &rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
      }
   }
   else





   if((ecode = TKUSB_GET_STRING_M(port,
                                  HOSTCMD_FW_PARTNUMBER_STRING,
                                  rsp_buf,sizeof(rsp_buf))) == NO_ERR)
   {
      strcpy((char *) ver_ptr, (char *) rsp_buf);
   }
   return ecode;
}

/****************************************************************************
   tkcmdset_hw_version

   About:      Reads hardware version information from the device

   Inputs:     UINT8_T     port        Identifier for device

   Outputs:    UINT8_T *   ver_ptr     HW version string stored at this location

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_hw_version(UINT8_T port, UINT8_T *ver_ptr)
{
   ERRCODE_T ecode = 0;
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT16_T x;
   /* Initialize version string to NULL */
   *ver_ptr = NULL;

   if( iface_map[port] == IFACE_TCP )
   {
      APHEADER_MANUFACTURE_STRUCT_T  mfg_info;
      /* Read EEPROM to get serial number */
      if( ecode = tkcmdset_read_eemem_block(port, APHEADER_MANUFACTURE_DATA_ADDRESS, (UINT8_T *) &mfg_info, sizeof(mfg_info)) )
      {
      }
      /* Validate structure */
      else if( APHEADER_MANUFACTURE_INFO_VALID_M(&mfg_info) )
      {
         /* Copy serial number */
         for(x=0;x<sizeof(mfg_info.pn_hw);x++)
         {
            ver_ptr[x] = mfg_info.pn_hw[x];
         }
      }
   }
   else if((ecode = TKUSB_GET_STRING_M(port,
                                  HOSTCMD_HW_PARTNUMBER_STRING,
                                  rsp_buf,sizeof(rsp_buf))) == NO_ERR)
   {
      strcpy((char *) ver_ptr, (char *) rsp_buf);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_serial_number

   About:      Reads the serial number information from the device

   Inputs:     UINT8_T     port        Identifier for device

   Outputs:    UINT8_T *   ser_ptr     Serial number string stored at this location

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_serial_number(UINT8_T port, UINT8_T *ser_ptr)
{
   ERRCODE_T ecode = 0;
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT16_T x;

   /* Initialize serial number string to NULL */
   *ser_ptr = NULL;

   if( iface_map[port] == IFACE_TCP )
   {
      APHEADER_MANUFACTURE_STRUCT_T  mfg_info;
      /* Read EEPROM to get serial number */
      if( ecode = tkcmdset_read_eemem_block(port, APHEADER_MANUFACTURE_DATA_ADDRESS, (UINT8_T *)&mfg_info, sizeof(mfg_info) ) )
      {
      }
      /* Validate structure */
      else if( APHEADER_MANUFACTURE_INFO_VALID_M(&mfg_info) )
      {
         /* Copy serial number */
         for(x=0;x<sizeof(mfg_info.serial_num);x++)
         {
            ser_ptr[x] = mfg_info.serial_num[x];
         }
      }
   }
   else if((ecode = TKUSB_GET_STRING_M(port,
                                       HOSTCMD_SERIAL_NUMBER_STRING,
                                       rsp_buf,sizeof(rsp_buf))) == NO_ERR)
   {
      strcpy((char *) ser_ptr, (char *) rsp_buf);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_status

   About:      Reads status information from the device

   Inputs:     UINT8_T     port        Identifier for device

   Outputs:    UINT32_T *  stat_ptr    Location to store status information

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_status(UINT8_T port, UINT16_T *stat_ptr)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Initialize status info */
   *stat_ptr = 0;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_STATUS_CMD);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save results if no error */
   if(!ecode)
   {
      tkcmdset_get_num16(stat_ptr,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_run_diags

   About:      Runs one or all of the built in diagnostics

   Inputs:     UINT8_T     port        Identifier for device
               UINT8_T     test_num    Desired test number or FF for all

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_run_diags(UINT8_T port, UINT8_T test_num)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_DIAGS_CMD);
   tkcmdset_add_num8(test_num,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_flash_write_timeout);

   return ecode;
}

/****************************************************************************
   tkcmdset_special

   About:      Performs special operations based on the command

   Inputs:     UINT8_T     port        Identifier for device
               UINT16_T    cmd         Special command to execute
               UINT32_T    in_data     Value send with command

   Outputs:    UINT32_T *  out_data    Location to store value returned
                                       with response

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_special(UINT8_T port, UINT16_T cmd, UINT32_T in_data, UINT32_T *out_data)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Initialize status info */
   if(out_data)
      *out_data = 0;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_SPECIAL_CMD);
   tkcmdset_add_num16(cmd,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   tkcmdset_add_num32(in_data,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET + sizeof(cmd)]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_flash_write_timeout);

   /* Save results if no error */
   if(!ecode)
   {
      if(out_data)
      {
         tkcmdset_get_num32(out_data,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
      }
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_set_an_out

   About:   Controls an analog output
   
   Inputs:  UINT8_T  port       Identifier for device
            UINT16_T chan       Channel number of output
            UINT16_T val        Value to be set

   Outputs: ERRCODE_T   ecode   Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_set_an_out(UINT8_T port, UINT16_T chan, UINT16_T val)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_SETANOUT_CMD);
   tkcmdset_add_num8(LO_BYTE(chan),&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   tkcmdset_add_num16(val,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET + sizeof(UINT8_T)]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   return ecode;
}

/****************************************************************************
   tkcmdset_get_an_out

   About:   Reads the last value loaded on an analog output
   
   Inputs:  UINT8_T  port       Identifier for device
            UINT16_T chan       Channel number of output
            UINT16_T *val       Last value loaded

   Outputs: ERRCODE_T   ecode   Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_get_an_out(UINT8_T port, UINT16_T chan, UINT16_T *val)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Initialize the returned value */
   *val = 0;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GETANOUT_CMD);
   tkcmdset_add_num8(LO_BYTE(chan),&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save results if no error */
   if(!ecode)
   {
      tkcmdset_get_num16(val,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_get_an_in

   About:      Reads the current value on an analog input channel
   
   Inputs:  UINT8_T  port       Identifier for device
            UINT16_T chan       Channel number of the input
            INT16_T *val        Current value of analog input

   Outputs: ERRCODE_T   ecode   Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_get_an_in(UINT8_T port, UINT16_T chan, INT16_T *val)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Initialize the returned value */
   *val = 0;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GETANIN_CMD);
   tkcmdset_add_num8(LO_BYTE(chan),&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save results if no error */
   if(!ecode)
   {
      tkcmdset_get_num16((UINT16_T *) val,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_get_volts

   About:      Returns the voltage for the specified analog input channel
               in engineering units.
   
   Inputs:  UINT8_T  port       Identifier for device
            UINT16_T chan       Channel number of the input

   Outputs: FLOAT32_T *val      Analog voltage

   Returns: ERRCODE_T   ecode   Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_get_volts(UINT8_T port, UINT16_T chan, FLOAT32_T *val)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Initialize the returned value */
   *val = 0;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GETVOLTS_CMD);
   tkcmdset_add_num8(LO_BYTE(chan),&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save results if no error */
   if(!ecode)
   {
      tkcmdset_get_float32(val,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_set_dig_out

   About:   Controls a digital output channel
   
   Inputs:  UINT8_T   port       Identifier for device
            UINT16_T  chan       Channel number of the output
            BOOLEAN_T state      TRUE / FALSE state of output

   Outputs: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_set_dig_out(UINT8_T port, UINT16_T chan, BOOLEAN state)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_SETDIGOUT_CMD);
   tkcmdset_add_num8(LO_BYTE(chan),&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   tkcmdset_add_num8(state,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET + sizeof(UINT8_T)]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   return ecode;
}

/****************************************************************************
   tkcmdset_get_dig_out

   About:   Reads the last value loaded on a digital output
   
   Inputs:  UINT8_T  port       Identifier for device
            UINT16_T chan       Channel number of the output
            BOOLEAN *state      TRUE / FALSE state of the output

   Outputs: ERRCODE_T   ecode   Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_get_dig_out(UINT8_T port, UINT16_T chan, BOOLEAN *state)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Initialize the returned value */
   *state = FALSE;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GETDIGOUT_CMD);
   tkcmdset_add_num8(LO_BYTE(chan),&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save results if no error */
   if(!ecode)
   {
      tkcmdset_get_num8(state,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
      *state ? *state = TRUE : *state = FALSE;
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_get_dig_in

   About:   Reads the current value of a digital input
   
   Inputs:  UINT8_T  port       Identifier for device
            UINT16_T chan       Channel number of the input
            BOOLEAN *state      TRUE / FALSE state of the input

   Outputs: ERRCODE_T   ecode   Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_get_dig_in(UINT8_T port, UINT16_T chan, BOOLEAN *state)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Initialize the returned value */
   *state = FALSE;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GETDIGIN_CMD);
   tkcmdset_add_num8(LO_BYTE(chan),&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save results if no error */
   if(!ecode)
   {
      tkcmdset_get_num8(state,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
      *state ? *state = TRUE : *state = FALSE;
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_write_mem

   About:   Writes a 8-bit value to memory
   
   Inputs:  UINT8_T  port       Identifier for device
            UINT32_T addr       Address to write
            UINT8_T val        Value to be written

   Outputs: ERRCODE_T   ecode   Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_write_mem(UINT8_T port, UINT32_T addr, UINT8_T val)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_WRITEMEM_CMD);
   tkcmdset_add_num32(addr,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   tkcmdset_add_num8(val,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET + sizeof(addr)]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   return ecode;
}


/****************************************************************************
   tkcmdset_read_mem

   About:   Reads a 8-bit value from local memory
   
   Inputs:  UINT8_T  port       Identifier for device
            UINT32_T addr       Address to read
            UINT16_T *val       Current value

   Outputs: ERRCODE_T   ecode   Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_read_mem(UINT8_T port, UINT32_T addr, UINT8_T *val)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_READMEM_CMD);
   tkcmdset_add_num32(addr,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save results if no error */
   if(!ecode)
   {
      tkcmdset_get_num8(val,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_write_mem_block

   About:   Writes block of values to local memory
   
   Inputs:  UINT8_T  port       Identifier for device
            UINT32_T addr       Address to write
            UINT8_T  *val       Location of values to be written
            UINT16_T slen       Number of bytes to be written

   Outputs: ERRCODE_T   ecode   Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_write_mem_block(UINT8_T port, UINT32_T addr, UINT8_T *val, UINT16_T len)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT16_T block_len;
   UINT16_T max_len;
   UINT8_T *block_ptr;
   UINT32_T i;
   UINT32_T msg_time = 0;

   /* Validate the port number */
#if 0
   if(ecode = TKUSB_VALIDATE_PORT_M(port))
   {
   }
   else
#endif
   {
      /* Get the size of the bigger output report */
      max_len = TKCMDSET_LONG_CMD_LEN(port);

      /* Adjust maximum length for header information */
      max_len -= HOSTCMD_CMD_DATA_OFFSET + sizeof(addr) + sizeof(len);

      while(len)
      {
         /* Build the command */
         cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_LONG_CMD_ID(port);
         rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
         cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_WRITEMEMBLK_CMD);

         /* Add the address information to the report */
         block_ptr = tkcmdset_add_num32(addr,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

         /* Add the length information to the report */
         (len < max_len) ? block_len = len : block_len = max_len;
         block_ptr = tkcmdset_add_num16((UINT16_T) block_len,block_ptr);

         for(i = 0; i < block_len; i++)
         {
            block_ptr = tkcmdset_add_num8(*val++,block_ptr);
         }

         /* Send out a progress message to Windows */
         if(TKTIME_ELAPSED_TIME_MS_M(msg_time) > TKCMDSET_PROGRESS_MESSAGE_TIME)
         {
            PostMessage(GetActiveWindow(),TKCMDSET_PROGRESS_MESSAGE,TKCMDSET_WRITE_RAM_DATA,addr);
            msg_time = TKTIME_GETTIME_M();
         }

         /* Execute the command */
         if(ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout))
         {
            break;
         }

         /* Setup for next block */
         len -= block_len;
         addr += block_len;
      }
   }
   return ecode;
}

/****************************************************************************
   tkcmdset_read_mem_block

   About:      Reads a block of values from local memory
   
   Inputs:  UINT8_T  port       Identifier for device
            UINT32_T addr       Address to read
            UINT8_T  *val       Location where values should be stored
            UINT16_T slen       Number of bytes to read

   Outputs: ERRCODE_T   ecode   Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_read_mem_block(UINT8_T port, UINT32_T addr, UINT8_T *val, UINT16_T len)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT16_T block_len;
   UINT16_T max_len;
   UINT8_T *block_ptr;
   UINT32_T i;
   UINT32_T msg_time = 0;

#if 0
   /* Validate the port number */
   if(ecode = TKUSB_VALIDATE_PORT_M(port))
   {
   }
   else
#endif
   {
      /* Get the size of the bigger input report */
      max_len = TKCMDSET_LONG_RSP_LEN(port);

      /* Adjust maximum length for header information */
      max_len -= HOSTCMD_HEADERSIZE + HOSTCMD_BLOCK_HEADER_SIZE;

      while(len)
      {
         /* Build the command */
         cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
         rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_LONG_RSP_ID(port);
         cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_READMEMBLK_CMD);

         /* Add the address information to the report */
         block_ptr = tkcmdset_add_num32(addr,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

         /* Add the length information to the report */
         (len < max_len) ? block_len = len : block_len = max_len;
         block_ptr = tkcmdset_add_num16((UINT16_T) block_len,block_ptr);

         /* Send out a progress message to Windows */
         if(TKTIME_ELAPSED_TIME_MS_M(msg_time) > TKCMDSET_PROGRESS_MESSAGE_TIME)
         {
            PostMessage(GetActiveWindow(),TKCMDSET_PROGRESS_MESSAGE,TKCMDSET_READ_RAM_DATA,addr);
            msg_time = TKTIME_GETTIME_M();
         }

         /* Execute the command */
         if(ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout))
         {
            break;
         }
         /* Copy this block to the response */
         else
         {
            block_ptr = &rsp_buf[HOSTCMD_RSP_DATA_OFFSET];
            for(i = 0; i < block_len; i++)
            {
               block_ptr = tkcmdset_get_num8(val++,block_ptr);
            }
         }

         /* Setup for next block */
         len -= block_len;
         addr += block_len;
      }
   }
   return ecode;
}


/****************************************************************************
   tkcmdset_write_eemem_block

   About:      Writes block of values to external EE memory
   
   Inputs:  UINT8_T  port       Identifier for device
            UINT16_T addr       Address in EEPROM to write
            UINT8_T  *val       Location of values to be written
            UINT16_T slen       Number of bytes to be written

   Outputs: ERRCODE_T   ecode   Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_write_eemem_block(UINT8_T port, UINT16_T addr, UINT8_T *val, UINT16_T len)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT16_T block_len;
   UINT16_T max_len;
   UINT8_T *block_ptr;
   UINT32_T i;
   UINT32_T msg_time = 0;

   /* Validate the port number */
#if 0
   if(ecode = TKUSB_VALIDATE_PORT_M(port))
   {
   }
   else
#endif
   {
      /* Get the size of the bigger output report */
      max_len = TKCMDSET_LONG_CMD_LEN(port);

      /* Adjust maximum length for header information */
      max_len -= HOSTCMD_HEADERSIZE + HOSTCMD_BLOCK_HEADER_SIZE;

      while(len)
      {
         /* Build the command */
         cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_LONG_CMD_ID(port);
         rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
         cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_WRITEEEMEMBLK_CMD);

         /* Add the address information to the report */
         block_ptr = tkcmdset_add_num32(addr,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

         /* Add the length information to the report */
         (len < max_len) ? block_len = len : block_len = max_len;
         block_ptr = tkcmdset_add_num16((UINT16_T) block_len,block_ptr);

         for(i = 0; i < block_len; i++)
         {
            block_ptr = tkcmdset_add_num8(*val++,block_ptr);
         }

         /* Send out a progress message to Windows */
         if(TKTIME_ELAPSED_TIME_MS_M(msg_time) > TKCMDSET_PROGRESS_MESSAGE_TIME)
         {
            PostMessage(GetActiveWindow(),TKCMDSET_PROGRESS_MESSAGE,TKCMDSET_WRITE_EE_DATA,addr);
            msg_time = TKTIME_GETTIME_M();
         }

         /* Execute the command */
         if(ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout))
         {
            break;
         }

         /* Setup for next block */
         len -= block_len;
         addr += block_len;
      }
   }
   return ecode;
}


/****************************************************************************
   tkcmdset_read_eemem_block

   About:      Reads a block of values from external EE memory
   
   Inputs:  UINT8_T  port       Identifier for device
            UINT16_T addr       Address in EEPROM to read
            UINT16_T *val       Location where values should be stored
            UINT16_T slen       Number of bytes to read

   Outputs: ERRCODE_T   ecode   Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_read_eemem_block(UINT8_T port, UINT16_T addr, UINT8_T *val, UINT16_T len)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT16_T block_len;
   UINT16_T max_len;
   UINT8_T *block_ptr;
   UINT32_T i;
   UINT32_T msg_time = 0;

#if 0
   /* Validate the port number */
   if(ecode = TKUSB_VALIDATE_PORT_M(port))
   {
   }
   else
#endif
   {
      /* Get the size of the bigger input report */
      max_len = TKCMDSET_LONG_RSP_LEN(port);

      /* Adjust maximum length for header information */
      max_len -= HOSTCMD_RSP_DATA_OFFSET;

      while(len)
      {
         /* Build the command */
         cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
         rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_LONG_RSP_ID(port);
         cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_READEEMEMBLK_CMD);

         /* Add the address information to the report */
         block_ptr = tkcmdset_add_num32(addr,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

         /* Add the length information to the report */
         (len < max_len) ? block_len = len : block_len = max_len;
         block_ptr = tkcmdset_add_num16((UINT16_T) block_len,block_ptr);

         /* Send out a progress message to Windows */
         if(TKTIME_ELAPSED_TIME_MS_M(msg_time) > TKCMDSET_PROGRESS_MESSAGE_TIME)
         {
            PostMessage(GetActiveWindow(),TKCMDSET_PROGRESS_MESSAGE,TKCMDSET_READ_EE_DATA,addr);
            msg_time = TKTIME_GETTIME_M();
         }

         /* Execute the command */
         if(ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout))
         {
            break;
         }
         /* Copy this block to the response */
         else
         {
            block_ptr = &rsp_buf[HOSTCMD_RSP_DATA_OFFSET];
            for(i = 0; i < block_len; i++)
            {
               block_ptr = tkcmdset_get_num8(val++,block_ptr);
            }
         }

         /* Setup for next block */
         len -= block_len;
         addr += block_len;
      }
   }
   return ecode;
}


/****************************************************************************
   tkcmdset_execute

   About:      Causes the code to execute a function call to the specified address

   Inputs:     UINT8_T     port        Identifier for device
               UINT32_T    addr        Address of function
               UINT32_T    in_data     Value passed to function

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_execute(UINT8_T port, UINT32_T addr)
{
   ERRCODE_T ecode = 0;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_EXECUTE_CMD);
   tkcmdset_add_num32(addr,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   return ecode;
}

/****************************************************************************
   tkcmdset_set_pulse

   About:   Configures pulse characteristics for the selected drive channels
   
   Inputs:  UINT8_T   port       Identifier for device
            UINT8_T   chan       Bitmask of selected drive channels
            UINT32_T  width      Pulse width in microseconds
            UINT16_T  delay      Pulse delay in microseconds
            FLOAT32_T current    Pulse current in amps

   Outputs: None
   
   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_set_pulse(UINT8_T port, UINT8_T chan, 
                                                UINT32_T width, UINT16_T delay, FLOAT32_T current)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_SET_PULSE_EX_CMD);
   bufptr = tkcmdset_add_num8(chan,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   bufptr = tkcmdset_add_num32(width,bufptr);
   bufptr = tkcmdset_add_num16(delay,bufptr);
   bufptr = tkcmdset_add_float32(current,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,5000);

   return ecode;
}

/****************************************************************************
   tkcmdset_get_pulse

   About:   Retrieves pulse characteristics for the selected drive channel
   
   Inputs:  UINT8_T    port      Identifier for device
            UINT8_T    chan      Selected drive channel
   
   Outputs: UINT32_T*  width     Active pulse width in microseconds
            UINT16_T*  delay     Active pulse delay in microseconds
            FLOAT32_T* current   Active pulse current in amps

   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_get_pulse(UINT8_T port, UINT8_T chan, 
                                                UINT32_T* width, UINT16_T* delay, FLOAT32_T* current)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Initialize the returned values */
   *width = 0;
   *delay = 0;
   *current = 0;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GET_PULSE_EX_CMD);
   bufptr = tkcmdset_add_num8(chan,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      bufptr = tkcmdset_get_num8(&chan,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
      bufptr = tkcmdset_get_num32(width,bufptr);
      bufptr = tkcmdset_get_num16(delay,bufptr);
      bufptr = tkcmdset_get_float32(current,bufptr);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_set_repeat

   About:   Configures pulse repeat characteristics for the selected drive channels
   
   Inputs:  UINT8_T   port       Identifier for device
            UINT8_T   chan       Bitmask of selected drive channels
            UINT32_T  offtime    Off time between pulses
            UINT16_T  count      Number of pulses per trigger

   Outputs: None
   
   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_set_repeat(UINT8_T port, UINT8_T chan, 
                                                UINT32_T offtime, UINT16_T count)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_SET_REPEAT_CMD);
   bufptr = tkcmdset_add_num8(chan,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   bufptr = tkcmdset_add_num32(offtime,bufptr);
   bufptr = tkcmdset_add_num16(count,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,5000);

   return ecode;
}

/****************************************************************************
   tkcmdset_get_repeat

   About:   Retrieves pulse characteristics for the selected drive channel
   
   Inputs:  UINT8_T    port      Identifier for device
            UINT8_T    chan      Selected drive channel
   
   Outputs: UINT32_T*  offtime   Active off time between pulsesi n microseconds
            UINT16_T*  count     Active number of pulses per trigger

   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_get_repeat(UINT8_T port, UINT8_T chan, 
                                                 UINT32_T* offtime, UINT16_T* count)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Initialize the returned values */
   *offtime = 0;
   *count = 0;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GET_REPEAT_CMD);
   bufptr = tkcmdset_add_num8(chan,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      bufptr = tkcmdset_get_num32(offtime,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
      bufptr = tkcmdset_get_num16(count,bufptr);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_set_trigger_map

   About:   Configures trigger characteristics for the selected trigger channel.
            A trigger can be mapped to multiple drive channels.
   
   Inputs:  UINT8_T   port       Identifier for device
            UINT8_T   trig       Selected trigger channel
            UINT8_T   chan       Bitmask of selected drive channels
            UINT8_T   rising     TRUE for rising edge triggers

   Outputs: None
   
   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_set_trigger_map(UINT8_T port, UINT8_T trig, UINT8_T chan, UINT8_T rising)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_SET_TRIG_MAP_CMD);
   bufptr = tkcmdset_add_num8(trig,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   bufptr = tkcmdset_add_num8(chan,bufptr);
   bufptr = tkcmdset_add_num8(rising,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   return ecode;
}


/****************************************************************************
   tkcmdset_set_gate_map

   About:   Configures gate characteristics for the selected channel.
            This function is used in continuous mode only.
            A gate can be mapped to multiple drive channels.
   
   Inputs:  UINT8_T   port       Identifier for device
            UINT8_T   gate       Selected gate channel
            UINT8_T   chan       Bitmask of selected drive channels
            UINT8_T   rising     TRUE for rising edge triggers

   Outputs: None
   
   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_set_gate_map(UINT8_T port, UINT8_T gate, UINT8_T chan, UINT8_T rising)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_SET_GATE_MAP_CMD);
   bufptr = tkcmdset_add_num8(gate,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   bufptr = tkcmdset_add_num8(chan,bufptr);
   bufptr = tkcmdset_add_num8(rising,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   return ecode;
}

/****************************************************************************
   tkcmdset_get_trigger_map

   About:   Retrieves trigger characteristics for the selected trigger channel
   
   Inputs:  UINT8_T    port      Identifier for device
            UINT8_T    trig      Selected trigger channel
   
   Outputs: UINT8_T*   chan      Active channel bitmask for this trigger
            UINT8_T*   rising    TRUE if rising edge triggered

   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_get_trigger_map(UINT8_T port, UINT8_T trig, UINT8_T* chan, UINT8_T* rising)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Initialize the returned values */
   *chan = 0;
   *rising = 0;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GET_TRIG_MAP_CMD);
   bufptr = tkcmdset_add_num8(trig,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      bufptr = tkcmdset_get_num8(&trig,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
      bufptr = tkcmdset_get_num8(chan,bufptr);
      bufptr = tkcmdset_get_num8(rising,bufptr);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_set_hv_supply

   About:   Adjusts the setpoint the High Voltage String Supply for the specified channels
   
   Inputs:  UINT8_T   port       Identifier for device
            UINT8_T   chan       Bitmask of channels to set voltage
            FLOAT32_T voltage    Desired voltage of specified channel

   Outputs: None
   
   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_set_hv_supply(UINT8_T port, UINT8_T chan, FLOAT32_T voltage)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_SET_HV_SUPPLY_CMD);
   bufptr = tkcmdset_add_num8(chan,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   tkcmdset_add_float32(voltage,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,5000);

   return ecode;
}

/****************************************************************************
   tkcmdset_get_hv_supply

   About:   Retrieves the active setpoint a channel or the High Voltage String Supply
   
   Inputs:  UINT8_T    port      Identifier for device
            UINT8_T    chan      Desired channel voltage to get, zero for actual supply voltage

   Outputs: FLOAT32_T *voltage   Active channel or HV Supply voltage

   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_get_hv_supply(UINT8_T port, UINT8_T chan, FLOAT32_T *voltage)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Initialize the returned value */
   *voltage = 0;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GET_HV_SUPPLY_CMD);
   tkcmdset_add_num8(chan,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      tkcmdset_get_float32(voltage,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_set_lv_supply

   About:   Adjusts the setpoint the Low Voltage String Supply for the specified channels
   
   Inputs:  UINT8_T   port       Identifier for device
            UINT8_T   chan       Bitmask of channels to set voltage
            FLOAT32_T voltage    Desired voltage of specified channels

   Outputs: None
   
   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_set_lv_supply(UINT8_T port, UINT8_T chan, FLOAT32_T voltage)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_SET_LV_SUPPLY_CMD);
   bufptr = tkcmdset_add_num8(chan,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   tkcmdset_add_float32(voltage,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,5000);

   return ecode;
}

/****************************************************************************
   tkcmdset_get_lv_supply

   About:   Retrieves the active setpoint a channel or the Low Voltage String Supply
   
   Inputs:  UINT8_T    port      Identifier for device
            UINT8_T    chan      Desired channel voltage to get, zero for actual supply voltage

   Outputs: FLOAT32_T *voltage   Active channel or LV Supply voltage

   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_get_lv_supply(UINT8_T port, UINT8_T chan, FLOAT32_T *voltage)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Initialize the returned value */
   *voltage = 0;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GET_LV_SUPPLY_CMD);
   tkcmdset_add_num8(chan,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      tkcmdset_get_float32(voltage,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_trigger

   About:   Generates a software trigger on the selected trigger channels
   
   Inputs:  UINT8_T   port       Identifier for device
            UINT8_T   trig       Bitmask of triggers to activate

   Outputs: None
   
   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_trigger(UINT8_T port, UINT8_T trig)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_TRIGGER_CMD);
   tkcmdset_add_num8(trig,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   return ecode;
}

/****************************************************************************
   tkcmdset_set_pulse_out

   About:   Sets the state of multiple pulse channel outputs
   
   Inputs:  UINT8_T   port       Identifier for device
            UINT8_T   chan       Bitmask of channels to set
            BOOLEAN_T state      Desired state of outputs

   Outputs: None
   
   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_set_pulse_out(UINT8_T port, UINT8_T chan, BOOLEAN_T state)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_SET_PULSE_OUT_CMD);
   bufptr = tkcmdset_add_num8(chan,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   bufptr = tkcmdset_add_num8(state,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   return ecode;
}

/****************************************************************************
   tkcmdset_get_pulse_out

   About:   Gets the state of a pulse channel output
   
   Inputs:  UINT8_T   port       Identifier for device
            UINT8_T   chan       Channel to get
            
   Outputs: BOOLEAN_T *state     Location to store state of channel
   
   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T   tkcmdset_get_pulse_out(UINT8_T port, UINT8_T chan, BOOLEAN_T *state)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GET_PULSE_OUT_CMD);
   tkcmdset_add_num8(chan,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      tkcmdset_get_num8(state,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return ecode;
}

/*******************************************************************************
   tkcmdset_calc_led_temp

   About:   Calculates the expected LED junction temperature based on the
            information stored in the lighthead data structure and the width
            and current information passed.

   Inputs:  UINT8_T     port        Identifier for device
            UINT8_T     ch          Channel to compute LED temperature
            UINT32_T    width       Pulse width in microseconds
            FLOAT32_T   current     Pulse current in amps
   
   Outputs: FLOAT32_T  *temp        Location to store calculated temperature

   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
********************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_calc_led_temp(UINT8_T port, UINT8_T ch, 
                                                  UINT32_T width, FLOAT32_T current, FLOAT32_T *temp)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_CALC_LED_TEMP_EX_CMD);
   bufptr = tkcmdset_add_num8(ch,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   bufptr = tkcmdset_add_num32(width,bufptr);
   bufptr = tkcmdset_add_float32(current,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      tkcmdset_get_float32(temp,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return(ecode);
}

/*******************************************************************************
   tkcmdset_calc_max_width

   About:   Calculates the maximum pulse width for the selected channel based 
            on the information stored in the lighthead data structure and the 
            current information passed.

   Inputs:  UINT8_T     port        Identifier for device
            UINT8_T     ch          Channel to compute LED temperature
            FLOAT32_T   current     Pulse current in amps
   
   Outputs: UINT32_T   *width       Maximum pulse width in microseconds

   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
********************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_calc_max_width(UINT8_T port, UINT8_T ch, 
                                                   FLOAT32_T current, UINT32_T * width)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_CALC_MAX_WIDTH_EX_CMD);
   bufptr = tkcmdset_add_num8(ch,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   bufptr = tkcmdset_add_float32(current,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      tkcmdset_get_num32(width,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return(ecode);
}

/*******************************************************************************
   tkcmdset_calc_max_current

   About:   Calculates the maximum pulse current for the selected channel based 
            on the information stored in the lighthead data structure and the 
            width information passed.

   Inputs:  UINT8_T     port        Identifier for device
            UINT8_T     ch          Channel to compute LED temperature
            UINT32_T    width       Pulse width in microseconds
   
   Outputs: FLOAT32_T  *current     Maximum pulse current in amps

   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
********************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_calc_max_current(UINT8_T port, UINT8_T ch, 
                                                     UINT32_T width, FLOAT32_T *current)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_CALC_MAX_CURRENT_EX_CMD);
   bufptr = tkcmdset_add_num8(ch,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   bufptr = tkcmdset_add_num32(width,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      tkcmdset_get_float32(current,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return(ecode);
}

/*******************************************************************************
   tkcmdset_calc_supply_voltage

   About:   Calculates the required supply voltage for a given width and current.

   Inputs:  UINT8_T     port        Identifier for device
            UINT8_T     ch          Channel to compute required supply voltage
            UINT32_T    width       Pulse width in microseconds
            FLOAT32_T   current     Pulse current in amps
   
   Outputs: FLOAT32_T  *voltage     Location to store calculated voltage

   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
********************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_calc_supply_voltage(UINT8_T port, UINT8_T ch, 
                                                        UINT32_T width, FLOAT32_T current, FLOAT32_T *voltage)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_CALC_SUPPLY_VOLTAGE_EX_CMD);
   bufptr = tkcmdset_add_num8(ch,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   bufptr = tkcmdset_add_num32(width,bufptr);
   bufptr = tkcmdset_add_float32(current,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      tkcmdset_get_float32(voltage,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return(ecode);
}

/*******************************************************************************
   tkcmdset_calc_max_freq

   About:   Calculates the maximum frequency allowed for a given width and current.
            This function computes maximum frequency based on the maximum power of
            the power supply and also from the LED cooling equations.  The lower
            value is returned.

   Inputs:  UINT8_T     port        Identifier for device
            UINT8_T     ch          Channel to compute maximum frequency
            UINT32_T    width       Pulse width in microseconds
            FLOAT32_T   current     Pulse current in amps
   
   Outputs: UINT16_T   *freq        Location to store calculated frequency

   Returns: ERRCODE_T ecode      Status of command
                                   Zero if successful,
                                   Error code otherwise
  
********************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_calc_max_freq(UINT8_T port, UINT8_T ch, 
                                                  UINT32_T width, FLOAT32_T current, UINT16_T *freq)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_CALC_MAX_FREQ_EX_CMD);
   bufptr = tkcmdset_add_num8(ch,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   bufptr = tkcmdset_add_num32(width,bufptr);
   bufptr = tkcmdset_add_float32(current,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      tkcmdset_get_num16(freq,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
   }

   return(ecode);
}

/****************************************************************************
   tkcmdset_set_lighthead_data

   About:      Loads the supplied data into the lighthead EEPROM

   Inputs:     UINT8_T     port        Communication port to use
               TKCMDSET_LIGHTHEAD_STRUCT_T * ap_data
                                       All lighthead data

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_set_lighthead_data(UINT8_T port,
                                                       TKCMDSET_LIGHTHEAD_STRUCT_T * ap_data)
{
   ERRCODE_T ecode = NO_ERR;
   UINT32_T crc_len;
   UINT16_T crc;
   APHEADER_LIGHTHEAD_STRUCT_T lh_info;
   long ltime;
   struct tm *now;

   time(&ltime);
   now = localtime(&ltime);

   UINT16_T day = now->tm_yday;
   UINT16_T year = 1900 + now->tm_year;

   /* Build the target structure */
   lh_info.struct_type = SWAP_UINT16_M(APHEADER_LIGHTHEAD_TYPE);
   lh_info.struct_rev  = SWAP_UINT16_M(APHEADER_LIGHTHEAD_STRUCT_REV);
   lh_info.active_channels = 0xF;
   lh_info.allowed_modes = 0x7;
   strftime((char *) lh_info.mfg_date,sizeof(lh_info.mfg_date),"%#d-%b-%Y %#H:%M",now);
   memset(lh_info.reserved,0,sizeof(lh_info.reserved));

   /* Add the supplied data */
   memcpy(lh_info.part_number,ap_data->part_number,sizeof(lh_info.part_number));
   for(int i = 0; i < 4; i++)
   {
      lh_info.chan[i].led_r_th = SWAP_UINT16_M((UINT16_T) (ap_data->chan[i].led_r_th * 10));
      lh_info.chan[i].tau  = SWAP_UINT16_M((UINT16_T) (ap_data->chan[i].tau * 1000));
      lh_info.chan[i].led_voltage_slope = SWAP_FLOAT32_M(ap_data->chan[i].led_voltage_slope);
      lh_info.chan[i].led_voltage_offset = SWAP_UINT16_M((UINT16_T) (ap_data->chan[i].led_voltage_offset * 1000));
      lh_info.chan[i].max_junct_temp = (UINT8_T) ap_data->chan[i].max_junct_temp;
      lh_info.chan[i].num_str_led = (UINT8_T) (ap_data->chan[i].num_str << 4) | 
                                    (UINT8_T) (ap_data->chan[i].num_led & 0xF);
      lh_info.chan[i].hs_r_th = SWAP_UINT16_M((UINT16_T) (ap_data->chan[i].hs_r_th * 10));
      lh_info.chan[i].default_current = SWAP_UINT16_M((UINT16_T) (ap_data->chan[i].default_current * 1000));
      lh_info.chan[i].max_pulse_current = SWAP_UINT16_M((UINT16_T) (ap_data->chan[i].max_pulse_current * 1000));
      lh_info.chan[i].max_cont_current = SWAP_UINT16_M((UINT16_T) (ap_data->chan[i].max_cont_current * 1000));
   }

   /* Determine the length of bytes for the structure CRC value */
   /* Include all bytes in the structure except the CRC itself  */
   crc_len = sizeof(APHEADER_LIGHTHEAD_STRUCT_T) - sizeof(lh_info.struct_crc);

   /* Calculate the CRC-16 value for the structure */
   TKCRC_CALC16_M((UINT8_T *) &lh_info + sizeof(lh_info.struct_crc),crc_len,&crc);

   /* Update the header information */
   lh_info.struct_crc = SWAP_UINT16_M(crc);

   /* Store the structure in EEPROM */
   if(ecode = tkcmdset_write_eemem_block(port,
                                         APHEADER_LIGHTHEAD_DATA_ADDRESS,
                                         (UINT8_T *) &lh_info,
                                         sizeof(APHEADER_LIGHTHEAD_STRUCT_T)))
   {
   }

   return(ecode);
}

/****************************************************************************
   tkcmdset_get_lighthead_data

   About:      Retrieves the data from the lighthead EEPROM

   Inputs:     UINT8_T     port        Communication port to use
               
   Outputs:    TKCMDSET_LIGHTHEAD_STRUCT_T * ap_data
                                       All lighthead data

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_get_lighthead_data(UINT8_T port,
                                                       TKCMDSET_LIGHTHEAD_STRUCT_T * ap_data)
{
   ERRCODE_T ecode = NO_ERR;
   APHEADER_LIGHTHEAD_STRUCT_T lh_info;

   /* Read the lighthead configuration information */
   if(ecode = tkcmdset_read_eemem_block(port,
                                        APHEADER_LIGHTHEAD_DATA_ADDRESS,
                                        (UINT8_T *) &lh_info,
                                        sizeof(APHEADER_LIGHTHEAD_STRUCT_T)))
   {
   }
   /* Load the data if it is valid */
   else if(APHEADER_LIGHTHEAD_INFO_VALID_M(&lh_info))
   {
      /* Build the returned structure */
      memcpy(ap_data->part_number,lh_info.part_number,sizeof(ap_data->part_number));
      memcpy(ap_data->mfg_date,lh_info.mfg_date,sizeof(ap_data->mfg_date));
      for(int i = 0; i < 4; i++)
      {
         ap_data->chan[i].led_r_th = (FLOAT32_T) SWAP_UINT16_M(lh_info.chan[i].led_r_th) / 10;
         ap_data->chan[i].tau = (FLOAT32_T) SWAP_UINT16_M(lh_info.chan[i].tau) / 1000;
         ap_data->chan[i].led_voltage_slope = SWAP_FLOAT32_M(lh_info.chan[i].led_voltage_slope);
         ap_data->chan[i].led_voltage_offset = (FLOAT32_T) SWAP_UINT16_M(lh_info.chan[i].led_voltage_offset) / 1000;
         ap_data->chan[i].max_junct_temp = lh_info.chan[i].max_junct_temp;
         ap_data->chan[i].num_str = lh_info.chan[i].num_str_led >> 4;
         ap_data->chan[i].num_led = lh_info.chan[i].num_str_led & 0xF;
         ap_data->chan[i].hs_r_th = (FLOAT32_T) SWAP_UINT16_M(lh_info.chan[i].hs_r_th) / 10;
         ap_data->chan[i].default_current = (FLOAT32_T) SWAP_UINT16_M(lh_info.chan[i].default_current) / 1000;
         ap_data->chan[i].max_pulse_current = (FLOAT32_T) SWAP_UINT16_M(lh_info.chan[i].max_pulse_current) / 1000;
         ap_data->chan[i].max_cont_current = (FLOAT32_T) SWAP_UINT16_M(lh_info.chan[i].max_cont_current) / 1000;
      }
   }
   /* Bad CRC */
   else
   {
      ecode = TKCMDSET_INVALID_LIGHTHEAD_CRC_ERR;
   }

   return(ecode);
}

/****************************************************************************
   tkcmdset_set_ip

   About:      Sets the current IP address

   Inputs:     UINT32_T    ip          IP address
               UINT32_T    subnet      Subnet address
               UINT32_T    gateway     Gateway address

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_set_ip(UINT8_T port,
                                           UINT32_T ip,
                                           UINT32_T subnet,
                                           UINT32_T gateway)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;
   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_SET_IP_CMD);
   bufptr = tkcmdset_add_num32(ip,&cmd_buf[HOSTCMD_CMD_DATA_OFFSET]);
   bufptr = tkcmdset_add_num32(subnet,bufptr);
   bufptr = tkcmdset_add_num32(gateway,bufptr);

   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   return ecode;
}

/****************************************************************************
   tkcmdset_get_ip

   About:      Retrieves the current IP address of the unit

   Inputs:     None
               
   Outputs:    UINT32_T*   ip          IP address
               UINT32_T*   subnet      Subnet address
               UINT32_T*   gateway     Gateway address

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_get_ip(UINT8_T port,
                                           UINT32_T* ip,
                                           UINT32_T* subnet,
                                           UINT32_T* gateway)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;
   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GET_IP_CMD);
   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      bufptr = tkcmdset_get_num32(ip,&rsp_buf[HOSTCMD_RSP_DATA_OFFSET]);
      bufptr = tkcmdset_get_num32(subnet,bufptr);
      bufptr = tkcmdset_get_num32(gateway,bufptr);
   }

   return ecode;
}

/****************************************************************************
   tkcmdset_set_mac

   About:      Sets the current MAC address of the unit

   Inputs:     UINT8_T*    mac_addr    6-byte array containing 48-bit MAC
               
   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_set_mac(UINT8_T port,
                                            UINT8_T* mac_addr)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;
   UINT16_T i;

   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_SET_MAC_CMD);
   bufptr = &cmd_buf[HOSTCMD_CMD_DATA_OFFSET];
   for( i=0; i<TKCMDSET_MAC_ADDR_LEN; i++)
   {
      bufptr = tkcmdset_add_num8(mac_addr[i],bufptr);
   }
 
   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   return ecode;
}

/****************************************************************************
   tkcmdset_get_mac

   About:      Retrieves the current MAC address of the unit

   Inputs:     None
               
   Outputs:    UINT8_T*    mac_addr    6-byte array to contain 48-bit MAC

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_get_mac(UINT8_T port,
                                            UINT8_T* mac_addr)
{
   ERRCODE_T ecode = NO_ERR;
   UINT8_T cmd_buf[TKUSB_MAX_PACKET_SIZE];
   UINT8_T rsp_buf[TKUSB_MAX_PACKET_SIZE] = {0};
   UINT8_T *bufptr;
   UINT8_T i;
   /* Build the command */
   cmd_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_CMD_ID(port);
   rsp_buf[HOSTCMD_REPORT_ID_OFFSET] = TKCMDSET_SHORT_RSP_ID(port);
   cmd_buf[HOSTCMD_CMD_ID_OFFSET] = LO_BYTE(HOSTCMD_GET_MAC_CMD);
   /* Execute the command */
   ecode = TKCMDSET_DO_COMMAND_M(port,cmd_buf,rsp_buf,tkcmdset_default_timeout);

   /* Save the results if no error */
   if(!ecode)
   {
      bufptr = &rsp_buf[HOSTCMD_RSP_DATA_OFFSET];
      for( i=0; i<TKCMDSET_MAC_ADDR_LEN; i++)
      {
         bufptr = tkcmdset_get_num8(&mac_addr[i],bufptr);
      }
   }

   return ecode;
}
/****************************************************************************
   tkcmdset_shutdown

   About:      Shuts down the Command Set Toolkit functions

   Inputs:     UINT8_T     port        Communication port to use

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
MPDH8USB_TCP_API ERRCODE_T tkcmdset_shutdown(UINT8_T port)
{
   return TKUSB_SHUTDOWN_M(port);
}

/*******************************************************************************
   tkcmdset_get_msg_data_len
   
   About:      This function determines the length of the data block sent with
               the specified command or response.  If the message identifier 
               is not found an error code is returned.
   
   Inputs:     UINT8_T     msg_id   Message identifier
               UINT8_T     is_cmd   True for command identifiers, false for response identifiers
   
   Outputs:    UINT16_T*   len      Location to store length of command or response data
   
   Returns:    ERRCODE_T   ecode    Zero if successful,
                                    otherwise error code
   
*******************************************************************************/
ERRCODE_T tkcmdset_get_msg_data_len(UINT8_T msg_id, UINT8_T is_cmd, UINT16_T *len) 
{
   ERRCODE_T ecode = NO_ERR;
   UINT16_T i;

   *len = 0;

   /* Search for a matching message ID */
   for(i = 0; i < LENGTH(tkcmdset_msg_info); i++)
   {
      ecode = TKCMDSET_INVALID_COMMAND_ERR;
      /* Check for match */
      if(msg_id == tkcmdset_msg_info[i].msg_id)
      {
         /* Return desired length information (Add 1 to RSP for command echo) */
         *len = (is_cmd ? tkcmdset_msg_info[i].cmd_len : tkcmdset_msg_info[i].rsp_len);
         ecode = NO_ERR;
         break;
      }
   }  

   return(ecode);
}
