/*******************************************************************************
********************************************************************************

   COPYRIGHT  2005, Nathaniel Group, Inc.

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Nathaniel Group.

   tk_crc.c       Interface for multiple CRC calculations

   About:         This file contains 8-bit and 16-bit CRC calculations
                  Review defines to configure table-driven or run-time
                  CRC calculation

   Target CPU:    N/A

   Written for:   Nathaniel Group
                  101 Panton Road, Suite 1
                  Vergennes, VT 05491-0173
                  http://www.nathaniel.com

   Contacts:      Joel Melnick
                  Phone:  (802) 877-2182
                  FAX:    (802) 877-3817
                  jmelnick@nathaniel.com

   Written by:    James Piette
                  Microprocessor Designs, Inc.
                  65 Longmeadow Drive
                  PO Box 160
                  Shelburne, VT 05482
                  http://www.updesigns.com

   $Revision: 1 $

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/Pulsar320/tar_src/H8App/tk_crc.h $
 * 
 * 1     8/15/08 8:45a Jpiet
 * Initial check-in
 * 
 * 1     5/12/06 2:38p Jpiet
 * 
 * 7     5/09/06 2:37p Jpiet
 * Fixed conditional for cam head loader.
 *
 * 6     5/09/06 2:18p Jpiet
 * Problems with conditionals.
 *
 * 5     5/09/06 2:05p Jpiet
 * Fixed conditionals to make files compatible with all builds (console,
 * camera and PC)
 *
 * 4     5/08/06 11:08a Jpiet
 * Added prototype cor crc16
 *
 * 3     5/08/06 9:26a Jpiet
 * Fixed problems with CRC16 function syntax
 *
 * 2     1/09/06 9:53a Jpiet
 * Most function implemented.  Working on EEPROM access interface.  Making
 * bootloader modification, so all shared files checked in from APP
 * changes.
 *
 * 1     12/30/05 5:31a Jpiet
********************************************************************************
*******************************************************************************/
#ifndef _TKCRC_H
#define _TKCRC_H

/*******************************************************************************
 *******************************************************************************
                     Definitions
********************************************************************************
*******************************************************************************/
/* These are used to configure the toolkit */
#define  TKCRC_CRC8            TRUE   //true to enable 8-bit crc functions
#define  TKCRC_CRC16           TRUE   //true to enable 16-bit crc functions
#define  TKCRC_USE_TABLES      TRUE  //true to enable lookup table implementation
#define  TKCRC_INCREMENTAL_CRC TRUE   //true to enable incremental CRC functions

#define TKCRC_CRC16_DEFAULT_COEFF         0xa001

/*******************************************************************************
 *******************************************************************************
                     Macros
********************************************************************************
*******************************************************************************/
#if TKCRC_CRC8
   #define TKCRC_CALC_CRC8_M(bufptr, numbytes) \
      tkcrc_calc_crc8(bufptr, numbytes)
   #if TKCRC_INCREMENTAL_CRC
      #define TKCRC_RESET_CRC8_M() \
         tkcrc_reset_crc8();
      #define TKCRC_ADD_CRC8_M(val) \
         tkcrc_add_crc8(val)
      #define TKCRC_GET_CRC8_M() \
         tkcrc_get_crc8()
   #endif
#endif

#if TKCRC_CRC16
#define TKCRC_ADD_VAL16_M(val)                                                \
   tkcrc_add_val16(val)
   
#define TKCRC_GET_CRC16_M(crc16)                                              \
   tkcrc_get_crc16(crc16)
   
#define TKCRC_CALC16_M(val_ptr,num_bytes,crc16)                               \
   tkcrc_calc16(val_ptr,num_bytes,crc16)

#if 0
#define TKCRC_CALC_CRC16_M(bufptr, numbytes) \
   tkcrc_calc_crc16(bufptr, numbytes)
   #if TKCRC_INCREMENTAL_CRC
      #define TKCRC_RESET_CRC16_M() \
         tkcrc_reset_crc16();
      #define TKCRC_ADD_CRC16_M(val) \
         tkcrc_add_crc16(val)
      #define TKCRC_GET_CRC16_M() \
         tkcrc_get_crc16()
   #endif
#endif

#endif
/*******************************************************************************
 *******************************************************************************
                      Prototypes
********************************************************************************
*******************************************************************************/
#ifdef __cplusplus
extern "C" {
#endif

#if TKCRC_CRC8
UINT8_T tkcrc_calc_crc8(UINT8_T *bufptr, UINT16_T num_bytes);
#if TKCRC_INCREMENTAL_CRC
void tkcrc_reset_crc8();
void tkcrc_add_crc8(UINT8_T val);
UINT8_T tkcrc_get_crc8();
#endif //TKCRC_INCREMENTAL_CRC
#endif //TKCRC_CRC8

#if TKCRC_CRC16
#ifdef __cplusplus
extern "C" {
#endif

ERRCODE_T tkcrc_init(UINT16_T coeff);
ERRCODE_T tkcrc_add_val16(UINT8_T val);
ERRCODE_T tkcrc_get_crc16(UINT16_T *crc16);
ERRCODE_T tkcrc_calc16(UINT8_T *val_ptr, UINT32_T num_bytes, UINT16_T *crc16);

#ifdef __cplusplus
}
#endif
#endif //TKCRC_CRC16

#ifdef __cplusplus
}
#endif


#endif /* _TKCRC_H */
