/*******************************************************************************
********************************************************************************

   COPYRIGHT  2004, Advanced Illumination, Inc. 

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Advanced Illumination.

   tk_tcp.cpp     Source code for the USB Host Interface Toolkit for the 
                  S1000 Strobe Controller.

   About:         This module contains the implementation of the USB Host
                  Interface Toolkit functions in the AICONS01 project.

   Target:        Win32 Compatible

   Written for:   Advanced Illumination, Inc.
                  24 Peavine Drive
                  Rochester, VT 05767
                  http://advancedillumination.com

   Contacts:      John Thrailkill
                  Phone:  (802) 767-3830
                  FAX:    (802) 767-3831
                  jthrailkill@advill.com

   Written by:    Rob Macklin
                  Microprocessor Designs, Inc.
                  65 Longmeadow Drive
                  PO Box 160
                  Shelburne, VT 05482
                  http://www.updesigns.com

   $Revision: 1 $

--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/Pulsar320/pc_src/MPDH8USB_TCP/tk_tcp.cpp $
 * 
 * 1     8/15/08 12:24p Jpiet
 * 
********************************************************************************
*******************************************************************************/
#include "stdafx.h"
#include "compiler.h"
#include "pc_error.h"
extern "C" {
}
#include <wchar.h>
#include "winsock.h"
#include "tk_tcp.h"
#include "tk_reg.h"
#include "tk_time.h"

/*************************************** MACROS ********************************/
#define TKTCP_GET_SOCKET_M(port) \
    tktcp_info[port].socket
#define TKTCP_SET_SOCKET_M(port, socket_val) \
    tktcp_info[port].socket = socket_val
#define TKTCP_GET_IPADDR_M(serial_num, addr_struct_ptr) \
	tktcp_get_ipaddr(serial_num, addr_struct_ptr)

/************************************ DEFINITIIONS ******************************/
#define TKTCP_MAX_PORTS     64
#define TKTCP_BUFFER_SIZE   512
/* Registry entry definitions */
#define TKTCP_REG_SUBKEY              "Software\\Advanced Illumination\\Pulsar 320\\Command Set Toolkit"
//#define TKTCP_REG_SUBKEY              "Software\\Advanced Illumination\\Pulsar 320\\IPMap"
#define TKTCP_PORT_ENTRY              "TCP Port"
#define TKTCP_DEFAULT_PORT            1234
#define TKTCP_HEADER_SOF			  0xF0
#define TKTCP_HEADER_DATA_OFFSET  TKTCP_HEADER_LEN
#if !defined(_WINSOCK2API_) 
// Winsock 2 header defines this, but Winsock 1.1 header doesn't.  In
// the interest of not requiring the Winsock 2 SDK which we don't really
// need, we'll just define this one constant ourselves.
#define SD_SEND 1
#endif

typedef struct
{
    SOCKET socket;
}TKTCP_INFO_T;

typedef enum
{
    TKTCP_STATE_INACTIVE,
    TKTCP_STATE_CONNECTED,
}TKTCP_STATE_E;

typedef enum 
{
	TKTCP_HEADER_SOF_OFFSET,
	TKTCP_HEADER_CMD_LEN_MSB_OFFSET,
	TKTCP_HEADER_CMD_LEN_LSB_OFFSET,
	TKTCP_HEADER_RSP_LEN_MSB_OFFSET,
	TKTCP_HEADER_RSP_LEN_LSB_OFFSET,
	TKTCP_HEADER_LEN,
}TKTCP_HEADER_OFFSET_E;

/************************************* GLOBALS *********************************/
/* Global variables unique to each process */
TKTCP_INFO_T tktcp_info[TKTCP_MAX_PORTS];
TKTCP_STATE_E	tktcp_state;
UINT8_T tktcp_rx_buf[TKTCP_BUFFER_SIZE];
UINT8_T tktcp_tx_buf[TKTCP_BUFFER_SIZE];

/*********************************** PROTOTYPES ********************************/
ERRCODE_T tktcp_get_ipaddr(INT8_T * serial_num, sockaddr_in* remote_addr);

/****************************************************************************
   tktcp_init

   About:      Initializes the USB Host Interface Toolkit functions

   Inputs:     UINT8_T     port        Identifier to associate with this connection
               INT8_T *    device_name Name of USB device
               INT8_T *    serial_num  Serial number string of desired device

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
ERRCODE_T tktcp_init(UINT8_T port, INT8_T * device_name, INT8_T * ip_addr)
{
    ERRCODE_T	ecode=NO_ERR;
    sockaddr_in remote_addr;
	WSAData		WSAData;
	int			error;
	int flag = 1;
	int flag2 = 0;

    /* Validate the port number */
    if(port >= TKTCP_MAX_PORTS)
    {
        ecode = TKTCP_INVALID_PORT_ERR;
    }   
    if (tktcp_state == TKTCP_STATE_INACTIVE)
    {
       // Initialize Winsock.
       if ( (error = WSAStartup (MAKEWORD(2,2), &WSAData)) != 0) 
       {
          MessageBox(NULL,"WSAStartup failed","Pulsar TCP Error",MB_OK);

       }
    }
    /* Get IP address of given serial number */
	if( ecode != NO_ERR )
	{
	}
    else if( ecode = TKTCP_GET_IPADDR_M(ip_addr, &remote_addr) )
    {
    }
    else
    {
        /* Create socket */
		SOCKET test_socket = socket(AF_INET, SOCK_STREAM, 0);
		if( test_socket == INVALID_SOCKET )
		{
			error = WSAGetLastError();
		}
        TKTCP_SET_SOCKET_M(port,socket(AF_INET, SOCK_STREAM, 0));
        if( TKTCP_GET_SOCKET_M(port) == INVALID_SOCKET )
        {
            ecode = TKTCP_SOCKET_ERR;
        }
        else if ((ecode = connect(TKTCP_GET_SOCKET_M(port), 
                         (sockaddr*)&remote_addr, 
                         sizeof(remote_addr) )) == SOCKET_ERROR) 
        {
            ecode = TKTCP_SOCKET_ERR;
        }
#if 1
		else if (ecode = setsockopt(TKTCP_GET_SOCKET_M(port),            /* socket affected */
                                 IPPROTO_TCP,     /* set option at TCP level */
                                 TCP_NODELAY,     /* name of option */
                                 (char *) &flag,  /* the cast is historical
                                                         cruft */
                                 sizeof(int)) < 0)    /* length of option value */
        {
            ecode = TKTCP_SOCKET_ERR;
        }
		else if ((ecode = setsockopt(TKTCP_GET_SOCKET_M(port),            /* socket affected */
                                 SOL_SOCKET,     /* set option at TCP level */
                                 SO_SNDBUF,     /* name of option */
                                 (char *) &flag2,  /* the cast is historical
                                                         cruft */
                                 sizeof(int))) < 0)    /* length of option value */
        {
            ecode = TKTCP_SOCKET_ERR;
        }
#endif

        if( ecode == NO_ERR )
        {
            tktcp_state = TKTCP_STATE_CONNECTED;
        }
		 else
		 {
            TKTCP_SET_SOCKET_M(port, INVALID_SOCKET);
		 }
    }
    
    return ecode;
}

/****************************************************************************
   tktcp_validate_port

   About:      Verifies the port identifier is valid for the current USB setup

   Inputs:     UINT8_T     port        Identifier to validate

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
ERRCODE_T tktcp_validate_port(UINT8_T port)
{
    ERRCODE_T ecode=NO_ERR;

    /* Validate the port number */
    if(port >= TKTCP_MAX_PORTS)
    {
        ecode = TKTCP_INVALID_PORT_ERR;
    }
    /* Make sure the socket has been initialized */
    else if( TKTCP_GET_SOCKET_M(port) == INVALID_SOCKET )
    {
        ecode = TKTCP_INVALID_PORT_ERR;
    }
    return ecode;
}

/****************************************************************************
   tktcp_write

   About:      Writes a packet of data to the USB device

   Inputs:     UINT8_T     port        Identifier to associate with this connection
               UINT8_T *   out_buf     Location of data to be transmitted
               UINT16_T    timeout     Duration in milliseconds to wait for transmit

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
ERRCODE_T tktcp_write(UINT8_T port, UINT8_T * out_buf, UINT16_T cmd_len, UINT16_T timeout)
{
    ERRCODE_T ecode = NO_ERR;
    OVERLAPPED Overlapped = {0};
	UINT16_T ret_val;

//	int nSockBuf = getsockopt( TKTCP_GET_SOCKET_M(port), SO_SNDBUF );


    /* Validate the port number */
    if( ecode = tktcp_validate_port(port) )
    {
        ecode = TKTCP_INVALID_PORT_ERR;
    }
    else
    {
#if 0
        /* Send packet header bytes */ 
        if(send(TKTCP_GET_SOCKET_M(port), (const char*)header_buf, TKTCP_HEADER_LEN, 0 ) == SOCKET_ERROR)
        {
            ecode = TKTCP_SOCKET_ERR; 
        }
		else
		{
			/* Create transmit buffer, add header and convert any SOF
			   characters in data stream */
			cmd_len += TKTCP_HEADER_LEN;
			for(i=TKTCP_HEADER_DATA_OFFSET; i<cmd_len; i++)
			{
				if( (*out_buf == TKTCP_HEADER_SOF) ||
					(*out_buf == TKTCP_SPECIAL_CHAR) )
				{
					tktcp_tx_buf[i++] = TKTCP_SPECIAL_CHAR;
					tktcp_tx_buf[i] = (*outbuf++) & (~TKTCP_SPECIAL_MASK);
					cmd_len+=1;
				}
				else
				{
					tktcp_tx_buf[i] = *outbuf++;
				}
			}

			tktcp_tx_buf[TKTCP_HEADER_SOF_OFFSET] = TKTCP_HEADER_SOF;
			tktcp_tx_buf[TKTCP_HEADER_CMD_LEN_MSB_OFFSET] = (UINT8_T)cmd_len>>8;
			tktcp_tx_buf[TKTCP_HEADER_CMD_LEN_LSB_OFFSET] = (UINT8_T)cmd_len&0xFF;
			tktcp_tx_buf[TKTCP_HEADER_RSP_LEN_MSB_OFFSET] = (UINT8_T)rsp_len>>8;
			tktcp_tx_buf[TKTCP_HEADER_RSP_LEN_LSB_OFFSET] = (UINT8_T)rsp_len&0xFF;
		}
        /* Write the packet to the socket */
        else
#endif		
			do{		
		if((ret_val = send(TKTCP_GET_SOCKET_M(port), (const char*)out_buf, cmd_len, 0 )) == SOCKET_ERROR)
        {
            ecode = TKTCP_SOCKET_ERR; 
        }
        else
        {
			cmd_len -= ret_val;
        }
		if( cmd_len != 0)
		{
			ecode = -1;
		}
			}while(cmd_len > 0);
    }
    return ecode;
}

/****************************************************************************
   tktcp_read

   About:      Reads a packet of data from the USB device

   Inputs:     UINT8_T     port        Identifier to associate with this connection
               UINT8_T *   in_buf      Location to store data received
               UINT16_T    timeout     Duration in milliseconds to wait for response

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
ERRCODE_T tktcp_read(UINT8_T port, UINT8_T * in_buf, UINT16_T len, UINT16_T timeout)
{
    ERRCODE_T ecode = NO_ERR;
	UINT16_T  bytes_available=0;
	UINT32_T	start_time;	



   struct timeval timev;

   fd_set rfds1;
   timev.tv_sec=0;
   FD_ZERO (&rfds1);
   timev.tv_usec=100;


    /* Validate the port number */
    if( ecode = tktcp_validate_port(port) )
    {
        ecode = TKTCP_INVALID_PORT_ERR;
    }
    else
    {
		start_time = TKTIME_GETTIME_M();
		/* Wait for specified number of bytes to be available */
		do
		{
         FD_ZERO (&rfds1);
         FD_SET (TKTCP_GET_SOCKET_M(port), &rfds1);
         if(select ((int)TKTCP_GET_SOCKET_M(port), &rfds1, NULL, NULL, &timev))
         {
			   if( (bytes_available = recv(TKTCP_GET_SOCKET_M(port), (char *)in_buf, len, MSG_PEEK ) ) == SOCKET_ERROR )
			   {
			   }
         }
			if( TKTIME_ELAPSED_TIME_MS_M(start_time) > timeout )
			{
				ecode = TKTCP_READ_TIMEOUT_ERR;
				break;
			}
		}while(bytes_available != len);
		/* Read data if no error */
		if( ecode )
		{
		}
		else if( (bytes_available = recv(TKTCP_GET_SOCKET_M(port), (char *)in_buf, len, 0) ) == SOCKET_ERROR )
		{
			ecode = TKTCP_SOCKET_ERR;
		}
	}
    return ecode;
}

/****************************************************************************
   tktcp_clear_rx

   About:      Clears the Winsock receive buffer

   Inputs:     UINT8_T     port        Identifier to associate with this connection

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
ERRCODE_T tktcp_clear_rx(UINT8_T port)
{
    ERRCODE_T ecode = NO_ERR;
    fd_set readfds;
    timeval wait_time;
    char buf[TKTCP_BUFFER_SIZE];
    int  i;

    /* Validate the port number */
    if( ecode = tktcp_validate_port(port) )
    {
        ecode = TKTCP_INVALID_PORT_ERR;
    }
    else
    {
        readfds.fd_count = 1;
        readfds.fd_array[0] = TKTCP_GET_SOCKET_M(port);

        wait_time.tv_sec = 0;
        wait_time.tv_usec = 0;

        /* Test if data is available in the socket */
        while(TRUE)
        {
            i = select(1,&readfds,NULL,NULL,&wait_time);

            if(i == SOCKET_ERROR)
            {

                switch(WSAGetLastError())
                {
                case WSANOTINITIALISED: i = 1;   break;   // A successful WSAStartup call must occur before using this function. 
                case WSAENETDOWN:       i = 2;   break;   // The network subsystem or the associated service provider has failed. 
                case WSAEAFNOSUPPORT:   i = 3;   break;   // The specified address family is not supported. 
                case WSAEINPROGRESS:    i = 4;   break;   // A blocking Windows Sockets 1.1 call is in progress, or the service provider is still processing a callback function. 
                case WSAEMFILE:         i = 5;   break;   // No more socket descriptors are available. 
                case WSAENOBUFS:        i = 6;   break;   // No buffer space is available. The socket cannot be created. 
                case WSAEPROTONOSUPPORT:i = 7;   break;   // The specified protocol is not supported. 
                case WSAEPROTOTYPE:     i = 8;   break;   // The specified protocol is the wrong type for this socket. 
                case WSAESOCKTNOSUPPORT:i = 9;   break;
                }
                ecode = TKTCP_SOCKET_ERR;
                break;         
            }
            else if(i == 0)
            {
                break;
            }
            else
            {
                i = recv(readfds.fd_array[0],buf,TKTCP_BUFFER_SIZE,0);

                if((i == SOCKET_ERROR) || (i == 0))
                {
                    ecode = TKTCP_SOCKET_ERR;
                    break;
                }
            }
        }
    }
    return(ecode);
}

/****************************************************************************
   tktcp_shutdown

   About:      Closes the socket connection for the given port 

   Inputs:     UINT8_T     port        Identifier to associate with this connection

   Outputs:    None

   Returns:    ERRCODE_T   ecode       Zero if successful
                                       Error code otherwise

****************************************************************************/
ERRCODE_T tktcp_shutdown(UINT8_T port)
{
    ERRCODE_T ecode = NO_ERR;
    int  i;
    char buf[TKTCP_BUFFER_SIZE];
    SOCKET local_socket;

    if( ecode = tktcp_validate_port(port) )
    {
    }
    else
    {
        local_socket = TKTCP_GET_SOCKET_M(port);

        /*
            Disallow any further data sends.  This will tell the other side
            that we want to go away now.  If we skip this step, we don't
            shut the connection down nicely.
        */
        if(shutdown(local_socket, SD_SEND) == SOCKET_ERROR)
        {
            ecode = TKTCP_SOCKET_ERR;
        }
        else
        {
            /* 
               Receive any extra data still sitting on the socket.  After all
               data is received, this call will block until the remote host
               acknowledges the TCP control packet sent by the shutdown above.
               Then we'll get a 0 back from recv, signalling that the remote
               host has closed its side of the connection.
            */
            while(1)
            {
                i = recv(local_socket, buf, TKTCP_BUFFER_SIZE, 0);
                if(i == SOCKET_ERROR)
                {
                    ecode = TKTCP_SOCKET_ERR;
                    break;
                }
                else if( i == 0)
                {
                    /* Okay, we're done! */
                    break;
                }
            }

            // Close the socket.
            if(closesocket(local_socket) == SOCKET_ERROR)
            {
                ecode = TKTCP_SOCKET_ERR;
            }
            else
            {
                TKTCP_SET_SOCKET_M(port, NULL);
            }
        }
    }
    return ecode;
}

ERRCODE_T tktcp_get_ipaddr(INT8_T * ip_addr, sockaddr_in* remote_addr)
{
    int     tcp_port;
	ERRCODE_T ecode = NO_ERR;
    /* Initialize registry connection */
    if( ecode = TKREG_INIT_M(TKTCP_REG_SUBKEY) )
    {
    }
    /* Get port number */
    else if( ecode = TKREG_OPEN_UINT32_M(TKTCP_PORT_ENTRY, TKTCP_DEFAULT_PORT, (UINT32_T*)(&tcp_port) ) )
    {
    }
    else
    {
        /* Lookup address */
        u_long nRemoteAddr = inet_addr(ip_addr);
        if (nRemoteAddr == INADDR_NONE) {
            // pcHost isn't a dotted IP, so resolve it through DNS
            hostent* pHE = gethostbyname(ip_addr);
            if (pHE == 0) {
                ecode = TKTCP_BAD_ADDR_ERR;
            }
            else
            {
                nRemoteAddr = *((u_long*)pHE->h_addr_list[0]);
            }
        }
        remote_addr->sin_family = AF_INET;
        remote_addr->sin_port = htons(tcp_port);
        remote_addr->sin_addr.s_addr = nRemoteAddr;
    }

    return ecode;
}

