/*******************************************************************************
********************************************************************************

   COPYRIGHT  2004, Advanced Illumination, Inc. 

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Advanced Illumination.

   tk_usb.h       External definitions for the USB Host Interface Toolkit functions

   About:         This module contains external definitions for the USB  Host Interface
                  Toolkit functions in the AICONS01 project.

   Target:        Win32 Compatible

   Written for:   Advanced Illumination, Inc.
                  24 Peavine Drive
                  Rochester, VT 05767
                  http://advancedillumination.com

   Contacts:      John Thrailkill
                  Phone:  (802) 767-3830
                  FAX:    (802) 767-3831
                  jthrailkill@advill.com

   Written by:    Rob Macklin
                  Microprocessor Designs, Inc.
                  65 Longmeadow Drive
                  PO Box 160
                  Shelburne, VT 05482
                  http://www.updesigns.com

   $Revision: 2 $

--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/S1000/pc_src/Common/tk_usb.h $
 * 
 * 2     8/24/04 8:44a Rmack
 * Updated header information.
 * 
 * 1     3/30/04 3:27p Rmack
 * 
********************************************************************************
*******************************************************************************/
#ifndef _TKUSB_H
#define _TKUSB_H

/* Definitions for common values */
#define TKUSB_MAX_PACKET_SIZE         200
#define TKUSB_MAX_USB_PORTS           50

typedef struct
{
   UINT8_T  report_id;
   UINT8_T  usage;
   UINT16_T report_size;
} TKUSB_REPORT_INFO_STRUCT;

#ifdef NOUSB
/* Macro interfaces for all toolkit functions */
#define TKUSB_INIT_M(port, name, num)     NO_ERR
#define TKUSB_VALIDATE_PORT_M(port)       NO_ERR
#define TKUSB_WRITE_M(port, buf, timeout) NO_ERR
#define TKUSB_READ_M(port, buf, timeout)  NO_ERR
#define TKUSB_CLEAR_RX_M(port)            NO_ERR
#define TKUSB_GET_REPORT_INFO_M(port, usage, pri)   NO_ERR
#define TKUSB_SET_FEATURE_M(port, feature) NO_ERR
#define TKUSB_GET_STRING_M(port, index, buf, len)   NO_ERR
#define TKUSB_SHUTDOWN_M(port)            NO_ERR
#else
/* Macro interfaces for all toolkit functions */
#define TKUSB_INIT_M(port, name, num)     tkusb_init(port, name, num)
#define TKUSB_VALIDATE_PORT_M(port)       tkusb_validate_port(port)
#define TKUSB_WRITE_M(port, buf, timeout) tkusb_write(port, buf, timeout)
#define TKUSB_READ_M(port, buf, timeout)  tkusb_read(port, buf, timeout)
#define TKUSB_CLEAR_RX_M(port)            tkusb_clear_rx(port)
#define TKUSB_GET_REPORT_INFO_M(port, usage, pri)                  \
   tkusb_get_report_info(port, usage, pri)
#define TKUSB_SET_FEATURE_M(port, feature)                         \
   tkusb_set_feature(port, feature)
#define TKUSB_GET_STRING_M(port, index, buf, len)                  \
   tkusb_get_string(port, index, buf, len)
#define TKUSB_SHUTDOWN_M(port)            tkusb_shutdown(port)
#endif

/* Prototypes for all toolkit functions */
ERRCODE_T tkusb_init(UINT8_T port, INT8_T * dev_name, INT8_T * serial_num);
ERRCODE_T tkusb_validate_port(UINT8_T port);
ERRCODE_T tkusb_write(UINT8_T port, UINT8_T * out_buf, UINT16_T timeout);
ERRCODE_T tkusb_read(UINT8_T port, UINT8_T * in_buf, UINT16_T timeout);
ERRCODE_T tkusb_get_report_info(UINT8_T port, UINT8_T usage, TKUSB_REPORT_INFO_STRUCT *pri);
ERRCODE_T tkusb_clear_rx(UINT8_T port);
ERRCODE_T tkusb_set_feature(UINT8_T port, UINT8_T * fr);
ERRCODE_T tkusb_get_string(UINT8_T port, UINT16_T index, UINT8_T *buf, UINT16_T len);
ERRCODE_T tkusb_shutdown(UINT8_T port);

#endif /* _TKTIME_H */