/*******************************************************************************
********************************************************************************

   COPYRIGHT  2004, Advanced Illumination, Inc. 

   Contains confidential and proprietary information which may not be copied,
   disclosed or used by others except as expressly authorized in writing
   by Advanced Illumination.

   tarerror.h     Target Error Code Definition Header File 

   About:         This is the common header file for all target error codes
                  used in the AICONS01 project.

   Target CPU:    H8S/2215

   Written for:   Advanced Illumination, Inc.
                  24 Peavine Drive
                  Rochester, VT 05767
                  http://advancedillumination.com

   Contacts:      John Thrailkill
                  Phone:  (802) 767-3830
                  FAX:    (802) 767-3831
                  jthrailkill@advill.com

   Written by:    Rob Macklin
                  Microprocessor Designs, Inc.
                  65 Longmeadow Drive
                  PO Box 160
                  Shelburne, VT 05482
                  http://www.updesigns.com

   $Revision: 1 $

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
   $Log: /Advanced Illumination/Pulsar320/tar_src/H8App/tarerror.h $
 * 
 * 1     8/15/08 8:45a Jpiet
 * Initial check-in
 * 
 * 19    12/30/05 3:40p Rmack
 * New error code for low VIN when using continuous mode.
 * 
 * 18    5/02/05 7:58a Rmack
 * New error for uninitialized lighthead.
 * 
 * 17    4/27/05 4:13p Rmack
 * New ecode for max width error.
 * 
 * 16    4/27/05 9:07a Rmack
 * Added conditional compilation for overlapping ecodes.
 * 
 * 15    4/27/05 8:09a Rmack
 * Added new ecode for Signatech I lightheads.
 * 
 * 14    2/23/05 3:49p Rmack
 * Added new error code for jumper check
 * 
 * 13    2/09/05 12:39p Rmack
 * New error codes.
 * 
 * 12    11/11/04 2:33p Rmack
 * Updated Signatech I support, no thermal calculation in continuous mode,
 * improved error display during lighthead detection.
 * 
 * 11    10/29/04 8:06a Rmack
 * Updated attenuation values.
 * 
 * 10    10/15/04 8:47a Rmack
 * Lighthead protection working, pulse, cont, bypass, int trig selectable.
 * 
 * 9     9/21/04 11:57a Rmack
 * 
 * 7     8/17/04 4:50p Rmack
 * Restructure of existing code, release for beta test site.  Still ISR
 * driven triggering.
 * 
 * 6     8/11/04 4:40p Jwayg
 * Final check-in under JWayg name, code currently setup to run LED tests.
 * 
 * 5     7/01/04 10:30a Jwayg
 * Last code release before transfer of source files to new programmer.
 * 
 * 4     6/07/04 9:54a Jwayg
 * S1000 - -005 firmware level - pulses created programatically via
 * special commands
 * 
 * 3     5/17/04 10:57a Jwayg
 * Common Files for Initial Running Version of S1000 
 * 
 * 2     4/20/04 11:11a Rmack
 * Check in before transfer of project to John
 * 
 * 1     3/30/04 3:32p Rmack
 *
********************************************************************************
*******************************************************************************/
#ifndef _TARERROR_H
#define _TARERROR_H

#define E_SUCCESS                         0
#define NO_ERR							  0
#define E_FAILED                          -1

/* Error codes returned by communication functions */
#define HOSTCMD_NO_ERR                     E_SUCCESS
#define HOSTCMD_BASE_ERR                   0
#define HOSTCMD_INVALID_GROUP_ERR          (HOSTCMD_BASE_ERR - 2)
#define HOSTCMD_INVALID_FUNCT_ERR          (HOSTCMD_BASE_ERR - 3)
#define HOSTCMD_INVALID_DATA_ERR           (HOSTCMD_BASE_ERR - 4)
#define HOSTCMD_INVALID_LENGTH_ERR         (HOSTCMD_BASE_ERR - 5)
#define HOSTCMD_BADCOMMAND_ERR             (HOSTCMD_BASE_ERR - 6)
#define HOSTCMD_232_PORT_NOT_FOUND_ERR     (HOSTCMD_BASE_ERR - 7)
#define HOSTCMD_MUTEX_TIMEOUT_ERR          (HOSTCMD_BASE_ERR - 8)
#define HOSTCMD_MUTEX_FAILED_ERR           (HOSTCMD_BASE_ERR - 9)
#define HOSTCMD_COMMAND_SEND_ERR           (HOSTCMD_BASE_ERR - 10)
#define HOSTCMD_INVALID_RESPONSE_ERR       (HOSTCMD_BASE_ERR - 11)
#define HOSTCMD_RESPONSE_TIMEOUT_ERR       (HOSTCMD_BASE_ERR - 12)
#define HOSTCMD_BAD_BLOCK_CSUM_ERR         (HOSTCMD_BASE_ERR - 13)
#define HOSTCMD_COMMAND_TIMEOUT_ERR        (HOSTCMD_BASE_ERR - 14)

/* Error codes returned by the Interrupt primitives */ 
#define IOINTS_BASE_ERR                   -20
#define IOINTS_NOT_MASKABLE_ERR           (IOINTS_BASE_ERR - 0)
#define IOINTS_INVALID_VECT_ERR           (IOINTS_BASE_ERR - 1)
#define IOINTS_NOT_REDIRECTABLE_ERR       (IOINTS_BASE_ERR - 2)
#define IOINTS_NO_SHADOW_ERR              (IOINTS_BASE_ERR - 3)

/* Error codes returned by the Digital I/O primitives */
#define IODIG_BASE_ERR                    -30
#define IODIG_INVALID_CHAN_ERR            (IODIG_BASE_ERR - 0)                    

/* Error codes returned by the CPU Management primitives */
#define IOCPU_BASE_ERR                    -35
#define IOCPU_INVALID_MODE_ERR            (IOCPU_BASE_ERR - 0)
#define IOCPU_INVALID_SPEED_ERR           (IOCPU_BASE_ERR - 1) 

/* Error codes returned by the TPU primitives */
#define IOTPU_BASE_ERR                    -40
#define IOTPU_INVALID_ID_ERR              (IOTPU_BASE_ERR - 0)
#define IOTPU_ZERO_RATE_ERR               (IOTPU_BASE_ERR - 1)
#define IOTPU_UNKNOWN_OUT_ERR             (IOTPU_BASE_ERR - 2)
#define IOTPU_INVALID_RATE_ERR            (IOTPU_BASE_ERR - 3)

/* Error codes returned by the WDT primitives */
#define IOWDT_BASE_ERR                    -45
#define IOWDT_UNKNOWN_MODE_ERR            (IOWDT_BASE_ERR - 0)
#define IOWDT_NO_TIMER_ISR_ERR            (IOWDT_BASE_ERR - 1)
 
/* Error codes returned by the ADC primitives */
#define IOADC_BASE_ERR                    -50  
#define IOADC_NO_CHAN_ERR                 (IOADC_BASE_ERR - 0)
#define IOADC_INVALID_CHAN_ERR            (IOADC_BASE_ERR - 1)
#define IOADC_INVALID_RATE_ERR            (IOADC_BASE_ERR - 2)
#define IOADC_INVALID_BUF_SIZE_ERR        (IOADC_BASE_ERR - 3) 

/* Error codes returned by the DAC primitives */
#define IODAC_BASE_ERR                    -55  
#define IODAC_INVALID_CHAN_ERR            (IODAC_BASE_ERR - 0)

/* Error codes returned by the I2C DAC primitives */
#define IOI2CDAC_BASE_ERR                 -60
#define IOI2CDAC_INVALID_CHAN_ERR         (IOI2CDAC_BASE_ERR - 0)

/* Error codes returned by the DTC primitives */ 
#define IODTC_BASE_ERR                    -65
#define IODTC_INVALID_VECT_ERR            (IODTC_BASE_ERR - 0)
                    
/* Error codes returned by the SCI primitives */
#define IOSCI_BASE_ERR                    -70  
#define IOSCI_INVALID_PORT_ERR            (IOSCI_BASE_ERR - 0) 
#define IOSCI_INVALID_BAUD_ERR            (IOSCI_BASE_ERR - 1)  
#define IOSCI_INVALID_MODE_ERR            (IOSCI_BASE_ERR - 2)
#define IOSCI_RX_OVERRUN_ERR              (IOSCI_BASE_ERR - 3)
#define IOSCI_TX_OVERRUN_ERR              (IOSCI_BASE_ERR - 4)
#define IOSCI_UART_OVERRUN_ERR            (IOSCI_BASE_ERR - 5)
#define IOSCI_UART_PARITY_ERR             (IOSCI_BASE_ERR - 6)
#define IOSCI_UART_FRAMING_ERR            (IOSCI_BASE_ERR - 7)
#define IOSCI_RX_UNDERRUN_ERR             (IOSCI_BASE_ERR - 8)
#define IOSCI_DTC_BUFFER_ERR              (IOSCI_BASE_ERR - 9)

/* Error codes returned by the SPI primitives */
#define IOSPI_BASE_ERR                    -90
#define IOSPI_INVALID_PORT_ERR            (IOSPI_BASE_ERR - 0) 
#define IOSPI_INVALID_BAUD_ERR            (IOSPI_BASE_ERR - 1)  
#define IOSPI_INVALID_MODE_ERR            (IOSPI_BASE_ERR - 2)
#define IOSPI_RX_OVERRUN_ERR              (IOSPI_BASE_ERR - 3)
#define IOSPI_TX_OVERRUN_ERR              (IOSPI_BASE_ERR - 4)
#define IOSPI_UART_OVERRUN_ERR            (IOSPI_BASE_ERR - 5)
#define IOSPI_UART_PARITY_ERR             (IOSPI_BASE_ERR - 6)
#define IOSPI_UART_FRAMING_ERR            (IOSPI_BASE_ERR - 7)
#define IOSPI_RX_UNDERRUN_ERR             (IOSPI_BASE_ERR - 8)
#define IOSPI_TX_TIMEOUT_ERR              (IOSPI_BASE_ERR - 9)
#define IOSPI_RX_TIMEOUT_ERR              (IOSPI_BASE_ERR - 10)

/* Error codes returned by the 1-Wire primitives */
#define IO1WIRE_BASE_ERR                  -102
#define IO1WIRE_NOT_PRESENT_ERR           (IO1WIRE_BASE_ERR - 0)
#define IO1WIRE_ROM_CRC_ERR               (IO1WIRE_BASE_ERR - 1)
#define IO1WIRE_INVALID_ADDR_ERR          (IO1WIRE_BASE_ERR - 2)
#define IO1WIRE_ADDR_VERIFY_ERR           (IO1WIRE_BASE_ERR - 3)
#define IO1WIRE_DATA_VERIFY_ERR           (IO1WIRE_BASE_ERR - 4)
#define IO1WIRE_PF_ERR                    (IO1WIRE_BASE_ERR - 5)
#define IO1WIRE_WRITE_FAILED_ERR          (IO1WIRE_BASE_ERR - 6)
#define IO1WIRE_INVALID_CHANNEL_ERR       (IO1WIRE_BASE_ERR - 7)

/* Error codes returned by the I2C primitive */
#define IOI2C_BASE_ERR                    -110
#define IOI2C_NOACK_ERR                   (IOI2C_BASE_ERR - 0)
#define IOI2C_INIT_WDR_TIMOUT_ERR         (IOI2C_BASE_ERR - 1)
#define IOI2C_INIT_XMIT_TIMOUT_ERR        (IOI2C_BASE_ERR - 2)
#define IOI2C_SEND_XMIT_TIMOUT_ERR        (IOI2C_BASE_ERR - 3)
#define IOI2C_GET_RECV_TIMOUT_ERR         (IOI2C_BASE_ERR - 4)
#define IOI2C_STOP_TIMEOUT_ERR            (IOI2C_BASE_ERR - 5)
#define IOI2C_WDR_TIMOUT_ERR              (IOI2C_BASE_ERR - 6)
#define IOI2C_INVALID_BAUD                (IOI2C_BASE_ERR - 7)

/* Error codes returned by the USB primitive */
#define IOUSB_BASE_ERR                    -120
#define IOUSB_BUS_INIT_ERR                (IOUSB_BASE_ERR - 0)  
#define IOUSB_INVALID_ENDPOINT_ERR        (IOUSB_BASE_ERR - 1)
#define IOUSB_RX_OVERRUN_ERR              (IOUSB_BASE_ERR - 2)
#define IOUSB_TX_OVERRUN_ERR              (IOUSB_BASE_ERR - 3)
#define IOUSB_RX_UNDERRUN_ERR             (IOUSB_BASE_ERR - 4)
#define IOUSB_WAKEUP_DISABLE_ERR          (IOUSB_BASE_ERR - 5)
#define IOUSB_BAD_IFCNUM_ERR              (IOUSB_BASE_ERR - 6)
#define IOUSB_TX_BUFFER_ERR               (IOUSB_BASE_ERR - 7)
#define IOUSB_EP_BUSY_ERR                 (IOUSB_BASE_ERR - 8)

/* Error codes returned by the 8-bit Timer primitive */
#define IOTMR_BASE_ERR                    -130
#define IOTMR_ZERO_RATE_ERR               (IOTMR_BASE_ERR - 0)
#define IOTMR_INVALID_ID_ERR              (IOTMR_BASE_ERR - 1)
#define IOTMR_UNKNOWN_OUT_ERR             (IOTMR_BASE_ERR - 2)
#define IOTMR_INVALID_RATE_ERR            (IOTMR_BASE_ERR - 3)

/* Error codes returned by the Diagnostics toolkit */
#define TKDIAG_BASE_ERR                   -200
#define TKDIAG_INVALID_TEST_ERR           (TKDIAG_BASE_ERR - 11)
#define TKDIAG_LO_33VMON_ERR              (TKDIAG_BASE_ERR - 12)
#define TKDIAG_HI_33VMON_ERR              (TKDIAG_BASE_ERR - 13)
#define TKDIAG_LO_5VMON_ERR               (TKDIAG_BASE_ERR - 14)
#define TKDIAG_HI_5VMON_ERR               (TKDIAG_BASE_ERR - 15)
#define TKDIAG_LO_VIN_SWMON_ERR           (TKDIAG_BASE_ERR - 16)
#define TKDIAG_HI_VIN_SWMON_ERR           (TKDIAG_BASE_ERR - 17)
#define TKDIAG_LO_10VMON_ERR              (TKDIAG_BASE_ERR - 18)
#define TKDIAG_HI_10VMON_ERR              (TKDIAG_BASE_ERR - 19)
#define TKDIAG_GND_SLO_ERR                (TKDIAG_BASE_ERR - 21)
#define TKDIAG_GND_MID_ERR                (TKDIAG_BASE_ERR - 22)
#define TKDIAG_GND_FAST_ERR               (TKDIAG_BASE_ERR - 23)
#define TKDIAG_EXT_RAM_ERR                (TKDIAG_BASE_ERR - 41)
#define TKDIAG_EXTREG1_ERR                (TKDIAG_BASE_ERR - 45)
#define TKDIAG_EXTREG2_ERR                (TKDIAG_BASE_ERR - 46)
#define TKDIAG_MFG_EEPROM_CRC_ERR         (TKDIAG_BASE_ERR - 51)
#define TKDIAG_CAL_EEPROM_CRC_ERR         (TKDIAG_BASE_ERR - 52)
#define TKDIAG_MFG_EEPROM_WRITE_ERR       (TKDIAG_BASE_ERR - 53)
#define TKDIAG_CONFIG_EEPROM_CRC_ERR      (TKDIAG_BASE_ERR - 54)
#define TKDIAG_APP1_FLASH_CRC_ERR         (TKDIAG_BASE_ERR - 61)
#define TKDIAG_APP2_FLASH_CRC_ERR         (TKDIAG_BASE_ERR - 62)
#define TKDIAG_APP3_FLASH_CRC_ERR         (TKDIAG_BASE_ERR - 63)
#define TKDIAG_LO_CH1_LO_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 71)
#define TKDIAG_LO_CH2_LO_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 72)
#define TKDIAG_LO_CH3_LO_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 73)
#define TKDIAG_LO_CH4_LO_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 74)
#define TKDIAG_HI_CH1_LO_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 75)
#define TKDIAG_HI_CH2_LO_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 76)
#define TKDIAG_HI_CH3_LO_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 77)
#define TKDIAG_HI_CH4_LO_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 78)
#define TKDIAG_LO_CH1_MD_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 79)
#define TKDIAG_LO_CH2_MD_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 81)
#define TKDIAG_LO_CH3_MD_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 82)
#define TKDIAG_LO_CH4_MD_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 83)
#define TKDIAG_HI_CH1_MD_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 84)
#define TKDIAG_HI_CH2_MD_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 85)
#define TKDIAG_HI_CH3_MD_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 86)
#define TKDIAG_HI_CH4_MD_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 87)
#define TKDIAG_LO_CH1_HI_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 88)
#define TKDIAG_LO_CH2_HI_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 89)
#define TKDIAG_LO_CH3_HI_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 91)
#define TKDIAG_LO_CH4_HI_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 92)
#define TKDIAG_HI_CH1_HI_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 93)
#define TKDIAG_HI_CH2_HI_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 94)
#define TKDIAG_HI_CH3_HI_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 95)
#define TKDIAG_HI_CH4_HI_DRV_LVL_ERR      (TKDIAG_BASE_ERR - 96)
#define TKDIAG_RS232_TX_ERR               (TKDIAG_BASE_ERR - 111)
#define TKDIAG_RS232_RX_ERR               (TKDIAG_BASE_ERR - 112)
#define TKDIAG_RS485_ERR                  (TKDIAG_BASE_ERR - 113)


/* Error codes returned by the Lighthead toolkit */
#define TKLIGHT_BASE_ERR                  -350
#define TKLIGHT_NOT_PRESENT_ERR           (TKLIGHT_BASE_ERR - 1)
#define TKLIGHT_BAD_SIG1_ID1_ERR          (TKLIGHT_BASE_ERR - 2)
#define TKLIGHT_BAD_SIG1_ID2_ERR          (TKLIGHT_BASE_ERR - 3)
#define TKLIGHT_SIG2_CRC_ERR              (TKLIGHT_BASE_ERR - 4)
#define TKLIGHT_OVERTEMP_ERR              (TKLIGHT_BASE_ERR - 5)
#define TKLIGHT_CONFIG_CRC_ERR            (TKLIGHT_BASE_ERR - 6)
#define TKLIGHT_MIN_SIG1_ID_VOLTS_ERR     (TKLIGHT_BASE_ERR - 7)
#define TKLIGHT_MAX_SIG1_ID_VOLTS_ERR     (TKLIGHT_BASE_ERR - 8)
#define TKLIGHT_MAX_CURR_ERR              (TKLIGHT_BASE_ERR - 9)
#define TKLIGHT_MAX_WIDTH_ERR             (TKLIGHT_BASE_ERR - 10)
#define TKLIGHT_NOT_SIG1_ERR              (TKLIGHT_BASE_ERR - 11)
#define TKLIGHT_NOT_INITIALIZED_ERR       (TKLIGHT_BASE_ERR - 12)
#define TKLIGHT_LOW_VIN_ERR               (TKLIGHT_BASE_ERR - 13)
#define TKLIGHT_NOT_SUPPORTED_ERR         (TKLIGHT_BASE_ERR - 14)
#define TKLIGHT_MAX_FREQ_ERR              (TKLIGHT_BASE_ERR - 15)


/* Error codes returned by the Pulse toolkit */
#define TKPULSE_BASE_ERR                  -410
#define TKPULSE_INVALID_CHAN_ERR          (TKPULSE_BASE_ERR - 1)
#define TKPULSE_MAX_WIDTH_ERR             (TKPULSE_BASE_ERR - 2)
#define TKPULSE_MAX_DELAY_ERR             (TKPULSE_BASE_ERR - 3)
#define TKPULSE_MAX_CURRENT_ERR           (TKPULSE_BASE_ERR - 4)
#define TKPULSE_MAX_HV_VOLTAGE_ERR        (TKPULSE_BASE_ERR - 5)
#define TKPULSE_MAX_LV_VOLTAGE_ERR        (TKPULSE_BASE_ERR - 6)
#define TKPULSE_BAD_TRIGGER_MODE_ERR      (TKPULSE_BASE_ERR - 7)
#define TKPULSE_INVALID_RATE_ERR          (TKPULSE_BASE_ERR - 8)
#define TKPULSE_MIN_HV_VOLTAGE_ERR        (TKPULSE_BASE_ERR - 9)
#define TKPULSE_HV_VERIFY_TIMEOUT_ERR     (TKPULSE_BASE_ERR - 11)
#define TKPULSE_LV_VERIFY_TIMEOUT_ERR     (TKPULSE_BASE_ERR - 12)
#define TKPULSE_POWER_SUPPLY_OFF_ERR      (TKPULSE_BASE_ERR - 13)
#define TKPULSE_SUPPLY_JUMPER_ERR         (TKPULSE_BASE_ERR - 14)
#define TKPULSE_MAX_PERIOD_ERR            (TKPULSE_BASE_ERR - 15)

/* Error codes returned by the Flash Programming toolkit */
#define TKFLASH_BASE_ERR                  -500
#define TKFLASH_BAD_ADDRESS_ERR           (TKFLASH_BASE_ERR - 0)
#define TKFLASH_PROG_FAILED_ERR           (TKFLASH_BASE_ERR - 1)
#define TKFLASH_ERASE_FAILED_ERR          (TKFLASH_BASE_ERR - 2)

/* Error codes returned by the Command Processor toolkit */                           
#define TKCMDPR_BASE_ERR                  -510  
#define TKCMDPR_BAD_MSG_TYPE_ERR          (TKCMDPR_BASE_ERR - 0)
#define TKCMDPR_BAD_CMD_ERR               (TKCMDPR_BASE_ERR - 1)
#define TKCMDPR_NO_DATA_AVAILABLE_ERR     (TKCMDPR_BASE_ERR - 2) 

/* Error codes returned by the Command Set toolkit */
#ifndef TKCMDSET_BASE_ERR
#define TKCMDSET_BASE_ERR                 -520
#define TKCMDSET_UNKNOWN_SPECIAL_ERR      (TKCMDSET_BASE_ERR - 1)
#define TKCMDSET_INVALID_PARAM_ERR        (TKCMDSET_BASE_ERR - 2)
#define TKCMDSET_MIN_MEASURE_WIDTH_ERR    (TKCMDSET_BASE_ERR - 3)
#endif

/* Error codes returned by the Realtime Timer toolkit */
#define TKRTTIMER_BASE_ERR                -530
#define TKRTTIMER_INVALID_TIMER_NUM_ERR   (TKRTTIMER_BASE_ERR - 0)
#define TKRTTIMER_INVALID_TIMER_MODE_ERR  (TKRTTIMER_BASE_ERR - 1)
#define TKRTTIMER_INVALID_STAT_INDEX_ERR  (TKRTTIMER_BASE_ERR - 2)
#define TKRTTIMER_INVALID_TASK_NUM_ERR    (TKRTTIMER_BASE_ERR - 3)

/* Error codes returned by the User toolkit */
#define TKUSER_BASE_ERR                   -540
#define TKUSER_INVALID_SWITCH_NUM_ERR     (TKUSER_BASE_ERR - 0)

/* Error codes returned by the Analog toolkit */
#define TKANALOG_BASE_ERR                 -550
#define TKANALOG_INVALID_CHAN_ERR         (TKANALOG_BASE_ERR - 0)

/* Error codes returned by the EEPROM toolkit */
#define TKEEPROM_BASE_ERR                 -600
#define TKEEPROM_INIT_ERR                 (TKEEPROM_BASE_ERR - 0)
#define TKEEPROM_ACK_ERR                  (TKEEPROM_BASE_ERR - 1)
#define TKEEPROM_STOP_ERR                 (TKEEPROM_BASE_ERR - 2)
#define TKEEPROM_INVALID_END_ADDR_ERR     (TKEEPROM_BASE_ERR - 3)
#define TKEEPROM_WRITE_TIMEOUT_ERR        (TKEEPROM_BASE_ERR - 4)
#define TKEEPROM_WRITE_READ_ERR           (TKEEPROM_BASE_ERR - 5)
#define TKEEPROM_WRITE_VERIFY_ERR         (TKEEPROM_BASE_ERR - 6)
#define TKEEPROM_CHKSM_READ_ERR           (TKEEPROM_BASE_ERR - 7)
#define TKEEPROM_STATUS_ERR               (TKEEPROM_BASE_ERR - 8)
#define TKEEPROM_INVALID_ADDRESS_ERR      (TKEEPROM_BASE_ERR - 9)

/* Error codes returned by the USB toolkit */
#define TKETH_BASE_ERR                    -620
#define TKETH_NO_FRAME_AVAILABLE_ERR      (TKETH_BASE_ERR - 0)
#define TKETH_RESPONSE_TIMEOUT_ERR        (TKETH_BASE_ERR - 1)
#define TKETH_NO_DATA_ERR                 (TKETH_BASE_ERR - 2)
#define TKETH_INVALID_CRC_ERR             (TKETH_BASE_ERR - 3)
#define TKETH_UNEXPECTED_RSP_ERR          (TKETH_BASE_ERR - 4)
#define TKETH_TCP_BUFFER_OVERRUN_ERR      (TKETH_BASE_ERR - 5)
#define TKETH_TCP_BUFFER_EMPTY_ERR        (TKETH_BASE_ERR - 6)
#define TKETH_INTERCHAR_TIMEOUT_ERR       (TKETH_BASE_ERR - 7)
#define TKETH_WRITE_ERR                   (TKETH_BASE_ERR - 8)
#define TKETH_WRITE_TIMEOUT_ERR           (TKETH_BASE_ERR - 9)

/* Error codes returned by the USB toolkit */
#ifndef TKUSB_BASE_ERR
#define TKUSB_BASE_ERR                    -640
#define TKUSB_INVALID_EPNUM_ERR           (TKUSB_BASE_ERR - 0)
#define TKUSB_INIT_EPNUM_ERR              (TKUSB_BASE_ERR - 1)
#define TKUSB_BAD_DESC_RQT_ERR            (TKUSB_BASE_ERR - 2)
#define TKUSB_EP_RX_OVERRUN_ERR           (TKUSB_BASE_ERR - 3)
#define TKUSB_RX_TIMEOUT_ERR              (TKUSB_BASE_ERR - 4)
#define TKUSB_GET_RX_CHAR_ERR             (TKUSB_BASE_ERR - 5)
#define TKUSB_INVALID_TX_LEN_ERR          (TKUSB_BASE_ERR - 6)
#define TKUSB_EP_TX_OVERRUN_ERR           (TKUSB_BASE_ERR - 7)
#define TKUSB_INVALID_STRING_ID_ERR       (TKUSB_BASE_ERR - 8)
#define TKUSB_NO_FEATURE_REPORT_ERR       (TKUSB_BASE_ERR - 9)
#endif

#define ETHCMD_BASE_ERR                    -660
#define ETHCMD_INVALID_COMMAND_ERR        (ETHCMD_BASE_ERR - 0)

/* Error codes returned by language toolkit */
#define TKLANG_BASE_ERR                    -680
#define TKLANG_INVALID_LANGUAGE_ERR        (TKLANG_BASE_ERR - 0)

#define TKMENU_BASE_ERR                    -700
#define TKMENU_VALIDATE_ERR                 (TKMENU_BASE_ERR - 0)


#endif /* _TARERROR_H */