﻿Imports System.Threading
Imports System.IO
Imports System.Runtime.Serialization.Formatters.Binary

Public Class CPulsar2
    Public Event evError(ByVal p_chan As Byte, ByVal p_ecode As Short, ByVal p_errName As String, ByVal p_errDesc As String)
    Const POLL_INTERVAL As Integer = 70
    Const TKCMDSET_H8_PRODUCT_NAME As String = "Ai Pulsar 320 Controller"
    Const E_SUCCESS As Short = 0
    Const NUM_CHANNELS As Byte = 2
    Const HV_SUPPLY_CURR As Single = 0.5

    Private PulsarSettings As New PulsarData
    'Private PCSettings As New PulsarData
    'Private PresetSettings As New PulsarData
    Private UserData As UserDataStruct

    Private myParent As Config
    Private EvMainReq As ManualResetEvent
    Private EvThreadRel As ManualResetEvent
    Private PollThread As Thread        'thread that sends commands to read pulsar settings
    Public PulsarConnected As Boolean  'tracks connection status to pulsar
    Private Port As Byte                'port used for comm
    Private Address As String           'hold IP address
    Private PulsarFWVersion As String = ""
    Private PulsarHWVersion As String = ""
    Private PulsarSerialNumber As String = ""
    Private PulsarIPAddr As String = ""
    Private PulsarSubnet As String = ""
    Private PulsarGateway As String = ""
    Private PulsarMAC As String = ""
    Private byPollChan As Byte = 1


    Public Function Initialize(ByVal p_Parent As Config, ByVal p_Port As Integer, ByVal p_Address As String) As Boolean
        myParent = p_Parent
        ' Create polling thread
        ' Prepare the polling thread
        EvMainReq = New ManualResetEvent(False)
        EvThreadRel = New ManualResetEvent(False)
        PollThread = New Thread(New ThreadStart(AddressOf Me.PulsarPollThread))
        PollThread.IsBackground = True
        PollThread.Start()
        Port = p_Port
        If (p_Address.StartsWith("USB")) Then
            p_Address = ""
        Else
            Address = p_Address
        End If
    End Function
    Private Sub PulsarPollThread()
        While True
            EvThreadRel.Set()
            If EvMainReq.WaitOne(0, False) Then
                'Main app talking to Pulsar, skip poll
                '                Thread.Sleep(1000)
            Else
                Me.PollData()
            End If
            EvThreadRel.Reset()
            Thread.Sleep(POLL_INTERVAL)
        End While
    End Sub
    Public Function PollData() As Boolean
        Dim e_code As Short
        Dim e_text As String = ""
        Dim temp_string_1 As String = ""
        Dim temp_string_2 As String = ""
        Dim temp_string_3 As String = ""
        Static Dim Iteration As Byte = 0
        Static Dim CmdFailCount As Byte = 0

        'If not connected, first establish connection
        If Not PulsarConnected Then
            CmdFailCount = 0
            If Not (Address = "") Then
                e_code = api_connect(Port, TKCMDSET_H8_PRODUCT_NAME, Address)
            Else
                e_code = api_init(Port, TKCMDSET_H8_PRODUCT_NAME, PulsarSerialNumber)
            End If
            If (e_code <> E_SUCCESS) Then
                api_get_error_text(e_code, e_text, 256)
            End If

            While (CmdFailCount < 4)

                'If (e_code <> E_SUCCESS) Then
                '    api_get_error_text(e_code, e_text, 256)
                'Else
                'Need to give these strings an initial length
                temp_string_1 = LSet(temp_string_1, 256)
                temp_string_2 = LSet(temp_string_2, 256)
                temp_string_3 = LSet(temp_string_3, 256)
                e_code = api_get_pulsar_version(Port, _
                                                        temp_string_1, _
                                                        temp_string_2, _
                                                        temp_string_3)
                If e_code <> 0 Then
                    CmdFailCount += 1
                Else
                    CmdFailCount = 0
                    PulsarFWVersion = CTrim(temp_string_1)
                    PulsarHWVersion = CTrim(temp_string_2)
                    PulsarSerialNumber = CTrim(temp_string_3)
                    Exit While
                End If
                'End If
            End While
            While (CmdFailCount < 4)
                'Need to give these strings an initial length
                temp_string_1 = LSet(temp_string_1, 256)
                temp_string_2 = LSet(temp_string_2, 256)
                temp_string_3 = LSet(temp_string_3, 256)
                e_code = api_get_pulsar_version(Port, _
                                                        temp_string_1, _
                                                        temp_string_2, _
                                                        temp_string_3)
                If e_code <> 0 Then
                    CmdFailCount += 1
                Else
                    CmdFailCount = 0
                    PulsarFWVersion = CTrim(temp_string_1)
                    PulsarHWVersion = CTrim(temp_string_2)
                    PulsarSerialNumber = CTrim(temp_string_3)
                    Exit While
                End If
            End While
            While (CmdFailCount < 4)
                e_code = api_get_user_data(Port, UserData.Rotary, _
                                                 UserData.Dip, _
                                                 UserData.Trigger, _
                                                 UserData.Connect, _
                                                 UserData.Supply, _
                                                 UserData.Expansion)
                If e_code <> 0 Then
                    CmdFailCount += 1
                Else
                    CmdFailCount = 0
                    If (UserData.Supply And API_SUPPLY_VIN_ON) <> 0 Then
                        PulsarSettings.Settings(0).current = 0
                        PulsarSettings.Settings(1).current = 0
                    Else
                    End If
                    Exit While
                End If
            End While
            While (CmdFailCount < 4)
                e_code = api_get_channel_config(Port, 1, _
                                                 API_CHAN_CONFIG_ACTIVE, _
                                                 PulsarSettings.Settings(0))
                If e_code <> 0 Then
                    CmdFailCount += 1
                Else
                    e_code = api_get_channel_config(Port, 2, _
                                                     API_CHAN_CONFIG_ACTIVE, _
                                                     PulsarSettings.Settings(1))
                    If e_code <> 0 Then
                        CmdFailCount += 1
                    Else
                        CmdFailCount = 0
                        Exit While
                    End If
                End If
            End While
            While (CmdFailCount < 4)
                PulsarIPAddr = LSet(PulsarIPAddr, 20)
                PulsarSubnet = LSet(PulsarSubnet, 20)
                PulsarGateway = LSet(PulsarGateway, 20)
                PulsarMAC = LSet(PulsarMAC, 20)
                e_code = api_get_pulsar_ipaddr(Port, PulsarIPAddr, _
                                                 PulsarSubnet, _
                                                 PulsarGateway, _
                                                 PulsarMAC)
                'If e_code <> 0 Then
                '    CmdFailCount += 1
                'Else
                '    e_code = api_get_channel_config(Port, 2, _
                '                                     API_CHAN_CONFIG_ACTIVE, _
                '                                     PulsarSettings.Settings(1))
                '    If e_code <> 0 Then
                '        CmdFailCount += 1
                '    Else
                '        CmdFailCount = 0
                Exit While
                '    End If
                'End If
            End While
            If (CmdFailCount < 4) Then
                PulsarConnected = True
                myParent.BeginInvoke(myParent.HandleConnectDelegate)
            End If
        End If

        'Once connection is established, send a single command on each poll so main
        'app has better oppurtunity to interrupt and do it's job.
        If PulsarConnected Then
            'If we are here the init command succeeded, so there is a pulsar.
            'Now assume all of the commands that depend on a pulsar will succeed
            'for the remainder of the sub.
            Select Case Iteration
                Case 0 'get pulsar data
                    e_code = api_get_channel_config(Port, byPollChan, _
                                                     API_CHAN_CONFIG_DEFAULT, _
                                                     PulsarSettings.Settings(byPollChan - 1))
                    If e_code <> 0 Then
                        CmdFailCount += 1
                    Else
                        'myParent.Invoke(myParent.HandleConnectDelegate)
                        CmdFailCount = 0
                        byPollChan += 1
                        If byPollChan > NUM_CHANNELS Then
                            byPollChan = 1
                            Iteration += 1
                            myParent.BeginInvoke(myParent.CheckSettingsDelegate)
                        End If
                        If (UserData.Supply And API_SUPPLY_VIN_ON) = 0 Then
                            PulsarSettings.Settings(0).current = 0
                            PulsarSettings.Settings(1).current = 0
                        Else
                        End If
                    End If
                Case 1 'Check max pulse current
                    Dim intMaxCurrent As Single
                    e_code = api_calc_max_current(Port, byPollChan, 200, intMaxCurrent)
                    If e_code <> 0 Then
                        CmdFailCount += 1
                    Else
                        'Unable to directly detect lighthead connection, using this as indicator
                        If (intMaxCurrent = 0) Then
                            PulsarSettings.Lighthead(byPollChan - 1).connected = False
                        Else
                            PulsarSettings.Lighthead(byPollChan - 1).connected = True
                        End If
                        CmdFailCount = 0
                        byPollChan += 1
                        If byPollChan > NUM_CHANNELS Then
                            byPollChan = 1
                            Iteration += 1
                        End If
                        If (UserData.Supply And API_SUPPLY_VIN_ON) = 0 Then
                            PulsarSettings.Settings(0).current = 0
                            PulsarSettings.Settings(1).current = 0
                        Else
                        End If
                    End If


                Case 2 'get pulsar connection status
                    Dim sig_model As Byte
                    Dim sig_id As Byte
                    Dim partnum As New String("")
                    Dim sernum As New String("")

                    partnum = LSet(partnum, 256)
                    sernum = LSet(sernum, 256)
                    e_code = api_get_light_data(Port, sig_model, sig_id, partnum, sernum)
                    '    e_code = api_get_user_data(Port, tempUserData.Rotary, _
                    '                                     tempUserData.Dip, _
                    '                                     tempUserData.Trigger, _
                    '                                     tempUserData.Connect, _
                    '                                     tempUserData.Supply, _
                    '                                     tempUserData.Expansion)
                    '    If e_code <> 0 Then
                    '        CmdFailCount += 1
                    '    Else
                    '        CmdFailCount = 0
                    '        If (tempUserData <> UserData) Then
                    '            myParent.BeginInvoke(myParent.HandleConnectDelegate)
                    '            UserData = tempUserData
                    '        End If
                    '        If (UserData.Supply And API_SUPPLY_VIN_ON) <> 0 Then
                    '            PulsarSettings.Settings(0).current = 0
                    '            PulsarSettings.Settings(1).current = 0
                    '        Else
                    '            Iteration += 1
                    '        End If
                    '    End If
                    'Case 2
                    '    e_code = api_get_user_data(Port, tempUserData.Rotary, _
                    '                                     tempUserData.Dip, _
                    '                                     tempUserData.Trigger, _
                    '                                     tempUserData.Connect, _
                    '                                     tempUserData.Supply, _
                    '                                     tempUserData.Expansion)
                    '    If e_code <> 0 Then
                    '        CmdFailCount += 1
                    '    Else
                    '        CmdFailCount = 0
                    '        If (tempUserData <> UserData) Then
                    '            myParent.BeginInvoke(myParent.HandleConnectDelegate)
                    '            UserData = tempUserData
                    '        End If
                    '        If (UserData.Supply And API_SUPPLY_VIN_ON) <> 0 Then
                    '            PulsarSettings.Settings(0).current = 0
                    '            PulsarSettings.Settings(1).current = 0
                    '        Else
                    '            '                            Iteration += 1
                    '            Iteration = 0
                    '        End If
                    '    End If
                    Iteration = 0
            End Select

        End If
        If CmdFailCount > 3 Then
            ' Signal disconnect to parent
            api_get_error_text(e_code, e_text, e_text.Length)
            PulsarConnected = False
            PulsarSettings.Lighthead(0).connected = False
            PulsarSettings.Lighthead(1).connected = False
            myParent.BeginInvoke(myParent.CheckSettingsDelegate)
        End If

    End Function
    Sub SetBusy(ByVal bState As Boolean)
        If bState = True Then
            'Issue request and wait for it to be granted
            EvMainReq.Set()
            'Ensure Polling thread is stopped
            While EvThreadRel.WaitOne(1, False)
            End While
        Else
            EvMainReq.Reset()
        End If
    End Sub
    Sub saveSettings()
        Dim binFor As New BinaryFormatter()
        Dim obFile As FileStream = File.Create("Demo.bin")
        binFor.Serialize(obFile, PulsarSettings)
        obFile.Close()
    End Sub
    Sub loadSettings()
        Dim binFor As New BinaryFormatter()
        Dim obFile As FileStream = File.Open("Demo.bin", FileMode.Open)
        'PCSettings = binFor.Deserialize(obFile)
        obFile.Close()
    End Sub
    Sub ProgramDevice(ByVal settings As API_CHAN_CONFIG_STRUCT_T, ByVal p_chan As Byte, ByVal p_changeCurrent As Boolean)
        Dim errcode As Integer
        Dim maxCurrent As Single
        Dim maxWidth As Integer
        Dim maxFreq As Integer
        Dim minWidth As Integer
        Dim currentDelta As Single
        Dim widthDelta As Single

        SetBusy(True)
        If (p_changeCurrent) Then
            errcode = api_calc_max_current(Port, p_chan, settings.width, maxCurrent)
            settings.current = Math.Min(Math.Round(maxCurrent - 0.005, 2), settings.current)
            'settings.current = Math.Min(Math.Round(maxCurrent - 0.5, 2), settings.current)
        Else
            errcode = api_calc_max_width(Port, p_chan, settings.current, maxWidth)
            settings.width = Math.Min(maxWidth, settings.width)
        End If
        If (settings.mode = API_MODE_E.API_MODE_REPEAT) Then
            errcode = api_calc_max_freq(Port, p_chan, settings.width, settings.current, maxFreq)

            If (p_changeCurrent) Then
                'binary search for optimum current
                currentDelta = settings.current / 2
                settings.current = settings.current / 2
                While (currentDelta > 0.005) And (settings.current > 0)
                    errcode = api_calc_max_freq(Port, p_chan, settings.width, settings.current, maxFreq)
                    currentDelta = currentDelta / 2
                    If ((settings.offtime + settings.width) < ((1 / maxFreq) * 1000000)) Then
                        settings.current = settings.current - currentDelta
                    Else
                        settings.current = settings.current + currentDelta
                    End If
                End While
            Else
                'binary search for optimum width
                settings.width = 100000
                widthDelta = settings.width / 2
                settings.width = settings.width / 2
                While widthDelta > 0.5
                    errcode = api_calc_max_freq(Port, p_chan, settings.width, settings.current, maxFreq)
                    If ((settings.offtime + settings.width) < ((1 / maxFreq) * 1000000)) Then
                        settings.width = settings.width + widthDelta
                    Else
                        settings.width = settings.width - widthDelta
                    End If
                    widthDelta = widthDelta / 2
                End While
            End If




        End If
        settings.current = Math.Round(settings.current - 0.005, 2)
        settings.width = Math.Round(settings.width - 0.5, 0)
        If (settings.current > 50) Then
            settings.current = 50
        End If
        If (settings.current < 0) Then
            settings.current = 0
        End If
        If (settings.width > 100000) Then
            settings.width = 100000
        End If
        If (settings.width < 0) Then
            settings.width = 0
        End If

        If (errcode = 0) Then
            errcode = api_set_channel_config(Port, p_chan, API_CHAN_CONFIG_DEFAULT, settings)
        End If
        If (errcode = 0) Then
            errcode = api_set_channel_config(Port, p_chan, API_CHAN_CONFIG_ACTIVE, settings)
        End If
        postError(p_chan, errcode)
        byPollChan = 1
        SetBusy(False)
    End Sub
    Function GetMaxCurrent(ByVal p_Chan As Byte, ByVal p_Width As Integer) As Single
        Dim errcode As Integer
        Dim current As Single
        SetBusy(True)
        errcode = api_calc_max_current(Port, p_Chan, p_Width, current)
        postError(p_Chan, errcode)
        SetBusy(False)
        If errcode <> 0 Then
            current = 0
        End If
        Return current
    End Function
    Public Sub GetSettings(ByRef p_Settings As API_CHAN_CONFIG_STRUCT_T, ByVal chan As Byte)
        p_Settings = PulsarSettings.Settings(chan - 1)
        If (p_Settings.current < 0) Then
            p_Settings.current = 0
        End If
        If (p_Settings.width < 0) Then
            p_Settings.width = 0
        End If
        If (p_Settings.Delay < 0) Then
            p_Settings.Delay = 0
        End If
    End Sub
    Public Function GetVersion(ByRef fw_version As String, ByRef hw_version As String, ByRef serial_num As String) As Short
        fw_version = PulsarFWVersion
        hw_version = PulsarHWVersion
        serial_num = PulsarSerialNumber
    End Function
    Public Function GetIPSettings(ByRef ipAddr As String, ByRef subnetAddr As String, ByRef gatewayAddr As String, ByRef MACAddr As String) As Short
        ipAddr = PulsarIPAddr
        subnetAddr = PulsarSubnet
        gatewayAddr = PulsarGateway
        MACAddr = PulsarMAC
    End Function
    Public Function SetIPSettings(ByRef ipAddr As String, ByRef subnetAddr As String, ByRef gatewayAddr As String) As Short
        Dim errcode As Short
        SetBusy(True)
        api_reset_default_tcp(Port)
        errcode = api_set_pulsar_ipaddr(Port, ipAddr, subnetAddr, gatewayAddr)
        postError(0, errcode)
        SetBusy(False)
    End Function
    Public Function GetLightheadStatus() As Short
        Dim returnVal As Short
        returnVal = 0
        If (PulsarSettings.Lighthead(0).connected) Then
            returnVal = 1
        End If
        If (PulsarSettings.Lighthead(1).connected) Then
            returnVal = returnVal Or 2
        End If
        Return returnVal
    End Function

    Sub StopPulsar()
        PollThread.Abort()
    End Sub
    Sub ProgramSetupMode(ByVal p_chan As Byte)
        Dim errcode As Short
        Dim tempSettings As API_CHAN_CONFIG_STRUCT_T
        Dim maxCurrent As Single
        tempSettings.mode = API_MODE_E.API_MODE_INTERNAL_TRIGGER
        tempSettings.current = 0.1
        tempSettings.width = 100000
        SetBusy(True)
        errcode = api_calc_max_current(Port, p_chan, tempSettings.width, maxCurrent)
        tempSettings.current = Math.Round((maxCurrent / 2), 2)
        errcode = api_set_channel_config(Port, p_chan, API_CHAN_CONFIG_ACTIVE, tempSettings)
        SetBusy(False)
        postError(p_chan, errcode)
    End Sub
    Sub GetMaxFreq(ByVal p_chan As Byte, ByVal p_width As Integer, ByVal p_current As Single, ByRef p_maxFreq As Short)
        Dim errcode As Short
        errcode = api_calc_max_freq(Port, p_chan, p_width, p_current, p_maxFreq)
    End Sub
    Sub Trigger(ByVal p_chan As Byte, ByVal p_trigger As Byte)
        Dim errcode As Short
        SetBusy(True)
        errcode = api_trigger(Port, p_chan + 1)
        postError(p_chan, errcode)
        SetBusy(False)
    End Sub
    Sub GetSettings(ByRef slewRate() As Byte, ByRef diffTrigger() As Byte, ByRef limit48v As Byte, ByRef disableProt As Byte)
        Dim errcode As Short
        SetBusy(True)
        errcode = api_get_configuration(Port, slewRate(0), diffTrigger(0), limit48v, disableProt)
        postError(255, errcode)
        SetBusy(False)

    End Sub
    Sub SetSettings(ByVal slewRate() As Byte, ByVal diffTrigger() As Byte, ByVal limit48v As Byte, ByVal disableProt As Byte)
        Dim errcode As Short
        SetBusy(True)

        api_set_configuration(Port, slewRate(0), diffTrigger(0), limit48v, disableProt)
        '        Me.DialogResult = System.Windows.Forms.DialogResult.OK
        '        Me.Close()
        postError(255, errcode)
        SetBusy(False)
    End Sub
    Sub postError(ByVal p_chan As Byte, ByVal p_ecode As Short)
        Dim errName As String = ""
        Dim errDesc As String = ""
        errName = LSet(" ", 500)
        errDesc = LSet(" ", 500)
        If (p_ecode <> 0) Then
            'Get error info
            api_get_error_name(p_ecode, errName, errName.Length)
            errName = errName.Trim()
            api_get_error_text(p_ecode, errDesc, errDesc.Length)
            errDesc = errDesc.Trim()
            RaiseEvent evError(p_chan, p_ecode, errName, errDesc)
        End If
    End Sub
End Class
