﻿Public Class ChannelConfig
    Dim CurrentScales() As Short = {50, 25, 10, 5, 2, 1}
    Dim PWScales() As Integer = {100000, 50000, 25000, 10000, 5000, 2500, 1000, 500, 250, 100, 50}
    Dim DelayScales() As Short = {1000, 500, 250, 100, 50}
    Dim Settings As API_CHAN_CONFIG_STRUCT_T
    Public Event evSettingsChanged(ByVal channel As ChannelConfig)
    Public Event evTrigger(ByRef p_channel As ChannelConfig, ByVal trigger As Byte)
    Public Structure SCALE_VALUES
        Dim current As Integer
        Dim pw As Integer
        Dim delay As Integer
    End Structure
    Private Sub Trigger_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Trigger1.CheckedChanged, Trigger2.CheckedChanged, TriggerNone.CheckedChanged
        If (Trigger1.Checked) Then
            Settings.trigger = API_TRIG_E.API_TRIG_1
        ElseIf Trigger2.Checked Then
            Settings.trigger = API_TRIG_E.API_TRIG_2
        ElseIf TriggerNone.Checked Then
            Settings.trigger = API_TRIG_E.API_TRIG_NONE
        End If
        RaiseEvent evSettingsChanged(Me)
    End Sub
    Private Sub TriggerEdge_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles EdgeRising.CheckedChanged, EdgeFalling.CheckedChanged
        If (EdgeRising.Checked) Then
            Settings.rising = 1
        Else
            Settings.rising = 0
        End If
        RaiseEvent evSettingsChanged(Me)
    End Sub
    Private Sub Mode_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ModeTimingBypass.CheckedChanged, ModeTest.CheckedChanged, ModeRepeat.CheckedChanged, ModePulsed.CheckedChanged, ModeSetup.CheckedChanged
        If (ModePulsed.Checked) Then
            Settings.mode = API_MODE_E.API_MODE_PULSED
        ElseIf ModeTimingBypass.Checked Then
            Settings.mode = API_MODE_E.API_MODE_TIMING_BYPASS
        ElseIf ModeTest.Checked Then
            Settings.mode = API_MODE_E.API_MODE_INTERNAL_TRIGGER
        ElseIf ModeRepeat.Checked Then
            Settings.mode = API_MODE_E.API_MODE_REPEAT
        End If
        If ModeRepeat.Checked Then
            If Settings.repeat_cnt < 0 Then
                RepeatCount.Text = Settings.repeat_cnt + 65536
            Else
                RepeatCount.Text = Settings.repeat_cnt
            End If
            RepeatOff.Text = Settings.offtime
            RepeatCount.Enabled = True
            RepeatOff.Enabled = True
        Else
            RepeatCount.Enabled = False
            RepeatOff.Enabled = False
        End If
        RaiseEvent evSettingsChanged(Me)
    End Sub
    Private Sub SetRange(ByVal highText As Label, ByVal slider As TrackBar, ByVal scale As Array, ByVal index As Integer)
        If index > scale.Length Then
            index = scale.Length - 1
        End If
        highText.Text = scale(index).ToString()
        If scale(0) = CurrentScales(0) Then
            slider.Maximum = scale(index) * 100
        Else
            slider.Maximum = scale(index)
        End If
        slider.LargeChange = slider.Maximum / 5
        slider.SmallChange = slider.Maximum / 25
        slider.TickFrequency = slider.SmallChange
    End Sub
    Private Sub DomainUpDown_SelectedItemChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CurrentScale.SelectedItemChanged, PWScale.SelectedItemChanged, DelayScale.SelectedItemChanged
        If CType(sender, DomainUpDown).SelectedIndex >= 0 Then

            Select Case CType(sender, DomainUpDown).Name
                Case "CurrentScale"
                    SetRange(CurrentHigh, CurrentSlider, CurrentScales, CurrentScale.SelectedIndex)
                    CurrentValue.Text = CurrentSlider.Value / 100
                Case "PWScale"
                    SetRange(PWHigh, PWSlider, PWScales, PWScale.SelectedIndex)
                    PWValue.Text = PWSlider.Value
                Case "DelayScale"
                    SetRange(DelayHigh, DelaySlider, DelayScales, DelayScale.SelectedIndex)
                    DelayValue.Text = DelaySlider.Value
            End Select
        End If
        RaiseEvent evSettingsChanged(Me)
    End Sub

    Public Sub New()
        Dim Index As Integer
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        CurrentScale.Items.Clear()
        For Index = 0 To CurrentScales.Length - 1
            CurrentScale.Items.Add("Scale " & CurrentScales(Index))
        Next
        PWScale.Items.Clear()
        For Index = 0 To PWScales.Length - 1
            PWScale.Items.Add("Scale " & PWScales(Index))
        Next
        DelayScale.Items.Clear()
        For Index = 0 To DelayScales.Length - 1
            DelayScale.Items.Add("Scale " & DelayScales(Index))
        Next
        CurrentScale.SelectedIndex = 0
        PWScale.SelectedIndex = 0
        DelayScale.SelectedIndex = 0
    End Sub
    Private Sub Slider_Scroll(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CurrentSlider.Scroll, PWSlider.Scroll, DelaySlider.Scroll
        Dim value As Integer
        value = CType(sender, TrackBar).Value
        Select Case CType(sender, TrackBar).Name
            Case "CurrentSlider"
                CurrentValue.Text = value / 100
                Settings.current = value / 100
            Case "PWSlider"
                PWValue.Text = value
                Settings.width = value
            Case "DelaySlider"
                DelayValue.Text = value
                Settings.Delay = value
        End Select
        '        RaiseEvent evSettingsChanged(Me)
    End Sub
    Private Sub Slider_ScrollDone(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CurrentSlider.MouseUp, PWSlider.MouseUp, DelaySlider.MouseUp
        RaiseEvent evSettingsChanged(Me)
    End Sub

    Private Sub ChannelConfig_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        SetRange(CurrentHigh, CurrentSlider, CurrentScales, CurrentScale.SelectedIndex)
        SetRange(PWHigh, PWSlider, PWScales, PWScale.SelectedIndex)
        SetRange(DelayHigh, DelaySlider, DelayScales, DelayScale.SelectedIndex)
        Trigger_CheckedChanged(Me, System.EventArgs.Empty)
        TriggerEdge_CheckedChanged(Me, System.EventArgs.Empty)
        Mode_CheckedChanged(Me, System.EventArgs.Empty)
    End Sub
    Public Sub GetSettings(ByRef p_Settings As API_CHAN_CONFIG_STRUCT_T)
        p_Settings = Settings
    End Sub
    Public Sub SetSettings(ByRef p_Settings As API_CHAN_CONFIG_STRUCT_T)
        Settings = p_Settings
        UpdateControls(Settings)
    End Sub
    Public Sub GetScales(ByRef p_Scales As SCALE_VALUES)
        p_Scales.current = CurrentScale.SelectedIndex
        p_Scales.pw = PWScale.SelectedIndex
        p_Scales.delay = DelayScale.SelectedIndex
    End Sub
    Public Sub SetScales(ByRef p_Scales As SCALE_VALUES)
        CurrentScale.SelectedIndex = p_Scales.current
        PWScale.SelectedIndex = p_Scales.pw
        DelayScale.SelectedIndex = p_Scales.delay
    End Sub
    Public Sub UpdateControls(ByVal p_Settings As API_CHAN_CONFIG_STRUCT_T)
        CurrentValue.Text = p_Settings.current
        PWValue.Text = p_Settings.width
        DelayValue.Text = p_Settings.Delay
        CurrentValue_TextChanged(CurrentValue, System.EventArgs.Empty)
        IntTextBox_TextChanged(PWValue, System.EventArgs.Empty)
        IntTextBox_TextChanged(DelayValue, System.EventArgs.Empty)
        CurrentValue_KeyUp(CurrentValue, System.EventArgs.Empty)
        IntTextBox_KeyUp(PWValue, System.EventArgs.Empty)
        IntTextBox_KeyUp(DelayValue, System.EventArgs.Empty)
        'Set trigger
        If (p_Settings.trigger = API_TRIG_E.API_TRIG_1) Then
            Trigger1.Checked = True
        ElseIf p_Settings.trigger = API_TRIG_E.API_TRIG_2 Then
            Trigger2.Checked = True
        ElseIf p_Settings.trigger = API_TRIG_E.API_TRIG_NONE Then
            TriggerNone.Checked = True
        End If
        'Set trigger edge
        If p_Settings.rising = 1 Then
            EdgeRising.Checked = True
            EdgeFalling.Checked = False
        ElseIf p_Settings.rising = 0 Then
            EdgeFalling.Checked = True
            EdgeRising.Checked = False
        Else
            EdgeFalling.Checked = False
            EdgeRising.Checked = False
        End If
        'Set mode
        If p_Settings.mode = API_MODE_E.API_MODE_PULSED Then
            ModePulsed.Checked = True
        ElseIf p_Settings.mode = API_MODE_E.API_MODE_TIMING_BYPASS Then
            ModeTimingBypass.Checked = True
        ElseIf p_Settings.mode = API_MODE_E.API_MODE_INTERNAL_TRIGGER Then
            ModeTest.Checked = True
        ElseIf p_Settings.mode = API_MODE_E.API_MODE_REPEAT Then
            ModeRepeat.Checked = True
        End If
        If ModeRepeat.Checked Then
            If Settings.repeat_cnt < 0 Then
                RepeatCount.Text = p_Settings.repeat_cnt + 65536
            Else
                RepeatCount.Text = p_Settings.repeat_cnt
            End If
            RepeatOff.Text = p_Settings.offtime
            RepeatCount.Enabled = True
            RepeatOff.Enabled = True
        Else
            RepeatCount.Enabled = False
            RepeatOff.Enabled = False
        End If
    End Sub

    Private Sub CurrentValue_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles CurrentValue.KeyPress
        Select Case Char.GetUnicodeCategory(e.KeyChar)
            Case Globalization.UnicodeCategory.Control, Globalization.UnicodeCategory.DecimalDigitNumber
            Case Else
                If e.KeyChar = "." Then
                    CurrentValue.Select(CurrentValue.Text.IndexOf(".") + 1, 0)
                Else
                    e.Handled = True

                End If
        End Select
    End Sub
    Private Sub IntValue_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles CurrentValue.KeyPress, PWValue.KeyPress, DelayValue.KeyPress, RepeatCount.KeyPress, RepeatOff.KeyPress
        Select Case Char.GetUnicodeCategory(e.KeyChar)
            Case Globalization.UnicodeCategory.Control, Globalization.UnicodeCategory.DecimalDigitNumber
            Case Else
                e.Handled = True
        End Select
    End Sub
    Private Function GetSliderControl(ByVal name As String) As TrackBar
        Dim bar As TrackBar
        If name.Contains("current") Then
            bar = CurrentSlider
        ElseIf name.Contains("delay") Then
            bar = DelaySlider
        Else
            bar = PWSlider
        End If
        Return bar
    End Function
    Private Sub CurrentValue_TextChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles CurrentValue.TextChanged
        Dim value As Single
        Try
            value = CurrentValue.Text
        Catch ex As System.InvalidCastException
            value = CurrentSlider.Value / 100
        End Try

        If value > CurrentScales(0) Then
            value = CurrentScales(0)
        End If

        Settings.current = CurrentValue.Text
    End Sub
    Private Sub CurrentValue_KeyUp(ByVal sender As Object, ByVal e As System.EventArgs) Handles CurrentValue.KeyUp
        Dim value As Single
        Dim tempPos As Integer
        Try
            value = CurrentValue.Text
        Catch ex As System.InvalidCastException
            value = CurrentSlider.Value / 100
        End Try
        'If value > 50 Then
        '    value = 50
        'End If

        tempPos = CurrentValue.SelectionStart()
        'SetValue(CurrentValue.Text, CurrentSlider, CurrentScales, Nothing)
        If value > CurrentScales(0) Then
            value = CurrentScales(0)
        End If
        While value * 100 > CurrentSlider.Maximum And CurrentScale.SelectedIndex > 0
            CurrentScale.SelectedIndex -= 1
            SetRange(CurrentHigh, CurrentSlider, CurrentScales, CurrentScale.SelectedIndex)
        End While
        'If value > CurrentSlider.Maximum Then
        '    value = CurrentSlider.Maximum
        'End If
        CurrentSlider.Value = value * 100

        Settings.current = CurrentValue.Text
        CurrentValue.Text = Format(value, "##.00")
        CurrentValue.SelectionStart = tempPos





        'tempPos = PWValue.SelectionStart()
        'LightheadData.Text += tempPos.ToString + "\n"
        'CurrentValue.Text = Format(value, "##.00")
        'CurrentValue.SelectionStart = tempPos
        'Settings.current = CurrentValue.Text
        'While value * 100 > CurrentSlider.Maximum
        '    CurrentScale.SelectedIndex -= 1
        '    SetRange(CurrentHigh, CurrentSlider, CurrentScales, CurrentScale.SelectedIndex)
        'End While
        'CurrentSlider.Value = value * 100

    End Sub
    'Private Sub SetValue(ByVal value As Integer, ByVal slider As TrackBar, ByVal scales As Array, ByRef scale As DomainUpDown)
    '        If Not IsNothing(scale) Then
    ' If scale.Items.Count > 0 Then
    ' scale.SelectedIndex = SelectScale(scales, value)
    'If scale.SelectedIndex = -1 Then
    ' scale.SelectedIndex = 0
    'value = scales(0)
    'scale.Text = value
    'End If
    'DomainUpDown_SelectedItemChanged(scale, System.EventArgs.Empty)

    'End If
    'End If
    '        If value > slider.Maximum Then
    '            value = slider.Maximum
    '        End If
    '
    '   slider.Value = value
    'End Sub
    Private Sub IntTextBox_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PWValue.TextChanged, DelayValue.TextChanged, RepeatCount.TextChanged, RepeatOff.TextChanged
        Dim value As Integer
        Dim tempPos As Integer
        Try
            value = CType(sender, TextBox).Text
        Catch except As InvalidCastException
            value = 0
        End Try
        CType(sender, TextBox).Text = value
        Select Case CType(sender, TextBox).Name
            Case "PWValue"
                If value > PWScales(0) Then
                    value = PWScales(0)
                End If
                Settings.width = value
            Case "DelayValue"
                If value > DelayScales(0) Then
                    value = DelayScales(0)
                End If
                Settings.Delay = value
            Case "RepeatCount"
                If value > 65535 Then
                    value = 65535
                End If
                Settings.repeat_cnt = value
            Case "RepeatOff"
                Settings.offtime = value
        End Select
    End Sub
    Private Sub IntTextBox_KeyUp(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PWValue.KeyUp, DelayValue.KeyUp, RepeatCount.KeyUp, RepeatOff.KeyUp
        Dim value As Integer
        Dim tempPos As Integer
        Try
            value = CType(sender, TextBox).Text
        Catch except As InvalidCastException
            value = 0
        End Try
        CType(sender, TextBox).Text = value
        Select Case CType(sender, TextBox).Name
            Case "PWValue"
                tempPos = PWValue.SelectionStart()
                'SetValue(PWValue.Text, PWSlider, PWScales, Nothing)
                If value > PWScales(0) Then
                    value = PWScales(0)
                End If
                While value > PWSlider.Maximum And PWScale.SelectedIndex > 0
                    PWScale.SelectedIndex -= 1
                    SetRange(PWHigh, PWSlider, PWScales, PWScale.SelectedIndex)
                End While
                'If value > PWSlider.Maximum Then
                '    value = PWSlider.Maximum
                'End If
                PWSlider.Value = value

                Settings.width = PWValue.Text
                PWValue.Text = PWSlider.Value
                PWValue.SelectionStart = tempPos
            Case "DelayValue"
                'SetValue(DelayValue.Text, DelaySlider, DelayScales, Nothing)
                'If value > DelaySlider.Maximum Then
                '    value = DelaySlider.Maximum
                'End If
                If value > DelayScales(0) Then
                    value = DelayScales(0)
                End If
                While value > DelaySlider.Maximum And DelayScale.SelectedIndex > 0
                    DelayScale.SelectedIndex -= 1
                    SetRange(DelayHigh, DelaySlider, DelayScales, DelayScale.SelectedIndex)
                End While
                DelaySlider.Value = value
                Settings.Delay = DelayValue.Text
                DelayValue.Text = DelaySlider.Value
                DelayValue.SelectionStart = DelayValue.Text.Length
            Case "RepeatCount"
                If value > 65535 Then
                    value = 65535
                    RepeatCount.Text = value
                    MsgBox("Maximum value is 65535.", MsgBoxStyle.Exclamation, "Maximum exceeded")
                End If
                Settings.repeat_cnt = value
            Case "RepeatOff"
                Settings.offtime = value
        End Select
        '        RaiseEvent evSettingsChanged(Me)
    End Sub
    Private Sub IntTextBox_LostFocus(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CurrentValue.LostFocus, PWValue.LostFocus, DelayValue.LostFocus, RepeatCount.LostFocus, RepeatOff.LostFocus
        RaiseEvent evSettingsChanged(Me)
    End Sub
    Private Function SelectScale(ByVal scales As Array, ByVal value As Double) As Integer
        Dim unI As Integer
        For unI = 0 To scales.Length - 1
            If scales(unI) < value Then
                Return unI - 1
            End If
        Next
        Return unI - 1
    End Function
    Public Sub AdjustScales()
        CurrentScale.SelectedIndex = SelectScale(CurrentScales, CurrentSlider.Value / 100)
        PWScale.SelectedIndex = SelectScale(PWScales, PWSlider.Value)
        DelayScale.SelectedIndex = SelectScale(DelayScales, DelaySlider.Value)
    End Sub

    '    Private Sub CurrentValue_Validating(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles CurrentValue.Validating
    '        RaiseEvent evSettingsChanged(Me)
    '    End Sub
    Public Sub SetMaxCurrent(ByVal p_Current As Single)
        MaxCurrent.Text = p_Current
    End Sub

    Private Sub TestTrigger_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TestTrigger.Click
        Dim trigger As Byte
        If Trigger1.Checked Then
            trigger = 1
        Else
            trigger = 2
        End If

        RaiseEvent evTrigger(Me, trigger)
    End Sub

    Private Sub SettingsChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PWValue.LostFocus, DelayValue.LostFocus, CurrentValue.LostFocus, RepeatCount.LostFocus, RepeatOff.LostFocus
        RaiseEvent evSettingsChanged(Me)
    End Sub
End Class
