Option Strict Off
Option Explicit On 
Imports System.Runtime.InteropServices

Module PulsarAPI
    'Attribute VB_Name = "PulsarAPI"
    '********************************************************************************
    '********************************************************************************
    '
    '  COPYRIGHT  2004, Advanced Illumination, Inc.
    '
    '  Contains confidential and proprietary information which may not be copied,
    '  disclosed or used by others except as expressly authorized in writing
    '  by Advanced Illumination.
    '
    '  PulsarAPI.h    External definitions for the High Level User Application Programming Interface
    '
    '  About:         This module contains external definitions for the High Level User Application
    '                 Programming Interface DLL functions in the AICONS01 project.
    '
    '  Target:        Win32 Compatible
    '
    '  Written for:   Advanced Illumination, Inc.
    '                 24 Peavine Drive
    '                 Rochester, VT 05767
    '                 http://advancedillumination.com
    '
    '  Contacts:      John Thrailkill
    '                 Phone:  (802) 767-3830
    '                 FAX:    (802) 767-3831
    '                 jthrailkill@ advill.com
    '
    '  Written By:    Rob Macklin
    '                 Microprocessor Designs, Inc.
    '                 65 Longmeadow Drive
    '                 PO Box 160
    '                 Shelburne, VT 05482
    '                 http://www.updesigns.com
    '
    '  $Revision:  $
    '
    '-------------------------------------------------------------------------------
    '  $Log:  $
    '*
    '*******************************************************************************
    '*******************************************************************************

    Public Const API_PULSAR_PRODUCT_NAME As String = "Ai S1000 Strobe Controller"

    '    Public Structure API_CHAN_CONFIG_STRUCT_T
    '    Public trigger As Byte
    '    Public rising As Byte
    '    Public mode As Byte
    '    Public reserved_byte As Byte
    '    Public width As Integer
    '    Public Delay As Integer
    '    Public current As Single
    '    Public reserved(8) As Byte
    '    End Structure

    <StructLayout(LayoutKind.Sequential)> Public Structure API_CHAN_CONFIG_STRUCT_T
        Dim trigger As Byte
        Dim rising As Byte
        Dim mode As Byte
        Dim reserved_byte As Byte
        Dim width As Short
        Dim delay As Short
        Dim current As Single
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=8)> Public _
        reserved() As Byte
    End Structure

    ' Define possible operating modes
    Public Enum API_MODE_E
        API_MODE_PULSED = 0
        API_MODE_CONTINUOUS
        API_MODE_TIMING_BYPASS
        API_MODE_INTERNAL_TRIGGER
    End Enum

    Public Const API_NUM_MODE As Byte = 4

    ' Define the drive channel identifiers used with the API interface
    Public Enum API_CHAN_E
        API_CHAN_1 = 1
        API_CHAN_2
        API_CHAN_3
        API_CHAN_4
    End Enum

    Public Const API_NUM_CHAN As Byte = 4

    ' Define the trigger input identifiers used with the API interface
    Public Enum API_TRIG_E
        API_TRIG_NONE = 0
        API_TRIG_1
        API_TRIG_2
        API_TRIG_3
        API_TRIG_4
    End Enum

    Public Const API_NUM_TRIG As Byte = 4

    ' Define identifiers used to handle how channel configuration data is used
    Public Const API_CHAN_CONFIG_DEFAULT As Byte = 1
    Public Const API_CHAN_CONFIG_ACTIVE As Byte = 2

    ' Define identifiers for diagnostics tests
    Public Enum API_DIAG_E
        API_POWER_TEST = 0
        API_RAM_TEST
        API_FLASH_TEST
        API_EXTIO_TEST
        API_EEPROM_TEST
        API_EXT_DAC_TEST
        API_SCI_TEST
        API_NUM_TESTS
    End Enum

    ' Optional diagnostics tests do not normally run on startup of the Pulsar
    ' They require additional hardware beyond the PCB Assembly itself to complete
    ' The optional SCI test requires a serial loopback connector between the external
    ' modular connector and the internal 10-pin header
    Public Const API_NUM_OPTIONAL_TESTS As Byte = 1


    ' Define bit identifiers for the information returned by the api_get_user_data command

    ' The rotary switch value is returned within a range of 0 to API_MAX_ROTARY
    Public Const API_MAX_ROTARY As Integer = 999

    ' Each bit is associated with one DIP switch position
    Public Const API_DIP_MODE0 As Byte = &H1
    Public Const API_DIP_MODE1 As Byte = &H2
    Public Const API_DIP_MODE2 As Byte = &H4
    Public Const API_DIP_MODE3 As Byte = &H8

    ' Each bit is associated with one TRIG input
    Public Const API_TRIG_IN1 As Byte = &H1
    Public Const API_TRIG_IN2 As Byte = &H2
    Public Const API_TRIG_IN3 As Byte = &H4
    Public Const API_TRIG_IN4 As Byte = &H8

    ' Connection state of lighthead, USB, and RS-232
    Public Const API_CONN_LH_DETECT As Byte = &H1
    Public Const API_CONN_USBDETECT As Byte = &H2
    Public Const API_CONN_PCPRESENT As Byte = &H4
    Public Const API_CONN_PC_READY As Byte = &H8

    ' Operating state of power supplies
    Public Const API_SUPPLY_VIN_ON As Byte = &H1
    Public Const API_SUPPLY_LV_ON As Byte = &H2
    Public Const API_SUPPLY_HV_ON As Byte = &H4

    ' Additional expansion general purpose inputs
    Public Const API_EXP_GPIN1 As Byte = &H1
    Public Const API_EXP_GPIN2 As Byte = &H2
    Public Const API_EXP_GPINOUT1 As Byte = &H4
    Public Const API_EXP_GPINOUT2 As Byte = &H8

    ' Define analog input identifiers used with api_measure_voltage command
    Public Const API_AI_VIN_SWMON As Byte = &H1
    Public Const API_AI_V_HV_STRINGMON As Byte = &H2
    Public Const API_AI_V_LV_STRINGMON As Byte = &H3
    Public Const API_AI_VCH1MON As Byte = &H4
    Public Const API_AI_VCH2MON As Byte = &H5
    Public Const API_AI_VCH3MON As Byte = &H6
    Public Const API_AI_VCH4MON As Byte = &H7
    Public Const API_AI_GND1 As Byte = &H8
    Public Const API_AI_SIG1_ID1_MON As Byte = &H9
    Public Const API_AI_SIG1_ID2_MON As Byte = &HA
    Public Const API_AI_CH1_DRV_LVL As Byte = &HB
    Public Const API_AI_CH2_DRV_LVL As Byte = &HC
    Public Const API_AI_CH3_DRV_LVL As Byte = &HD
    Public Const API_AI_CH4_DRV_LVL As Byte = &HE
    Public Const API_AI_GND2 As Byte = &HF
    Public Const API_AI_10VMON As Byte = &H10
    Public Const API_AI_CH1CURRSENS As Byte = &H11
    Public Const API_AI_CH2CURRSENS As Byte = &H12
    Public Const API_AI_CH3CURRSENS As Byte = &H13
    Public Const API_AI_CH4CURRSENS As Byte = &H14
    Public Const API_AI_INT_TEMP As Byte = &H15
    Public Const API_AI_GND3 As Byte = &H16
    Public Const API_AI_33VMON As Byte = &H17
    Public Const API_AI_5VMON As Byte = &H18



    '****************************************************************************
    '  api_get_dll_version()
    '
    '  About:      Returns the current version number of this software.
    '
    '  Inputs:     None
    '
    '  Outputs:    None
    '
    '  Returns:    Integer      ver_info    High byte is major version number
    '                                       Low byte is minor version number
    '
    '****************************************************************************
    Public Declare Function api_get_dll_version Lib "PulsarAPI.dll" Alias "_api_get_dll_version@0" () As Integer

    '****************************************************************************
    '  api_get_error_name
    '
    '  About:      Returns the name of the error associated with the code
    '
    '  Inputs:     Integer     ecode       Code to be looked up
    '              Integer     namelen     Maximum length of name string
    '
    '  Outputs:    String      nameptr     Location to place name string
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '****************************************************************************
    Public Declare Function api_get_error_name Lib "PulsarAPI.dll" Alias "_api_get_error_name@12" ( _
       ByVal ECode As Integer, _
       ByVal name As String, _
    ByVal namelen As Integer) As Integer

    '****************************************************************************
    '  api_get_error_text
    '
    '  About:      Returns the text description of the error associated with the code
    '
    '  Inputs:     Integer     ecode       Code to be looked up
    '              Integer     textlen     Maximum length of text description string
    '
    '  Outputs:    String      nameptr     Location to place text description string
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '****************************************************************************
    Public Declare Function api_get_error_text Lib "PulsarAPI.dll" Alias "_api_get_error_text@12" ( _
       ByVal ECode As Integer, _
       ByVal text As String, _
    ByVal textlen As Integer) As Integer

    '****************************************************************************
    '  api_init
    '
    '  About:      Initializes the connection to a specific Pulsar
    '
    '  Inputs:     Byte        port        Identifier for device
    '              String      product_name Name of USB device to open
    '              String      serial_num  Serial number of device to open
    '
    '  Outputs:    None
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '****************************************************************************
    Public Declare Function api_init Lib "PulsarAPI.dll" Alias "_api_init@12" ( _
       ByVal Port As Byte, _
       ByVal product_name As String, _
       ByVal serial_num As String) As Integer

    '****************************************************************************
    '  api_get_pulsar_version
    '
    '  About:      Reads firmware version, hardware version, and serial number
    '              Each string can be up to 32 characters long.
    '
    '  Inputs:     Byte        port        Identifier for device
    '
    '  Outputs:    String      fw_version  Firmware version
    '              String      hw_version  Hardware version
    '              String      serial_num  Serial number

    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '****************************************************************************
    Public Declare Function api_get_pulsar_version Lib "PulsarAPI.dll" Alias "_api_get_pulsar_version@16" ( _
       ByVal Port As Byte, _
       ByVal fw_version As String, _
       ByVal hw_version As String, _
       ByVal serial_num As String) As Integer

    '****************************************************************************
    '  api_get_status
    '
    '  About:      Reads status information from the device
    '
    '  Inputs:     Byte        port        Identifier for device
    '
    '  Outputs:    None
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '****************************************************************************
    Public Declare Function api_get_status Lib "PulsarAPI.dll" Alias "_api_get_status@4" ( _
       ByVal Port As Byte) As Integer

    '****************************************************************************
    '  api_run_diags
    '
    '  About:      Runs one or all of the built in diagnostics
    '
    '  Inputs:     Byte        port        Identifier for device
    '              Byte        test_num    Desired test number or FF for all
    '
    '  Outputs:    None
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '****************************************************************************
    Public Declare Function api_run_diags Lib "PulsarAPI.dll" Alias "_api_run_diags@8" ( _
       ByVal Port As Byte, _
       ByVal test_num As Byte) As Integer


    '****************************************************************************
    '  api_get_channel_config
    '
    '  About:      Gets the configuration data for one drive channel
    '
    '  Inputs:     Byte        port        Identifier for device
    '              Byte        chan        Identifier for drive channel
    '              Byte        data_type   Defines if active or default data is returned
    '
    '  Outputs:    API_CHAN_CONFIG_STRUCT_T   config_data
    '                                      Active channel configuration data
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '****************************************************************************
    Public Declare Function api_get_channel_config Lib "PulsarAPI.dll" Alias "_api_get_channel_config@16" ( _
       ByVal Port As Byte, _
       ByVal chan As Byte, _
       ByVal data_type As Byte, _
       ByRef config_data As API_CHAN_CONFIG_STRUCT_T) As Integer


    '****************************************************************************
    '  api_set_channel_config
    '
    '  About:      Sets the configuration data for one drive channel
    '
    '  Inputs:     Byte        port        Identifier for device
    '              Byte        chan        Identifier for drive channel
    '              Byte        data_type   Defines if active or default data is returned
    '              API_CHAN_CONFIG_STRUCT_T   config_data
    '                                      New channel configuration data
    '
    '  Outputs:    None
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '****************************************************************************
    Public Declare Function api_set_channel_config Lib "PulsarAPI.dll" Alias "_api_set_channel_config@12" ( _
       ByVal Port As Byte, _
       ByVal chan As Byte, _
       ByVal data_type As Byte, _
       ByRef config_data As API_CHAN_CONFIG_STRUCT_T) As Integer

    '****************************************************************************
    '  api_trigger
    '
    '  About:      Generates a software trigger on the selected trigger channel
    '
    '  Inputs:     Byte        port       Identifier for device
    '              Byte        trig       Trigger to activate
    '
    '  Outputs:    None
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '****************************************************************************
    Public Declare Function api_trigger Lib "PulsarAPI.dll" Alias "_api_trigger@8" ( _
       ByVal Port As Byte, _
       ByVal trig As Byte) As Integer

    '*******************************************************************************
    '  api_calc_led_temp
    '
    '  About:      Calculates the expected LED junction temperature based on the
    '              information stored in the lighthead data structure and the width
    '              and current information passed.
    '
    '  Inputs:     Byte        port        Identifier for device
    '              Byte        ch          Channel to compute LED temperature
    '              Integer     width       Pulse width in microseconds
    '              Single      current     Pulse current in amps
    '
    '  Outputs:    Single      temp        Location to store calculated temperature
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '********************************************************************************
    Public Declare Function api_calc_led_temp Lib "PulsarAPI.dll" Alias "_api_calc_led_temp@20" ( _
       ByVal Port As Byte, _
       ByVal ch As Byte, _
       ByVal width As Integer, _
       ByVal current As Single, _
       ByRef temp As Single) As Integer

    '*******************************************************************************
    '  api_calc_max_width
    '
    '  About:      Calculates the maximum pulse width for the selected channel based
    '              on the information stored in the lighthead data structure and the
    '              current information passed.
    '
    '  Inputs:     Byte        port        Identifier for device
    '              Byte        ch          Channel to compute LED temperature
    '              Single      current     Pulse current in amps
    '
    '  Outputs:    Integer     width       Maximum pulse width in microseconds
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '********************************************************************************
    Public Declare Function api_calc_max_width Lib "PulsarAPI.dll" Alias "_api_calc_max_width@16" ( _
       ByVal PortID As Byte, _
       ByVal ch As Byte, _
       ByVal current As Single, _
       ByRef width As Integer) As Integer


    '*******************************************************************************
    '  api_calc_max_current
    '
    '  About:      Calculates the maximum pulse current for the selected channel based
    '              on the information stored in the lighthead data structure and the
    '              width information passed.
    '
    '  Inputs:     Byte        port        Identifier for device
    '              Byte        ch          Channel to compute LED temperature
    '              Integer     width       Pulse width in microseconds
    '
    '  Outputs:    Single      current     Maximum pulse current in amps
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '********************************************************************************
    Public Declare Function api_calc_max_current Lib "PulsarAPI.dll" Alias "_api_calc_max_current@16" ( _
       ByVal Port As Byte, _
       ByVal ch As Byte, _
       ByVal width As Integer, _
       ByRef current As Single) As Integer


    '*******************************************************************************
    '  api_calc_supply_voltage
    '
    '  About:      Calculates the required supply voltage for a given width and current.
    '
    '  Inputs:     Byte        port        Identifier for device
    '              Byte        ch          Channel to compute required supply voltage
    '              Integer     width       Pulse width in microseconds
    '              Single      current     Pulse current in amps
    '
    '  Outputs:    Single      voltage     Location to store calculated voltage
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '********************************************************************************
    Public Declare Function api_calc_supply_voltage Lib "PulsarAPI.dll" Alias "_api_calc_supply_voltage@20" ( _
       ByVal Port As Byte, _
       ByVal ch As Byte, _
       ByVal width As Integer, _
       ByVal current As Single, _
       ByRef voltage As Single) As Integer


    '*******************************************************************************
    '  api_calc_max_freq
    '
    '  About:      Calculates the maximum frequency allowed for a given width and current.
    '              This function computes maximum frequency based on the maximum power of
    '              the power supply and also from the LED cooling equations.  The lower
    '              value is returned.
    '
    '  Inputs:     Byte        port        Identifier for device
    '              Byte        ch          Channel to compute maximum frequency
    '              Integer     width       Pulse width in microseconds
    '              Single      current     Pulse current in amps
    '
    '  Outputs:    Integer     freq        Location to store calculated frequency
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '********************************************************************************
    Public Declare Function api_calc_max_freq Lib "PulsarAPI.dll" Alias "_api_calc_max_freq@20" ( _
       ByVal Port As Byte, _
       ByVal ch As Byte, _
       ByVal width As Integer, _
       ByVal current As Single, _
       ByRef freq As Integer) As Integer

    '*******************************************************************************
    '  api_measure_drive_current
    '
    '  About:      Measures the drive current of the selected drive channel.  This
    '              function can be used in either pulse or continuous modes.
    '
    '  Inputs:     Byte        port        Identifier for device
    '              Byte        ch          Channel to measure pulse current
    '
    '  Outputs:    Single      current     Location to store measured pulse current
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '********************************************************************************
    Public Declare Function api_measure_drive_current Lib "PulsarAPI.dll" Alias "_api_measure_drive_current@12" ( _
       ByVal Port As Byte, _
       ByVal ch As Byte, _
       ByRef current As Single) As Integer

    '*******************************************************************************
    '  api_measure_voltage
    '
    '  About:      Measures the voltage of one of the signals monitored by the Pulsar.
    '
    '  Inputs:     Byte        port        Identifier for device
    '              Byte        id          Identifier for desired voltage to measure
    '
    '  Outputs:    Single      voltage     Location to store measured voltage
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '********************************************************************************
    Public Declare Function api_measure_voltage Lib "PulsarAPI.dll" Alias "_api_measure_voltage@12" ( _
       ByVal Port As Byte, _
       ByVal ch As Byte, _
       ByRef voltage As Single) As Integer

    '*******************************************************************************
    '  api_get_user_data
    '
    '  About:      Gathers various information about the operating state of the Pulsar.
    '
    '  Inputs:     Byte        port        Identifier for device
    '
    '  Outputs:    Integer    rotary       Value on rotary switches
    '              Byte       dip          Value on DIP switches
    '              Byte       trigger      State of trigger inputs
    '              Byte       connect      Communication connection status
    '              Byte       supply       Power supply status
    '              Byte       expansion    Expansion status
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '********************************************************************************
    Public Declare Function api_get_user_data Lib "PulsarAPI.dll" Alias "_api_get_user_data@28" ( _
       ByVal Port As Byte, _
       ByRef Rotary As Integer, _
       ByRef Dip As Byte, _
       ByRef trigger As Byte, _
       ByRef Connect As Byte, _
       ByRef Supply As Byte, _
       ByRef Expansion As Byte) As Integer

    '*******************************************************************************
    '  api_set_stored_data
    '
    '  About:      Saves a block of Ai defined data to EEPROM.
    '
    '  Inputs:     Byte        port        Identifier for device
    '              Byte        data        Pointer to data block to be written
    '              Byte        len         Length of data block to be written
    '
    '  Outputs:    None
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '********************************************************************************
    Public Declare Function api_set_stored_data Lib "PulsarAPI.dll" Alias "_api_set_stored_data@12" ( _
       ByVal Port As Byte, _
       ByRef Data As Byte, _
       ByVal length As Byte) As Integer

    '*******************************************************************************
    '  api_get_stored_data
    '
    '  About:      Reads a block of Ai defined data from EEPROM.
    '
    '  Inputs:     Byte        port        Identifier for device
    '              Byte        len         Length of data block to read
    '
    '  Outputs:    Byte        data        Pointer to data block to be written
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '********************************************************************************
    Public Declare Function api_get_stored_data Lib "PulsarAPI.dll" Alias "_api_get_stored_data@12" ( _
       ByVal Port As Byte, _
       ByRef Data As Byte, _
       ByVal length As Byte) As Integer

    '****************************************************************************
    '  api_shutdown
    '
    '  About:      Shuts down the connection to a specific Pulsar
    '
    '  Inputs:     Byte       port        Communication port to use
    '
    '  Outputs:    None
    '
    '  Returns:    Integer     ecode       Status of command
    '                                         Zero if successful,
    '                                         Error code otherwise
    '
    '****************************************************************************
    Public Declare Function api_shutdown Lib "PulsarAPI.dll" Alias "_api_shutdown@4" (ByVal Port As Byte) As Integer


End Module